@@ -9,14 +9,16 @@
 #include "nsContentCID.h"
 #include "nsContentUtils.h"
 #include "nsDOMClassInfoID.h"
-#include "nsDOMError.h"
+#include "nsError.h"
 #include "nsICharsetDetector.h"
 #include "nsICharsetConverterManager.h"
+#include "nsIClassInfo.h"
 #include "nsIConverterInputStream.h"
 #include "nsIDocument.h"
 #include "nsIFileStreams.h"
 #include "nsIInputStream.h"
-#include "nsIIPCSerializable.h"
+#include "nsIIPCSerializableInputStream.h"
+#include "nsIMemoryReporter.h"
 #include "nsIMIMEService.h"
 #include "nsIPlatformCharset.h"
 #include "nsISeekableStream.h"
@@ -28,12 +30,15 @@
 #include "nsBlobProtocolHandler.h"
 #include "nsStringStream.h"
 #include "nsJSUtils.h"
+#include "nsPrintfCString.h"
+#include "mozilla/SHA1.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Attributes.h"
 
 #include "plbase64.h"
 #include "prmem.h"
+#include "mozilla/dom/FileListBinding.h"
 #include "dombindings.h"
 
 using namespace mozilla;
@@ -45,26 +50,32 @@ using namespace mozilla::dom;
 // from NS_NewByteInputStream is held alive as long as the
 // stream is.  We do that by passing back this class instead.
 class DataOwnerAdapter MOZ_FINAL : public nsIInputStream,
-                                   public nsISeekableStream
+                                   public nsISeekableStream,
+                                   public nsIIPCSerializableInputStream
 {
   typedef nsDOMMemoryFile::DataOwner DataOwner;
 public:
   static nsresult Create(DataOwner* aDataOwner,
-                         PRUint32 aStart,
-                         PRUint32 aLength,
+                         uint32_t aStart,
+                         uint32_t aLength,
                          nsIInputStream** _retval);
 
   NS_DECL_ISUPPORTS
 
+  // These are mandatory.
   NS_FORWARD_NSIINPUTSTREAM(mStream->)
-
   NS_FORWARD_NSISEEKABLESTREAM(mSeekableStream->)
 
+  // This is optional. We use a conditional QI to keep it from being called
+  // if the underlying stream doesn't support it.
+  NS_FORWARD_NSIIPCSERIALIZABLEINPUTSTREAM(mSerializableInputStream->)
+
 private:
   DataOwnerAdapter(DataOwner* aDataOwner,
                    nsIInputStream* aStream)
     : mDataOwner(aDataOwner), mStream(aStream),
-      mSeekableStream(do_QueryInterface(aStream))
+      mSeekableStream(do_QueryInterface(aStream)),
+      mSerializableInputStream(do_QueryInterface(aStream))
   {
     NS_ASSERTION(mSeekableStream, "Somebody gave us the wrong stream!");
   }
@@ -72,15 +83,23 @@ private:
   nsRefPtr<DataOwner> mDataOwner;
   nsCOMPtr<nsIInputStream> mStream;
   nsCOMPtr<nsISeekableStream> mSeekableStream;
+  nsCOMPtr<nsIIPCSerializableInputStream> mSerializableInputStream;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(DataOwnerAdapter,
-                              nsIInputStream,
-                              nsISeekableStream)
+NS_IMPL_THREADSAFE_ADDREF(DataOwnerAdapter)
+NS_IMPL_THREADSAFE_RELEASE(DataOwnerAdapter)
+
+NS_INTERFACE_MAP_BEGIN(DataOwnerAdapter)
+  NS_INTERFACE_MAP_ENTRY(nsIInputStream)
+  NS_INTERFACE_MAP_ENTRY(nsISeekableStream)
+  NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsIIPCSerializableInputStream,
+                                     mSerializableInputStream)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIInputStream)
+NS_INTERFACE_MAP_END
 
 nsresult DataOwnerAdapter::Create(DataOwner* aDataOwner,
-                                  PRUint32 aStart,
-                                  PRUint32 aLength,
+                                  uint32_t aStart,
+                                  uint32_t aLength,
                                   nsIInputStream** _retval)
 {
   nsresult rv;
@@ -91,7 +110,7 @@ nsresult DataOwnerAdapter::Create(DataOwner* aDataOwner,
   rv = NS_NewByteInputStream(getter_AddRefs(stream),
                              static_cast<const char*>(aDataOwner->mData) +
                              aStart,
-                             (PRInt32)aLength,
+                             (int32_t)aLength,
                              NS_ASSIGNMENT_DEPEND);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -112,8 +131,10 @@ nsDOMFileBase::GetName(nsAString &aFileName)
 }
 
 NS_IMETHODIMP
-nsDOMFileBase::GetLastModifiedDate(JSContext* cx, JS::Value *aLastModifiedDate)
+nsDOMFileBase::GetLastModifiedDate(JSContext* cx, JS::Value* aLastModifiedDate)
 {
+  NS_ASSERTION(mIsFile, "Should only be called on files");
+
   aLastModifiedDate->setNull();
   return NS_OK;
 }
@@ -144,7 +165,7 @@ nsDOMFileBase::GetMozFullPathInternal(nsAString &aFileName)
 }
 
 NS_IMETHODIMP
-nsDOMFileBase::GetSize(PRUint64 *aSize)
+nsDOMFileBase::GetSize(uint64_t *aSize)
 {
   *aSize = mLength;
   return NS_OK;
@@ -157,10 +178,18 @@ nsDOMFileBase::GetType(nsAString &aType)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMFileBase::GetMozLastModifiedDate(uint64_t* aLastModifiedDate)
+{
+  NS_ASSERTION(mIsFile, "Should only be called on files");
+  *aLastModifiedDate = mLastModificationDate;
+  return NS_OK;
+}
+
 // Makes sure that aStart and aEnd is less then or equal to aSize and greater
 // than 0
 static void
-ParseSize(PRInt64 aSize, PRInt64& aStart, PRInt64& aEnd)
+ParseSize(int64_t aSize, int64_t& aStart, int64_t& aEnd)
 {
   CheckedInt64 newStartOffset = aStart;
   if (aStart < -aSize) {
@@ -195,35 +224,35 @@ ParseSize(PRInt64 aSize, PRInt64& aStart, PRInt64& aEnd)
 }
 
 NS_IMETHODIMP
-nsDOMFileBase::Slice(PRInt64 aStart, PRInt64 aEnd,
-                     const nsAString& aContentType, PRUint8 optional_argc,
+nsDOMFileBase::Slice(int64_t aStart, int64_t aEnd,
+                     const nsAString& aContentType, uint8_t optional_argc,
                      nsIDOMBlob **aBlob)
 {
-  *aBlob = nsnull;
+  *aBlob = nullptr;
 
   // Truncate aStart and aEnd so that we stay within this file.
-  PRUint64 thisLength;
+  uint64_t thisLength;
   nsresult rv = GetSize(&thisLength);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (optional_argc < 2) {
-    aEnd = (PRInt64)thisLength;
+    aEnd = (int64_t)thisLength;
   }
 
-  ParseSize((PRInt64)thisLength, aStart, aEnd);
+  ParseSize((int64_t)thisLength, aStart, aEnd);
   
   // Create the new file
-  *aBlob = CreateSlice((PRUint64)aStart, (PRUint64)(aEnd - aStart),
+  *aBlob = CreateSlice((uint64_t)aStart, (uint64_t)(aEnd - aStart),
                        aContentType).get();
 
   return *aBlob ? NS_OK : NS_ERROR_UNEXPECTED;
 }
 
 NS_IMETHODIMP
-nsDOMFileBase::MozSlice(PRInt64 aStart, PRInt64 aEnd,
+nsDOMFileBase::MozSlice(int64_t aStart, int64_t aEnd,
                         const nsAString& aContentType, 
                         JSContext* aCx,
-                        PRUint8 optional_argc,
+                        uint8_t optional_argc,
                         nsIDOMBlob **aBlob)
 {
   MOZ_ASSERT(NS_IsMainThread());
@@ -280,10 +309,10 @@ nsDOMFileBase::GetInternalUrl(nsIPrincipal* aPrincipal, nsAString& aURL)
   return NS_OK;
 }
 
-NS_IMETHODIMP_(PRInt64)
+NS_IMETHODIMP_(int64_t)
 nsDOMFileBase::GetFileId()
 {
-  PRInt64 id = -1;
+  int64_t id = -1;
 
   if (IsStoredFile() && IsWholeFile() && !IsSnapshot()) {
     if (!indexedDB::IndexedDatabaseManager::IsClosed()) {
@@ -330,14 +359,14 @@ nsDOMFileBase::GetFileInfo(indexedDB::FileManager* aFileManager)
 {
   if (indexedDB::IndexedDatabaseManager::IsClosed()) {
     NS_ERROR("Shouldn't be called after shutdown!");
-    return nsnull;
+    return nullptr;
   }
 
   // A slice created from a stored file must keep the file info alive.
   // However, we don't support sharing of slices yet, so the slice must be
   // copied again. That's why we have to ignore the first file info.
   // Snapshots are handled in a similar way (they have to be copied).
-  PRUint32 startIndex;
+  uint32_t startIndex;
   if (IsStoredFile() && (!IsWholeFile() || IsSnapshot())) {
     startIndex = 1;
   }
@@ -347,18 +376,19 @@ nsDOMFileBase::GetFileInfo(indexedDB::FileManager* aFileManager)
 
   MutexAutoLock lock(indexedDB::IndexedDatabaseManager::FileMutex());
 
-  for (PRUint32 i = startIndex; i < mFileInfos.Length(); i++) {
+  for (uint32_t i = startIndex; i < mFileInfos.Length(); i++) {
     nsRefPtr<indexedDB::FileInfo>& fileInfo = mFileInfos.ElementAt(i);
     if (fileInfo->Manager() == aFileManager) {
       return fileInfo;
     }
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 NS_IMETHODIMP
 nsDOMFileBase::GetSendInfo(nsIInputStream** aBody,
+                           uint64_t* aContentLength,
                            nsACString& aContentType,
                            nsACString& aCharset)
 {
@@ -368,6 +398,9 @@ nsDOMFileBase::GetSendInfo(nsIInputStream** aBody,
   rv = this->GetInternalStream(getter_AddRefs(stream));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  rv = this->GetSize(aContentLength);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   nsString contentType;
   rv = this->GetType(contentType);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -400,7 +433,7 @@ nsDOMFileBase::SetMutable(bool aMutable)
     rv = this->GetType(dummyString);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    PRUint64 dummyInt;
+    uint64_t dummyInt;
     rv = this->GetSize(&dummyInt);
     NS_ENSURE_SUCCESS(rv, rv);
   }
@@ -454,7 +487,7 @@ NS_IMPL_ISUPPORTS_INHERITED1(nsDOMFileFile, nsDOMFile,
                              nsIJSNativeInitializer)
 
 already_AddRefed<nsIDOMBlob>
-nsDOMFileFile::CreateSlice(PRUint64 aStart, PRUint64 aLength,
+nsDOMFileFile::CreateSlice(uint64_t aStart, uint64_t aLength,
                            const nsAString& aContentType)
 {
   nsCOMPtr<nsIDOMBlob> t = new nsDOMFileFile(this, aStart, aLength, aContentType);
@@ -479,8 +512,16 @@ nsDOMFileFile::GetMozFullPathInternal(nsAString &aFilename)
 NS_IMETHODIMP
 nsDOMFileFile::GetLastModifiedDate(JSContext* cx, JS::Value *aLastModifiedDate)
 {
-  PRInt64 msecs;
+  PRTime msecs;
   mFile->GetLastModifiedTime(&msecs);
+  if (IsDateUnknown()) {
+    nsresult rv = mFile->GetLastModifiedTime(&msecs);
+    NS_ENSURE_SUCCESS(rv, rv);
+    mLastModificationDate = msecs;
+  } else {
+    msecs = mLastModificationDate;
+  }
+
   JSObject* date = JS_NewDateObjectMsec(cx, msecs);
   if (date) {
     aLastModifiedDate->setObject(*date);
@@ -493,12 +534,12 @@ nsDOMFileFile::GetLastModifiedDate(JSContext* cx, JS::Value *aLastModifiedDate)
 }
 
 NS_IMETHODIMP
-nsDOMFileFile::GetSize(PRUint64 *aFileSize)
+nsDOMFileFile::GetSize(uint64_t *aFileSize)
 {
   if (IsSizeUnknown()) {
     NS_ASSERTION(mWholeFile,
                  "Should only use lazy size when using the whole file");
-    PRInt64 fileSize;
+    int64_t fileSize;
     nsresult rv = mFile->GetFileSize(&fileSize);
     NS_ENSURE_SUCCESS(rv, rv);
   
@@ -525,7 +566,7 @@ nsDOMFileFile::GetType(nsAString &aType)
       do_GetService(NS_MIMESERVICE_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCAutoString mimeType;
+    nsAutoCString mimeType;
     rv = mimeService->GetTypeFromFile(mFile, mimeType);
     if (NS_FAILED(rv)) {
       mimeType.Truncate();
@@ -540,7 +581,15 @@ nsDOMFileFile::GetType(nsAString &aType)
   return NS_OK;
 }
 
-const PRUint32 sFileStreamFlags =
+NS_IMETHODIMP
+nsDOMFileFile::GetMozLastModifiedDate(uint64_t* aLastModifiedDate)
+{
+  NS_ASSERTION(mIsFile, "Should only be called on files");
+  *aLastModifiedDate = mLastModificationDate;
+  return NS_OK;
+}
+
+const uint32_t sFileStreamFlags =
   nsIFileInputStream::CLOSE_ON_EOF |
   nsIFileInputStream::REOPEN_ON_REWIND |
   nsIFileInputStream::DEFER_OPEN;
@@ -558,7 +607,7 @@ NS_IMETHODIMP
 nsDOMFileFile::Initialize(nsISupports* aOwner,
                           JSContext* aCx,
                           JSObject* aObj,
-                          PRUint32 aArgc,
+                          uint32_t aArgc,
                           JS::Value* aArgv)
 {
   nsresult rv;
@@ -623,7 +672,7 @@ nsDOMFileFile::Initialize(nsISupports* aOwner,
 // nsDOMMemoryFile implementation
 
 already_AddRefed<nsIDOMBlob>
-nsDOMMemoryFile::CreateSlice(PRUint64 aStart, PRUint64 aLength,
+nsDOMMemoryFile::CreateSlice(uint64_t aStart, uint64_t aLength,
                              const nsAString& aContentType)
 {
   nsCOMPtr<nsIDOMBlob> t =
@@ -634,12 +683,127 @@ nsDOMMemoryFile::CreateSlice(PRUint64 aStart, PRUint64 aLength,
 NS_IMETHODIMP
 nsDOMMemoryFile::GetInternalStream(nsIInputStream **aStream)
 {
-  if (mLength > PR_INT32_MAX)
+  if (mLength > INT32_MAX)
     return NS_ERROR_FAILURE;
 
   return DataOwnerAdapter::Create(mDataOwner, mStart, mLength, aStream);
 }
 
+/* static */ StaticAutoPtr<LinkedList<nsDOMMemoryFile::DataOwner> >
+nsDOMMemoryFile::DataOwner::sDataOwners;
+
+/* static */ bool
+nsDOMMemoryFile::DataOwner::sMemoryReporterRegistered;
+
+NS_MEMORY_REPORTER_MALLOC_SIZEOF_FUN(DOMMemoryFileDataOwnerSizeOf,
+                                     "memory-file-data");
+
+class nsDOMMemoryFileDataOwnerMemoryReporter
+  : public nsIMemoryMultiReporter
+{
+  NS_DECL_ISUPPORTS
+
+  NS_IMETHOD GetName(nsACString& aName)
+  {
+    aName.AssignASCII("dom-memory-file-data-owner");
+    return NS_OK;
+  }
+
+  NS_IMETHOD GetExplicitNonHeap(int64_t *aResult)
+  {
+    // All of this reporter's memory is on the heap.
+    *aResult = 0;
+    return NS_OK;
+  }
+
+  NS_IMETHOD CollectReports(nsIMemoryMultiReporterCallback *aCallback,
+                            nsISupports *aClosure)
+  {
+    typedef nsDOMMemoryFile::DataOwner DataOwner;
+
+    if (!DataOwner::sDataOwners) {
+      return NS_OK;
+    }
+
+    const size_t LARGE_OBJECT_MIN_SIZE = 8 * 1024;
+    size_t smallObjectsTotal = 0;
+
+    for (DataOwner *owner = DataOwner::sDataOwners->getFirst();
+         owner; owner = owner->getNext()) {
+
+      size_t size = DOMMemoryFileDataOwnerSizeOf(owner->mData);
+
+      if (size < LARGE_OBJECT_MIN_SIZE) {
+        smallObjectsTotal += size;
+      }
+      else {
+        SHA1Sum sha1;
+        sha1.update(owner->mData, owner->mLength);
+        uint8_t digest[SHA1Sum::HashSize]; // SHA1 digests are 20 bytes long.
+        sha1.finish(digest);
+
+        nsAutoCString digestString;
+        for (size_t i = 0; i < sizeof(digest); i++) {
+          digestString.AppendPrintf("%02x", digest[i]);
+        }
+
+        nsresult rv = aCallback->Callback(
+          /* process */ NS_LITERAL_CSTRING(""),
+          nsPrintfCString(
+            "explicit/dom/memory-file-data/large/file(length=%llu, sha1=%s)",
+            owner->mLength, digestString.get()),
+          nsIMemoryReporter::KIND_HEAP,
+          nsIMemoryReporter::UNITS_BYTES,
+          size,
+          nsPrintfCString(
+            "Memory used to back a memory file of length %llu bytes.  The file "
+            "has a sha1 of %s.\n\n"
+            "Note that the allocator may round up a memory file's length -- "
+            "that is, an N-byte memory file may take up more than N bytes of "
+            "memory.",
+            owner->mLength, digestString.get()),
+          aClosure);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+    }
+
+    if (smallObjectsTotal > 0) {
+      nsresult rv = aCallback->Callback(
+        /* process */ NS_LITERAL_CSTRING(""),
+        NS_LITERAL_CSTRING("explicit/dom/memory-file-data/small"),
+        nsIMemoryReporter::KIND_HEAP,
+        nsIMemoryReporter::UNITS_BYTES,
+        smallObjectsTotal,
+        nsPrintfCString(
+          "Memory used to back small memory files (less than %d bytes each).\n\n"
+          "Note that the allocator may round up a memory file's length -- "
+          "that is, an N-byte memory file may take up more than N bytes of "
+          "memory."),
+        aClosure);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
+
+    return NS_OK;
+  }
+};
+
+NS_IMPL_ISUPPORTS1(nsDOMMemoryFileDataOwnerMemoryReporter,
+                   nsIMemoryMultiReporter);
+
+/* static */ void
+nsDOMMemoryFile::DataOwner::EnsureMemoryReporterRegistered()
+{
+  if (sMemoryReporterRegistered) {
+    return;
+  }
+
+  nsRefPtr<nsDOMMemoryFileDataOwnerMemoryReporter> reporter = new
+    nsDOMMemoryFileDataOwnerMemoryReporter();
+  NS_RegisterMemoryMultiReporter(reporter);
+
+  sMemoryReporterRegistered = true;
+}
+
 ////////////////////////////////////////////////////////////////////////////
 // nsDOMFileList implementation
 
@@ -661,27 +825,33 @@ JSObject*
 nsDOMFileList::WrapObject(JSContext *cx, JSObject *scope,
                           bool *triedToWrap)
 {
-  return mozilla::dom::binding::FileList::create(cx, scope, this, triedToWrap);
+  JSObject* obj = FileListBinding::Wrap(cx, scope, this, triedToWrap);
+  if (obj || *triedToWrap) {
+    return obj;
+  }
+
+  *triedToWrap = true;
+  return oldproxybindings::FileList::create(cx, scope, this);
 }
 
 nsIDOMFile*
-nsDOMFileList::GetItemAt(PRUint32 aIndex)
+nsDOMFileList::GetItemAt(uint32_t aIndex)
 {
-  return mFiles.SafeObjectAt(aIndex);
+  return Item(aIndex);
 }
 
 NS_IMETHODIMP
-nsDOMFileList::GetLength(PRUint32* aLength)
+nsDOMFileList::GetLength(uint32_t* aLength)
 {
-  *aLength = mFiles.Count();
+  *aLength = Length();
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsDOMFileList::Item(PRUint32 aIndex, nsIDOMFile **aFile)
+nsDOMFileList::Item(uint32_t aIndex, nsIDOMFile **aFile)
 {
-  NS_IF_ADDREF(*aFile = nsDOMFileList::GetItemAt(aIndex));
+  NS_IF_ADDREF(*aFile = Item(aIndex));
 
   return NS_OK;
 }
@@ -698,7 +868,7 @@ nsDOMFileInternalUrlHolder::nsDOMFileInternalUrlHolder(nsIDOMBlob* aFile,
  
 nsDOMFileInternalUrlHolder::~nsDOMFileInternalUrlHolder() {
   if (!mUrl.IsEmpty()) {
-    nsCAutoString narrowUrl;
+    nsAutoCString narrowUrl;
     CopyUTF16toUTF8(mUrl, narrowUrl);
     nsBlobProtocolHandler::RemoveFileDataEntry(narrowUrl);
   }
