@@ -105,7 +105,7 @@ nsPluginByteRangeStreamListener::OnStartRequest(nsIRequest *request, nsISupports
     rv = serv->AsyncConvertData(MULTIPART_BYTERANGES,
                                 "*/*",
                                 finalStreamListener,
-                                nsnull,
+                                nullptr,
                                 getter_AddRefs(mStreamConverter));
     if (NS_SUCCEEDED(rv)) {
       rv = mStreamConverter->OnStartRequest(request, ctxt);
@@ -120,7 +120,7 @@ nsPluginByteRangeStreamListener::OnStartRequest(nsIRequest *request, nsISupports
     return NS_ERROR_FAILURE;
   }
   
-  PRUint32 responseCode = 0;
+  uint32_t responseCode = 0;
   rv = httpChannel->GetResponseStatus(&responseCode);
   if (NS_FAILED(rv)) {
     return NS_ERROR_FAILURE;
@@ -171,7 +171,7 @@ nsPluginByteRangeStreamListener::OnStopRequest(nsIRequest *request, nsISupports
     // remove magic number from container
     nsCOMPtr<nsISupportsPRUint32> container = do_QueryInterface(ctxt);
     if (container) {
-      PRUint32 magicNumber = 0;
+      uint32_t magicNumber = 0;
       container->GetData(&magicNumber);
       if (magicNumber == MAGIC_REQUEST_CONTEXT) {
         // to allow properly finish nsPluginStreamListenerPeer->OnStopRequest()
@@ -218,7 +218,9 @@ CachedFileHolder::Release()
 
 NS_IMETHODIMP
 nsPluginByteRangeStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
-                                                 nsIInputStream *inStr, PRUint32 sourceOffset, PRUint32 count)
+                                                 nsIInputStream *inStr,
+                                                 uint64_t sourceOffset,
+                                                 uint32_t count)
 {
   if (!mStreamConverter)
     return NS_ERROR_FAILURE;
@@ -246,11 +248,11 @@ nsPluginByteRangeStreamListener::GetInterface(const nsIID& aIID, void** result)
 
 class nsPRUintKey : public nsHashKey {
 protected:
-  PRUint32 mKey;
+  uint32_t mKey;
 public:
-  nsPRUintKey(PRUint32 key) : mKey(key) {}
+  nsPRUintKey(uint32_t key) : mKey(key) {}
   
-  PRUint32 HashCode() const {
+  uint32_t HashCode() const {
     return mKey;
   }
   
@@ -260,7 +262,7 @@ public:
   nsHashKey *Clone() const {
     return new nsPRUintKey(mKey);
   }
-  PRUint32 GetValue() { return mKey; }
+  uint32_t GetValue() { return mKey; }
 };
 
 // nsPluginStreamListenerPeer
@@ -282,7 +284,7 @@ nsPluginStreamListenerPeer::nsPluginStreamListenerPeer()
   
   mPendingRequests = 0;
   mHaveFiredOnStartRequest = false;
-  mDataForwardToRequest = nsnull;
+  mDataForwardToRequest = nullptr;
   
   mSeekable = false;
   mModified = 0;
@@ -298,13 +300,13 @@ nsPluginStreamListenerPeer::~nsPluginStreamListenerPeer()
 #endif
 
   if (mPStreamListener) {
-    mPStreamListener->SetStreamListenerPeer(nsnull);
+    mPStreamListener->SetStreamListenerPeer(nullptr);
   }
 
   // close FD of mFileCacheOutputStream if it's still open
   // or we won't be able to remove the cache file
   if (mFileCacheOutputStream)
-    mFileCacheOutputStream = nsnull;
+    mFileCacheOutputStream = nullptr;
   
   delete mDataForwardToRequest;
 
@@ -318,8 +320,8 @@ nsresult nsPluginStreamListenerPeer::Initialize(nsIURI *aURL,
                                                 nsNPAPIPluginStreamListener* aListener)
 {
 #ifdef PLUGIN_LOGGING
-  nsCAutoString urlSpec;
-  if (aURL != nsnull) aURL->GetAsciiSpec(urlSpec);
+  nsAutoCString urlSpec;
+  if (aURL != nullptr) aURL->GetAsciiSpec(urlSpec);
   
   PR_LOG(nsPluginLogging::gPluginLog, PLUGIN_LOG_NORMAL,
          ("nsPluginStreamListenerPeer::Initialize instance=%p, url=%s\n", aInstance, urlSpec.get()));
@@ -348,7 +350,7 @@ nsresult nsPluginStreamListenerPeer::InitializeEmbedded(nsIURI *aURL,
                                                         nsObjectLoadingContent *aContent)
 {
 #ifdef PLUGIN_LOGGING
-  nsCAutoString urlSpec;
+  nsAutoCString urlSpec;
   aURL->GetSpec(urlSpec);
   
   PR_LOG(nsPluginLogging::gPluginLog, PLUGIN_LOG_NORMAL,
@@ -365,7 +367,7 @@ nsresult nsPluginStreamListenerPeer::InitializeEmbedded(nsIURI *aURL,
   mURL = aURL;
   
   if (aInstance) {
-    NS_ASSERTION(mPluginInstance == nsnull, "nsPluginStreamListenerPeer::InitializeEmbedded mPluginInstance != nsnull");
+    NS_ASSERTION(mPluginInstance == nullptr, "nsPluginStreamListenerPeer::InitializeEmbedded mPluginInstance != nullptr");
     mPluginInstance = aInstance;
   } else {
     mContent = aContent;
@@ -386,7 +388,7 @@ nsresult nsPluginStreamListenerPeer::InitializeFullPage(nsIURI* aURL, nsNPAPIPlu
   PLUGIN_LOG(PLUGIN_LOG_NORMAL,
              ("nsPluginStreamListenerPeer::InitializeFullPage instance=%p\n",aInstance));
   
-  NS_ASSERTION(mPluginInstance == nsnull, "nsPluginStreamListenerPeer::InitializeFullPage mPluginInstance != nsnull");
+  NS_ASSERTION(mPluginInstance == nullptr, "nsPluginStreamListenerPeer::InitializeFullPage mPluginInstance != nullptr");
   mPluginInstance = aInstance;
   
   mURL = aURL;
@@ -417,10 +419,10 @@ nsPluginStreamListenerPeer::SetupPluginCacheFile(nsIChannel* channel)
 
   // Look for an existing cache file for the URI.
   nsTArray< nsRefPtr<nsNPAPIPluginInstance> > *instances = pluginHost->InstanceArray();
-  for (PRUint32 i = 0; i < instances->Length(); i++) {
+  for (uint32_t i = 0; i < instances->Length(); i++) {
     // most recent streams are at the end of list
     nsTArray<nsPluginStreamListenerPeer*> *streamListeners = instances->ElementAt(i)->FileCachedStreamListeners();
-    for (PRInt32 i = streamListeners->Length() - 1; i >= 0; --i) {
+    for (int32_t i = streamListeners->Length() - 1; i >= 0; --i) {
       nsPluginStreamListenerPeer *lp = streamListeners->ElementAt(i);
       if (lp && lp->mLocalCachedFileHolder) {
         useExistingCacheFile = lp->UseExistingPluginCacheFile(this);
@@ -451,7 +453,7 @@ nsPluginStreamListenerPeer::SetupPluginCacheFile(nsIChannel* channel)
     if (!url)
       return NS_ERROR_FAILURE;
     
-    nsCAutoString filename;
+    nsAutoCString filename;
     url->GetFileName(filename);
     if (NS_FAILED(rv))
       return rv;
@@ -509,7 +511,7 @@ nsPluginStreamListenerPeer::OnStartRequest(nsIRequest *request,
   // just return, this causes the request to be ignored.
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
   if (httpChannel) {
-    PRUint32 responseCode = 0;
+    uint32_t responseCode = 0;
     rv = httpChannel->GetResponseStatus(&responseCode);
     if (NS_FAILED(rv)) {
       // NPP_Notify() will be called from OnStopRequest
@@ -554,7 +556,7 @@ nsPluginStreamListenerPeer::OnStartRequest(nsIRequest *request,
   if (loadGroup)
     mWeakPtrChannelLoadGroup = do_GetWeakReference(loadGroup);
   
-  PRInt32 length;
+  int32_t length;
   rv = channel->GetContentLength(&length);
   
   // it's possible for the server to not send a Content-Length.
@@ -573,7 +575,7 @@ nsPluginStreamListenerPeer::OnStartRequest(nsIRequest *request,
     mLength = length;
   }
   
-  nsCAutoString aContentType; // XXX but we already got the type above!
+  nsAutoCString aContentType; // XXX but we already got the type above!
   rv = channel->GetContentType(aContentType);
   if (NS_FAILED(rv))
     return rv;
@@ -601,7 +603,7 @@ nsPluginStreamListenerPeer::OnStartRequest(nsIRequest *request,
   // if we have a mime type now.
   if (!mPluginInstance && mContent && !aContentType.IsEmpty()) {
     nsObjectLoadingContent *olc = static_cast<nsObjectLoadingContent*>(mContent.get());
-    rv = olc->InstantiatePluginInstance(aContentType.get(), aURL.get());
+    rv = olc->InstantiatePluginInstance();
     if (NS_SUCCEEDED(rv)) {
       rv = olc->GetPluginInstance(getter_AddRefs(mPluginInstance));
       if (NS_FAILED(rv)) {
@@ -619,8 +621,8 @@ nsPluginStreamListenerPeer::OnStartRequest(nsIRequest *request,
 
 NS_IMETHODIMP nsPluginStreamListenerPeer::OnProgress(nsIRequest *request,
                                                      nsISupports* aContext,
-                                                     PRUint64 aProgress,
-                                                     PRUint64 aProgressMax)
+                                                     uint64_t aProgress,
+                                                     uint64_t aProgressMax)
 {
   nsresult rv = NS_OK;
   return rv;
@@ -650,14 +652,14 @@ nsPluginStreamListenerPeer::IsSeekable(bool* result)
 }
 
 nsresult
-nsPluginStreamListenerPeer::GetLength(PRUint32* result)
+nsPluginStreamListenerPeer::GetLength(uint32_t* result)
 {
   *result = mLength;
   return NS_OK;
 }
 
 nsresult
-nsPluginStreamListenerPeer::GetLastModified(PRUint32* result)
+nsPluginStreamListenerPeer::GetLastModified(uint32_t* result)
 {
   *result = mModified;
   return NS_OK;
@@ -672,7 +674,7 @@ nsPluginStreamListenerPeer::GetURL(const char** result)
 
 void
 nsPluginStreamListenerPeer::MakeByteRangeString(NPByteRange* aRangeList, nsACString &rangeRequest,
-                                                PRInt32 *numRequests)
+                                                int32_t *numRequests)
 {
   rangeRequest.Truncate();
   *numRequests  = 0;
@@ -680,10 +682,10 @@ nsPluginStreamListenerPeer::MakeByteRangeString(NPByteRange* aRangeList, nsACStr
   if (!aRangeList)
     return;
   
-  PRInt32 requestCnt = 0;
-  nsCAutoString string("bytes=");
+  int32_t requestCnt = 0;
+  nsAutoCString string("bytes=");
   
-  for (NPByteRange * range = aRangeList; range != nsnull; range = range->next) {
+  for (NPByteRange * range = aRangeList; range != nullptr; range = range->next) {
     // XXX zero length?
     if (!range->length)
       continue;
@@ -709,8 +711,8 @@ nsPluginStreamListenerPeer::MakeByteRangeString(NPByteRange* aRangeList, nsACStr
 nsresult
 nsPluginStreamListenerPeer::RequestRead(NPByteRange* rangeList)
 {
-  nsCAutoString rangeString;
-  PRInt32 numRequests;
+  nsAutoCString rangeString;
+  int32_t numRequests;
   
   MakeByteRangeString(rangeList, rangeString, &numRequests);
   
@@ -722,7 +724,7 @@ nsPluginStreamListenerPeer::RequestRead(NPByteRange* rangeList)
   nsCOMPtr<nsIInterfaceRequestor> callbacks = do_QueryReferent(mWeakPtrChannelCallbacks);
   nsCOMPtr<nsILoadGroup> loadGroup = do_QueryReferent(mWeakPtrChannelLoadGroup);
   nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewChannel(getter_AddRefs(channel), mURL, nsnull, loadGroup, callbacks);
+  rv = NS_NewChannel(getter_AddRefs(channel), mURL, nullptr, loadGroup, callbacks);
   if (NS_FAILED(rv))
     return rv;
   
@@ -770,14 +772,14 @@ nsPluginStreamListenerPeer::RequestRead(NPByteRange* rangeList)
 }
 
 nsresult
-nsPluginStreamListenerPeer::GetStreamOffset(PRInt32* result)
+nsPluginStreamListenerPeer::GetStreamOffset(int32_t* result)
 {
   *result = mStreamOffset;
   return NS_OK;
 }
 
 nsresult
-nsPluginStreamListenerPeer::SetStreamOffset(PRInt32 value)
+nsPluginStreamListenerPeer::SetStreamOffset(int32_t value)
 {
   mStreamOffset = value;
   return NS_OK;
@@ -792,10 +794,9 @@ nsresult nsPluginStreamListenerPeer::ServeStreamAsFile(nsIRequest *request,
   // mPluginInstance->Stop calls mPStreamListener->CleanUpStream(), so stream will be properly clean up
   mPluginInstance->Stop();
   mPluginInstance->Start();
-  nsCOMPtr<nsIPluginInstanceOwner> owner;
-  mPluginInstance->GetOwner(getter_AddRefs(owner));
+  nsRefPtr<nsPluginInstanceOwner> owner = mPluginInstance->GetOwner();
   if (owner) {
-    NPWindow* window = nsnull;
+    NPWindow* window = nullptr;
     owner->GetWindow(window);
 #if defined(MOZ_WIDGET_GTK2) || defined(MOZ_WIDGET_QT)
     // Should call GetPluginPort() here.
@@ -849,8 +850,8 @@ nsPluginStreamListenerPeer::UseExistingPluginCacheFile(nsPluginStreamListenerPee
 NS_IMETHODIMP nsPluginStreamListenerPeer::OnDataAvailable(nsIRequest *request,
                                                           nsISupports* aContext,
                                                           nsIInputStream *aIStream,
-                                                          PRUint32 sourceOffset,
-                                                          PRUint32 aLength)
+                                                          uint64_t sourceOffset,
+                                                          uint32_t aLength)
 {
   NS_ASSERTION(mRequests.IndexOfObject(GetBaseRequest(request)) != -1,
                "Received OnDataAvailable for untracked request.");
@@ -859,7 +860,7 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnDataAvailable(nsIRequest *request,
     return NS_ERROR_FAILURE;
   
   if (mAbort) {
-    PRUint32 magicNumber = 0;  // set it to something that is not the magic number.
+    uint32_t magicNumber = 0;  // set it to something that is not the magic number.
     nsCOMPtr<nsISupportsPRUint32> container = do_QueryInterface(aContext);
     if (container)
       container->GetData(&magicNumber);
@@ -876,11 +877,11 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnDataAvailable(nsIRequest *request,
   if (!mPStreamListener)
     return NS_ERROR_FAILURE;
   
-  const char * url = nsnull;
+  const char * url = nullptr;
   GetURL(&url);
   
   PLUGIN_LOG(PLUGIN_LOG_NOISY,
-             ("nsPluginStreamListenerPeer::OnDataAvailable this=%p request=%p, offset=%d, length=%d, url=%s\n",
+             ("nsPluginStreamListenerPeer::OnDataAvailable this=%p request=%p, offset=%llu, length=%u, url=%s\n",
               this, request, sourceOffset, aLength, url ? url : "no url set"));
   
   // if the plugin has requested an AsFileOnly stream, then don't
@@ -892,11 +893,11 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnDataAvailable(nsIRequest *request,
       if (!mDataForwardToRequest)
         return NS_ERROR_FAILURE;
       
-      PRInt64 absoluteOffset64 = LL_ZERO;
+      int64_t absoluteOffset64 = LL_ZERO;
       brr->GetStartRange(&absoluteOffset64);
       
       // XXX handle 64-bit for real
-      PRInt32 absoluteOffset = (PRInt32)PRInt64(absoluteOffset64);
+      int32_t absoluteOffset = (int32_t)int64_t(absoluteOffset64);
       
       // we need to track how much data we have forwarded to the
       // plugin.
@@ -906,7 +907,7 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnDataAvailable(nsIRequest *request,
       // Why couldn't this be tracked on the plugin info, and not in a
       // *hash table*?
       nsPRUintKey key(absoluteOffset);
-      PRInt32 amtForwardToPlugin =
+      int32_t amtForwardToPlugin =
       NS_PTR_TO_INT32(mDataForwardToRequest->Get(&key));
       mDataForwardToRequest->Put(&key, NS_INT32_TO_PTR(amtForwardToPlugin + aLength));
       
@@ -938,7 +939,7 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnDataAvailable(nsIRequest *request,
   {
     // if we don't read from the stream, OnStopRequest will never be called
     char* buffer = new char[aLength];
-    PRUint32 amountRead, amountWrote = 0;
+    uint32_t amountRead, amountWrote = 0;
     rv = aIStream->Read(buffer, aLength, &amountRead);
     
     // if we are caching this to disk ourselves, lets write the bytes out.
@@ -973,10 +974,10 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnStopRequest(nsIRequest *request,
   // for ByteRangeRequest we're just updating the mDataForwardToRequest hash and return.
   nsCOMPtr<nsIByteRangeRequest> brr = do_QueryInterface(request);
   if (brr) {
-    PRInt64 absoluteOffset64 = LL_ZERO;
+    int64_t absoluteOffset64 = LL_ZERO;
     brr->GetStartRange(&absoluteOffset64);
     // XXX support 64-bit offsets
-    PRInt32 absoluteOffset = (PRInt32)PRInt64(absoluteOffset64);
+    int32_t absoluteOffset = (int32_t)int64_t(absoluteOffset64);
     
     nsPRUintKey key(absoluteOffset);
     
@@ -991,7 +992,7 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnStopRequest(nsIRequest *request,
     // if this is not byte range request and
     // if we are writting the stream to disk ourselves,
     // close & tear it down here
-    mFileCacheOutputStream = nsnull;
+    mFileCacheOutputStream = nullptr;
   }
   
   // if we still have pending stuff to do, lets not close the plugin socket.
@@ -1001,7 +1002,7 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnStopRequest(nsIRequest *request,
   // we keep our connections around...
   nsCOMPtr<nsISupportsPRUint32> container = do_QueryInterface(aContext);
   if (container) {
-    PRUint32 magicNumber = 0;  // set it to something that is not the magic number.
+    uint32_t magicNumber = 0;  // set it to something that is not the magic number.
     container->GetData(&magicNumber);
     if (magicNumber == MAGIC_REQUEST_CONTEXT) {
       // this is one of our range requests
@@ -1016,7 +1017,7 @@ NS_IMETHODIMP nsPluginStreamListenerPeer::OnStopRequest(nsIRequest *request,
   if (!channel)
     return NS_ERROR_FAILURE;
   // Set the content type to ensure we don't pass null to the plugin
-  nsCAutoString aContentType;
+  nsAutoCString aContentType;
   rv = channel->GetContentType(aContentType);
   if (NS_FAILED(rv) && !mRequestFailed)
     return rv;
@@ -1090,7 +1091,7 @@ nsresult nsPluginStreamListenerPeer::SetUpStreamListener(nsIRequest *request,
     }
 
     nsRefPtr<nsNPAPIPluginStreamListener> streamListener;
-    rv = mPluginInstance->NewStreamListener(nsnull, nsnull,
+    rv = mPluginInstance->NewStreamListener(nullptr, nullptr,
                                             getter_AddRefs(streamListener));
     if (NS_FAILED(rv) || !streamListener) {
       return NS_ERROR_FAILURE;
@@ -1117,7 +1118,7 @@ nsresult nsPluginStreamListenerPeer::SetUpStreamListener(nsIRequest *request,
     // listener.  Would be nice if we could get the raw status line,
     // but nsIHttpChannel doesn't currently provide that.
     // Status code: required; the status line isn't useful without it.
-    PRUint32 statusNum;
+    uint32_t statusNum;
     if (NS_SUCCEEDED(httpChannel->GetResponseStatus(&statusNum)) &&
         statusNum < 1000) {
       // HTTP version: provide if available.  Defaults to empty string.
@@ -1125,7 +1126,7 @@ nsresult nsPluginStreamListenerPeer::SetUpStreamListener(nsIRequest *request,
       nsCOMPtr<nsIHttpChannelInternal> httpChannelInternal =
       do_QueryInterface(channel);
       if (httpChannelInternal) {
-        PRUint32 major, minor;
+        uint32_t major, minor;
         if (NS_SUCCEEDED(httpChannelInternal->GetResponseVersion(&major,
                                                                  &minor))) {
           ver = nsPrintfCString("/%lu.%lu", major, minor);
@@ -1155,17 +1156,17 @@ nsresult nsPluginStreamListenerPeer::SetUpStreamListener(nsIRequest *request,
     // nsPluginStreamType_AsFile stream type and we have to save decompressed
     // file into local plugin cache, because necko cache contains original
     // compressed file.
-    nsCAutoString contentEncoding;
+    nsAutoCString contentEncoding;
     if (NS_SUCCEEDED(httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Content-Encoding"),
                                                     contentEncoding))) {
       useLocalCache = true;
     } else {
       // set seekability (seekable if the stream has a known length and if the
       // http server accepts byte ranges).
-      PRUint32 length;
+      uint32_t length;
       GetLength(&length);
       if (length) {
-        nsCAutoString range;
+        nsAutoCString range;
         if (NS_SUCCEEDED(httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("accept-ranges"), range)) &&
             range.Equals(NS_LITERAL_CSTRING("bytes"), nsCaseInsensitiveCStringComparator())) {
           mSeekable = true;
@@ -1175,7 +1176,7 @@ nsresult nsPluginStreamListenerPeer::SetUpStreamListener(nsIRequest *request,
     
     // we require a content len
     // get Last-Modified header for plugin info
-    nsCAutoString lastModified;
+    nsAutoCString lastModified;
     if (NS_SUCCEEDED(httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("last-modified"), lastModified)) &&
         !lastModified.IsEmpty()) {
       PRTime time64;
@@ -1184,7 +1185,7 @@ nsresult nsPluginStreamListenerPeer::SetUpStreamListener(nsIRequest *request,
       // Convert PRTime to unix-style time_t, i.e. seconds since the epoch
       double fpTime;
       LL_L2D(fpTime, time64);
-      mModified = (PRUint32)(fpTime * 1e-6 + 0.5);
+      mModified = (uint32_t)(fpTime * 1e-6 + 0.5);
     }
   }
   
@@ -1223,7 +1224,7 @@ nsPluginStreamListenerPeer::OnFileAvailable(nsIFile* aFile)
   if (!mPStreamListener)
     return NS_ERROR_FAILURE;
   
-  nsCAutoString path;
+  nsAutoCString path;
   rv = aFile->GetNativePath(path);
   if (NS_FAILED(rv)) return rv;
   
@@ -1250,8 +1251,7 @@ nsPluginStreamListenerPeer::GetInterfaceGlobal(const nsIID& aIID, void** result)
     return NS_ERROR_FAILURE;
   }
 
-  nsCOMPtr<nsIPluginInstanceOwner> owner;
-  mPluginInstance->GetOwner(getter_AddRefs(owner));
+  nsRefPtr<nsPluginInstanceOwner> owner = mPluginInstance->GetOwner();
   if (owner) {
     nsCOMPtr<nsIDocument> doc;
     nsresult rv = owner->GetDocument(getter_AddRefs(doc));
@@ -1326,7 +1326,7 @@ NS_IMPL_ISUPPORTS1(ChannelRedirectProxyCallback, nsIAsyncVerifyRedirectCallback)
 
 NS_IMETHODIMP
 nsPluginStreamListenerPeer::AsyncOnChannelRedirect(nsIChannel *oldChannel, nsIChannel *newChannel,
-                                                   PRUint32 flags, nsIAsyncVerifyRedirectCallback* callback)
+                                                   uint32_t flags, nsIAsyncVerifyRedirectCallback* callback)
 {
   // Disallow redirects if we don't have a stream listener.
   if (!mPStreamListener) {
@@ -1345,13 +1345,13 @@ nsPluginStreamListenerPeer::AsyncOnChannelRedirect(nsIChannel *oldChannel, nsICh
   // Don't allow cross-origin 307 POST redirects.
   nsCOMPtr<nsIHttpChannel> oldHttpChannel(do_QueryInterface(oldChannel));
   if (oldHttpChannel) {
-    PRUint32 responseStatus;
+    uint32_t responseStatus;
     nsresult rv = oldHttpChannel->GetResponseStatus(&responseStatus);
     if (NS_FAILED(rv)) {
       return rv;
     }
     if (responseStatus == 307) {
-      nsCAutoString method;
+      nsAutoCString method;
       rv = oldHttpChannel->GetRequestMethod(method);
       if (NS_FAILED(rv)) {
         return rv;
