@@ -18,6 +18,8 @@
 #include "nsIDocShellTreeItem.h"
 #include "nsIWebProgressListener2.h"
 
+#include "nsISettingsService.h"
+
 #include "nsDOMEventTargetHelper.h"
 #include "TabChild.h"
 
@@ -40,6 +42,7 @@
 #include "mozilla/unused.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/ClearOnShutdown.h"
 
 #include <math.h>
 
@@ -66,10 +69,60 @@
 // that a window can make.
 #define MAX_GEO_REQUESTS_PER_WINDOW  1500
 
+// The settings key.
+#define GEO_SETINGS_ENABLED          "geolocation.enabled"
+
 using mozilla::unused;          // <snicker>
 using namespace mozilla;
 using namespace mozilla::dom;
 
+class GeolocationSettingsCallback : public nsISettingsServiceCallback
+{
+public:
+  NS_DECL_ISUPPORTS
+
+  GeolocationSettingsCallback() {
+    MOZ_COUNT_CTOR(GeolocationSettingsCallback);
+  }
+
+  virtual ~GeolocationSettingsCallback() {
+    MOZ_COUNT_DTOR(GeolocationSettingsCallback);
+  }
+
+  NS_IMETHOD Handle(const nsAString& aName, const jsval& aResult)
+  {
+    MOZ_ASSERT(NS_IsMainThread());
+
+    // The geolocation is enabled by default:
+    bool value = true;
+    if (aResult.isBoolean()) {
+      value = aResult.toBoolean();
+    }
+
+    MozSettingValue(value);
+    return NS_OK;
+  }
+
+  NS_IMETHOD HandleError(const nsAString& aName)
+  {
+    NS_WARNING("Unable to get value for '" GEO_SETINGS_ENABLED "'");
+
+    // Default it's enabled:
+    MozSettingValue(true);
+    return NS_OK;
+  }
+
+  void MozSettingValue(const bool aValue)
+  {
+    nsRefPtr<nsGeolocationService> gs = nsGeolocationService::GetGeolocationService();
+    if (gs) {
+      gs->HandleMozsettingValue(aValue);
+    }
+  }
+};
+
+NS_IMPL_ISUPPORTS1(GeolocationSettingsCallback, nsISettingsServiceCallback);
+
 class RequestPromptEvent : public nsRunnable
 {
 public:
@@ -100,10 +153,11 @@ public:
   }
 
   NS_IMETHOD Run() {
-    if (mAllow)
+    if (mAllow) {
       mRequest->Allow();
-    else
+    } else {
       mRequest->Cancel();
+    }
     return NS_OK;
   }
 
@@ -129,8 +183,9 @@ public:
 
   NS_IMETHOD Run() {
     mRequest->SendLocation(mPosition);
-    if (mLocator)
+    if (mLocator) {
       mLocator->RemoveRequest(mRequest);
+    }
     return NS_OK;
   }
 
@@ -151,12 +206,12 @@ public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIDOMGEOPOSITIONERROR
 
-  nsDOMGeoPositionError(PRInt16 aCode);
+  nsDOMGeoPositionError(int16_t aCode);
   void NotifyCallback(nsIDOMGeoPositionErrorCallback* callback);
 
 private:
   ~nsDOMGeoPositionError();
-  PRInt16 mCode;
+  int16_t mCode;
 };
 
 DOMCI_DATA(GeoPositionError, nsDOMGeoPositionError)
@@ -170,7 +225,7 @@ NS_INTERFACE_MAP_END
 NS_IMPL_THREADSAFE_ADDREF(nsDOMGeoPositionError)
 NS_IMPL_THREADSAFE_RELEASE(nsDOMGeoPositionError)
 
-nsDOMGeoPositionError::nsDOMGeoPositionError(PRInt16 aCode)
+nsDOMGeoPositionError::nsDOMGeoPositionError(int16_t aCode)
   : mCode(aCode)
 {
 }
@@ -179,7 +234,7 @@ nsDOMGeoPositionError::~nsDOMGeoPositionError(){}
 
 
 NS_IMETHODIMP
-nsDOMGeoPositionError::GetCode(PRInt16 *aCode)
+nsDOMGeoPositionError::GetCode(int16_t *aCode)
 {
   NS_ENSURE_ARG_POINTER(aCode);
   *aCode = mCode;
@@ -189,17 +244,19 @@ nsDOMGeoPositionError::GetCode(PRInt16 *aCode)
 void
 nsDOMGeoPositionError::NotifyCallback(nsIDOMGeoPositionErrorCallback* aCallback)
 {
-  if (!aCallback)
+  if (!aCallback) {
     return;
-  
+  }
+
   // Ensure that the proper context is on the stack (bug 452762)
   nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
-  if (!stack || NS_FAILED(stack->Push(nsnull)))
+  if (!stack || NS_FAILED(stack->Push(nullptr))) {
     return;
+  }
 
   nsAutoMicroTask mt;
   aCallback->HandleEvent(this);
-  
+
   // remove the stack
   JSContext* cx;
   stack->Pop(&cx);
@@ -211,13 +268,16 @@ nsDOMGeoPositionError::NotifyCallback(nsIDOMGeoPositionErrorCallback* aCallback)
 nsGeolocationRequest::nsGeolocationRequest(nsGeolocation* aLocator,
                                            nsIDOMGeoPositionCallback* aCallback,
                                            nsIDOMGeoPositionErrorCallback* aErrorCallback,
-                                           bool aWatchPositionRequest)
+                                           bool aWatchPositionRequest,
+                                           int32_t aWatchId)
   : mAllowed(false),
     mCleared(false),
+    mIsFirstUpdate(true),
     mIsWatchPositionRequest(aWatchPositionRequest),
     mCallback(aCallback),
     mErrorCallback(aErrorCallback),
-    mLocator(aLocator)
+    mLocator(aLocator),
+    mWatchId(aWatchId)
 {
 }
 
@@ -249,12 +309,13 @@ NS_IMPL_CYCLE_COLLECTION_3(nsGeolocationRequest, mCallback, mErrorCallback, mLoc
 
 
 void
-nsGeolocationRequest::NotifyError(PRInt16 errorCode)
+nsGeolocationRequest::NotifyError(int16_t errorCode)
 {
   nsRefPtr<nsDOMGeoPositionError> positionError = new nsDOMGeoPositionError(errorCode);
-  if (!positionError)
+  if (!positionError) {
     return;
-  
+  }
+
   positionError->NotifyCallback(mErrorCallback);
 }
 
@@ -265,22 +326,22 @@ nsGeolocationRequest::Notify(nsITimer* aTimer)
   // If we haven't gotten an answer from the geolocation
   // provider yet, cancel the request.  Same logic as
   // ::Cancel, just a different error
-  
+
   // remove ourselves from the locator's callback lists.
   mLocator->RemoveRequest(this);
   NotifyError(nsIDOMGeoPositionError::TIMEOUT);
 
-  mTimeoutTimer = nsnull;
+  mTimeoutTimer = nullptr;
   return NS_OK;
 }
- 
-NS_IMETHODIMP
-nsGeolocationRequest::GetUri(nsIURI * *aRequestingURI)
-{
-  NS_ENSURE_ARG_POINTER(aRequestingURI);
 
-  nsCOMPtr<nsIURI> uri = mLocator->GetURI();
-  uri.forget(aRequestingURI);
+NS_IMETHODIMP
+nsGeolocationRequest::GetPrincipal(nsIPrincipal * *aRequestingPrincipal)
+{
+  NS_ENSURE_ARG_POINTER(aRequestingPrincipal);
+
+  nsCOMPtr<nsIPrincipal> principal = mLocator->GetPrincipal();
+  principal.forget(aRequestingPrincipal);
 
   return NS_OK;
 }
@@ -292,6 +353,13 @@ nsGeolocationRequest::GetType(nsACString & aType)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsGeolocationRequest::GetAccess(nsACString & aAccess)
+{
+  aAccess = "unused";
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsGeolocationRequest::GetWindow(nsIDOMWindow * *aRequestingWindow)
 {
@@ -307,7 +375,7 @@ NS_IMETHODIMP
 nsGeolocationRequest::GetElement(nsIDOMElement * *aRequestingElement)
 {
   NS_ENSURE_ARG_POINTER(aRequestingElement);
-  *aRequestingElement = nsnull;
+  *aRequestingElement = nullptr;
   return NS_OK;
 }
 
@@ -324,21 +392,22 @@ nsGeolocationRequest::Cancel()
 NS_IMETHODIMP
 nsGeolocationRequest::Allow()
 {
-  nsRefPtr<nsGeolocationService> geoService = nsGeolocationService::GetInstance();
+  nsRefPtr<nsGeolocationService> gs = nsGeolocationService::GetGeolocationService();
 
   // Kick off the geo device, if it isn't already running
-  nsresult rv = geoService->StartDevice();
-  
+  nsresult rv = gs->StartDevice();
+
   if (NS_FAILED(rv)) {
     // Location provider error
     NotifyError(nsIDOMGeoPositionError::POSITION_UNAVAILABLE);
     return NS_OK;
   }
-  
-  nsCOMPtr<nsIDOMGeoPosition> lastPosition = geoService->GetCachedPosition();
+
+  nsCOMPtr<nsIDOMGeoPosition> lastPosition = gs->GetCachedPosition();
   DOMTimeStamp cachedPositionTime;
-  if (lastPosition)
+  if (lastPosition) {
     lastPosition->GetTimestamp(&cachedPositionTime);
+  }
 
   // check to see if we can use a cached value
   //
@@ -346,14 +415,14 @@ nsGeolocationRequest::Allow()
   // a) the user has specified a maximumAge which allows us to return a cached value,
   // -or-
   // b) the cached position time is some reasonable value to return to the user (<30s)
-  
-  PRUint32 maximumAge = 30 * PR_MSEC_PER_SEC;
+
+  uint32_t maximumAge = 30 * PR_MSEC_PER_SEC;
   if (mOptions) {
     if (mOptions->maximumAge >= 0) {
       maximumAge = mOptions->maximumAge;
     }
     if (mOptions->enableHighAccuracy) {
-      geoService->SetHigherAccuracy(true);
+      gs->SetHigherAccuracy(true);
     }
   }
 
@@ -362,10 +431,10 @@ nsGeolocationRequest::Allow()
         PRTime(cachedPositionTime) )) {
     // okay, we can return a cached position
     mAllowed = true;
-    
-     nsCOMPtr<nsIRunnable> ev =
-         new RequestSendLocationEvent(lastPosition, this,
-                                      mIsWatchPositionRequest ? nsnull : mLocator);
+
+    nsCOMPtr<nsIRunnable> ev = new RequestSendLocationEvent(lastPosition,
+							    this,
+							    mIsWatchPositionRequest ? nullptr : mLocator);
     NS_DispatchToMainThread(ev);
   }
 
@@ -380,15 +449,17 @@ nsGeolocationRequest::SetTimeoutTimer()
 {
   if (mTimeoutTimer) {
     mTimeoutTimer->Cancel();
-    mTimeoutTimer = nsnull;
+    mTimeoutTimer = nullptr;
   }
-  PRInt32 timeout;
+
+  int32_t timeout;
   if (mOptions && (timeout = mOptions->timeout) != 0) {
 
-    if (timeout < 0)
+    if (timeout < 0) {
       timeout = 0;
-    else if (timeout < 10)
+    } else if (timeout < 10) {
       timeout = 10;
+    }
 
     mTimeoutTimer = do_CreateInstance("@mozilla.org/timer;1");
     mTimeoutTimer->InitWithCallback(this, timeout, nsITimer::TYPE_ONE_SHOT);
@@ -400,7 +471,7 @@ nsGeolocationRequest::MarkCleared()
 {
   if (mTimeoutTimer) {
     mTimeoutTimer->Cancel();
-    mTimeoutTimer = nsnull;
+    mTimeoutTimer = nullptr;
   }
   mCleared = true;
 }
@@ -408,12 +479,13 @@ nsGeolocationRequest::MarkCleared()
 void
 nsGeolocationRequest::SendLocation(nsIDOMGeoPosition* aPosition)
 {
-  if (mCleared || !mAllowed)
+  if (mCleared || !mAllowed) {
     return;
+  }
 
   if (mTimeoutTimer) {
     mTimeoutTimer->Cancel();
-    mTimeoutTimer = nsnull;
+    mTimeoutTimer = nullptr;
   }
 
   // we should not pass null back to the DOM.
@@ -424,8 +496,9 @@ nsGeolocationRequest::SendLocation(nsIDOMGeoPosition* aPosition)
 
   // Ensure that the proper context is on the stack (bug 452762)
   nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
-  if (!stack || NS_FAILED(stack->Push(nsnull)))
+  if (!stack || NS_FAILED(stack->Push(nullptr))) {
     return; // silently fail
+  }
 
   nsAutoMicroTask mt;
   mCallback->HandleEvent(aPosition);
@@ -434,19 +507,31 @@ nsGeolocationRequest::SendLocation(nsIDOMGeoPosition* aPosition)
   JSContext* cx;
   stack->Pop(&cx);
 
-  if (mIsWatchPositionRequest)
+  if (mIsWatchPositionRequest) {
     SetTimeoutTimer();
+  }
 }
 
 bool
-nsGeolocationRequest::Update(nsIDOMGeoPosition* aPosition)
+nsGeolocationRequest::Update(nsIDOMGeoPosition* aPosition, bool aIsBetter)
 {
-  if (!mAllowed)
+  if (!mAllowed) {
     return false;
-  nsCOMPtr<nsIRunnable> ev  =
-      new RequestSendLocationEvent(aPosition, this,
-                                   mIsWatchPositionRequest ? nsnull : mLocator);
-  NS_DispatchToMainThread(ev);
+  }
+  // Only dispatch callbacks if this is the first position for this request, or
+  // if the accuracy is as good or improving.
+  //
+  // This ensures that all listeners get at least one position callback, particularly
+  // in the case when newly detected positions are all less accurate than the cached one.
+  //
+  // Fixes bug 596481
+  if (mIsFirstUpdate || aIsBetter) {
+    mIsFirstUpdate = PR_FALSE;
+    nsCOMPtr<nsIRunnable> ev  = new RequestSendLocationEvent(aPosition,
+                                                             this,
+                                                             mIsWatchPositionRequest ? nullptr : mLocator);
+    NS_DispatchToMainThread(ev);
+  }
   return true;
 }
 
@@ -454,27 +539,28 @@ void
 nsGeolocationRequest::Shutdown()
 {
   if (mOptions && mOptions->enableHighAccuracy) {
-    nsRefPtr<nsGeolocationService> geoService = nsGeolocationService::GetInstance();
-    if (geoService) {
-      geoService->SetHigherAccuracy(false);
+    nsRefPtr<nsGeolocationService> gs = nsGeolocationService::GetGeolocationService();
+    if (gs) {
+      gs->SetHigherAccuracy(false);
     }
   }
 
   if (mTimeoutTimer) {
     mTimeoutTimer->Cancel();
-    mTimeoutTimer = nsnull;
+    mTimeoutTimer = nullptr;
   }
   mCleared = true;
-  mCallback = nsnull;
-  mErrorCallback = nsnull;
+  mCallback = nullptr;
+  mErrorCallback = nullptr;
 }
 
 bool nsGeolocationRequest::Recv__delete__(const bool& allow)
 {
-  if (allow)
+  if (allow) {
     (void) Allow();
-  else
+  } else {
     (void) Cancel();
+  }
   return true;
 }
 ////////////////////////////////////////////////////
@@ -491,8 +577,9 @@ NS_IMPL_THREADSAFE_RELEASE(nsGeolocationService)
 
 
 static bool sGeoEnabled = true;
+static bool sGeoInitPending = true;
 static bool sGeoIgnoreLocationFilter = false;
-static PRInt32 sProviderTimeout = 6000; // Time, in milliseconds, to wait for the location provider to spin up.
+static int32_t sProviderTimeout = 6000; // Time, in milliseconds, to wait for the location provider to spin up.
 
 nsresult nsGeolocationService::Init()
 {
@@ -500,24 +587,52 @@ nsresult nsGeolocationService::Init()
   Preferences::AddBoolVarCache(&sGeoEnabled, "geo.enabled", sGeoEnabled);
   Preferences::AddBoolVarCache(&sGeoIgnoreLocationFilter, "geo.ignore.location_filter", sGeoIgnoreLocationFilter);
 
-  if (!sGeoEnabled)
+  if (!sGeoEnabled) {
     return NS_ERROR_FAILURE;
+  }
+
+  if (XRE_GetProcessType() == GeckoProcessType_Content) {
+    sGeoInitPending = false;
+    return NS_OK;
+  }
 
   nsCOMPtr<nsIGeolocationProvider> provider = do_GetService(NS_GEOLOCATION_PROVIDER_CONTRACTID);
-  if (provider)
+  if (provider) {
     mProviders.AppendObject(provider);
+  }
 
   // look up any providers that were registered via the category manager
   nsCOMPtr<nsICategoryManager> catMan(do_GetService("@mozilla.org/categorymanager;1"));
-  if (!catMan)
+  if (!catMan) {
     return NS_ERROR_FAILURE;
+  }
+
+  // check if the geolocation service is enable from settings
+  nsCOMPtr<nsISettingsService> settings =
+    do_GetService("@mozilla.org/settingsService;1");
+
+  if (settings) {
+    nsCOMPtr<nsISettingsServiceLock> settingsLock;
+    nsresult rv = settings->CreateLock(getter_AddRefs(settingsLock));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsRefPtr<GeolocationSettingsCallback> callback = new GeolocationSettingsCallback();
+    rv = settingsLock->Get(GEO_SETINGS_ENABLED, callback);
+    NS_ENSURE_SUCCESS(rv, rv);
+  } else {
+    // If we cannot obtain the settings service, we continue
+    // assuming that the geolocation is enabled:
+    sGeoInitPending = false;
+  }
 
   // geolocation service can be enabled -> now register observer
   nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
-  if (!obs)
+  if (!obs) {
     return NS_ERROR_FAILURE;
+  }
 
   obs->AddObserver(this, "quit-application", false);
+  obs->AddObserver(this, "mozsettings-changed", false);
 
   nsCOMPtr<nsISimpleEnumerator> geoproviders;
   catMan->EnumerateCategory("geolocation-provider", getter_AddRefs(geoproviders));
@@ -529,16 +644,17 @@ nsresult nsGeolocationService::Init()
       geoproviders->GetNext(getter_AddRefs(elem));
 
       nsCOMPtr<nsISupportsCString> elemString = do_QueryInterface(elem);
-      
-      nsCAutoString name;
+
+      nsAutoCString name;
       elemString->GetData(name);
 
       nsXPIDLCString spec;
       catMan->GetCategoryEntry("geolocation-provider", name.get(), getter_Copies(spec));
 
       provider = do_GetService(spec);
-      if (provider)
+      if (provider) {
         mProviders.AppendObject(provider);
+      }
     }
   }
 
@@ -546,26 +662,30 @@ nsresult nsGeolocationService::Init()
 
 #ifdef MOZ_MAEMO_LIBLOCATION
   provider = new MaemoLocationProvider();
-  if (provider)
+  if (provider) {
     mProviders.AppendObject(provider);
+  }
 #endif
 
 #ifdef MOZ_ENABLE_QTMOBILITY
   provider = new QTMLocationProvider();
-  if (provider)
+  if (provider) {
     mProviders.AppendObject(provider);
+  }
 #endif
 
 #ifdef MOZ_WIDGET_ANDROID
   provider = new AndroidLocationProvider();
-  if (provider)
+  if (provider) {
     mProviders.AppendObject(provider);
+  }
 #endif
 
 #ifdef MOZ_WIDGET_GONK
-  provider = GonkGPSGeolocationProvider::GetSingleton();
-  if (provider)
+  provider = do_GetService(GONK_GPS_GEOLOCATION_PROVIDER_CONTRACTID);
+  if (provider) {
     mProviders.AppendObject(provider);
+  }
 #endif
 
   return NS_OK;
@@ -575,39 +695,104 @@ nsGeolocationService::~nsGeolocationService()
 {
 }
 
+void
+nsGeolocationService::HandleMozsettingChanged(const PRUnichar* aData)
+{
+    // The string that we're interested in will be a JSON string that looks like:
+    //  {"key":"gelocation.enabled","value":true}
+
+    nsCOMPtr<nsIThreadJSContextStack> stack = do_GetService("@mozilla.org/js/xpc/ContextStack;1");
+    if (!stack) {
+      return;
+    }
+
+    JSContext *cx = stack->GetSafeJSContext();
+    if (!cx) {
+      return;
+    }
+
+    nsDependentString dataStr(aData);
+    JS::Value val;
+    if (!JS_ParseJSON(cx, dataStr.get(), dataStr.Length(), &val) || !val.isObject()) {
+      return;
+    }
+
+    JSObject &obj(val.toObject());
+    JS::Value key;
+    if (!JS_GetProperty(cx, &obj, "key", &key) || !key.isString()) {
+      return;
+    }
+
+    JSBool match;
+    if (!JS_StringEqualsAscii(cx, key.toString(), GEO_SETINGS_ENABLED, &match) || (match != JS_TRUE)) {
+      return;
+    }
+
+    JS::Value value;
+    if (!JS_GetProperty(cx, &obj, "value", &value) || !value.isBoolean()) {
+      return;
+    }
+
+    HandleMozsettingValue(value.toBoolean());
+}
+
+void
+nsGeolocationService::HandleMozsettingValue(const bool aValue)
+{
+    if (!aValue) {
+      // turn things off
+      StopDevice();
+      Update(nullptr);
+      mLastPosition = nullptr;
+      sGeoEnabled = false;
+    } else {
+      sGeoEnabled = true;
+    }
+
+    if (sGeoInitPending) {
+      sGeoInitPending = false;
+      for (uint32_t i = 0, length = mGeolocators.Length(); i < length; ++i) {
+        mGeolocators[i]->ServiceReady();
+      }
+    }
+}
+
 NS_IMETHODIMP
 nsGeolocationService::Observe(nsISupports* aSubject,
                               const char* aTopic,
                               const PRUnichar* aData)
 {
-  if (!strcmp("quit-application", aTopic))
-  {
+  if (!strcmp("quit-application", aTopic)) {
     nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
     if (obs) {
       obs->RemoveObserver(this, "quit-application");
+      obs->RemoveObserver(this, "mozsettings-changed");
     }
 
-    for (PRUint32 i = 0; i< mGeolocators.Length(); i++)
+    for (uint32_t i = 0; i< mGeolocators.Length(); i++) {
       mGeolocators[i]->Shutdown();
-
+    }
     StopDevice();
 
     return NS_OK;
   }
-  
-  if (!strcmp("timer-callback", aTopic))
-  {
+
+  if (!strcmp("mozsettings-changed", aTopic)) {
+    HandleMozsettingChanged(aData);
+    return NS_OK;
+  }
+
+  if (!strcmp("timer-callback", aTopic)) {
     // decide if we can close down the service.
-    for (PRUint32 i = 0; i< mGeolocators.Length(); i++)
-      if (mGeolocators[i]->HasActiveCallbacks())
-      {
+    for (uint32_t i = 0; i< mGeolocators.Length(); i++)
+      if (mGeolocators[i]->HasActiveCallbacks()) {
         SetDisconnectTimer();
         return NS_OK;
       }
-    
+
     // okay to close up.
     StopDevice();
-    Update(nsnull);
+    Update(nullptr);
     return NS_OK;
   }
 
@@ -617,13 +802,107 @@ nsGeolocationService::Observe(nsISupports* aSubject,
 NS_IMETHODIMP
 nsGeolocationService::Update(nsIDOMGeoPosition *aSomewhere)
 {
-  SetCachedPosition(aSomewhere);
+  // here we have to determine this aSomewhere is a "better"
+  // position than any previously recv'ed.
 
-  for (PRUint32 i = 0; i< mGeolocators.Length(); i++)
-    mGeolocators[i]->Update(aSomewhere);
+  bool isBetter = IsBetterPosition(aSomewhere);
+
+  if (isBetter) {
+    SetCachedPosition(aSomewhere);
+  }
+
+  for (uint32_t i = 0; i< mGeolocators.Length(); i++) {
+    mGeolocators[i]->Update(aSomewhere, isBetter);
+  }
   return NS_OK;
 }
 
+PRBool
+nsGeolocationService::IsBetterPosition(nsIDOMGeoPosition *aSomewhere)
+{
+  if (!aSomewhere) {
+    return false;
+  }
+  
+  if (mProviders.Count() == 1 || !mLastPosition) {
+    return true;
+  }
+
+  nsCOMPtr<nsIDOMGeoPositionCoords> coords;
+  mLastPosition->GetCoords(getter_AddRefs(coords));
+  if (!coords) {
+    return false;
+  }
+
+  double oldAccuracy;
+  nsresult rv = coords->GetAccuracy(&oldAccuracy);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  double oldLat, oldLon;
+  rv = coords->GetLongitude(&oldLon);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  rv = coords->GetLatitude(&oldLat);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  aSomewhere->GetCoords(getter_AddRefs(coords));
+  if (!coords) {
+    return false;
+  }
+
+  double newAccuracy;
+  rv = coords->GetAccuracy(&newAccuracy);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  double newLat, newLon;
+  rv = coords->GetLongitude(&newLon);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  rv = coords->GetLatitude(&newLat);
+  NS_ENSURE_SUCCESS(rv, false);
+
+  // Latitude and longitude is reported in degrees.
+  // However, it is easier to work in radian:
+  // see: http://en.wikipedia.org/wiki/Radian
+  double radsInDeg = M_PI / 180.0;
+
+  newLat *= radsInDeg;
+  newLon *= radsInDeg;
+  oldLat *= radsInDeg;
+  oldLon *= radsInDeg;
+
+  // WGS84 equatorial radius of earth = 6378137m
+  // http://en.wikipedia.org/wiki/WGS84
+  double radius = 6378137;
+
+  // We want to calculate the "Great Circle distance"
+  // between the point (lat1, lon1) and (lat2, lon2).  We
+  // will use the spherical law of cosines to the triangle
+  // formed by our two points and the north pole.
+  //
+  // a = sin ( lat1 ) * sin ( lat2 )  + cos ( lat1 ) * cos (lat2) * cos (lon1 - lon2)
+  // R = radius of circle
+  // distance = arccos ( a ) * R 
+  //
+  // http://en.wikipedia.org/wiki/Great-circle_distance
+
+  double delta = acos( (sin(newLat) * sin(oldLat)) +
+                       (cos(newLat) * cos(oldLat) * cos(oldLon - newLon)) ) * radius; 
+
+  // The threshold is when the distance between the two
+  // positions exceeds the worse (larger value) of the two
+  // accuracies.
+  double max_accuracy = NS_MAX(oldAccuracy, newAccuracy);
+  if (delta > max_accuracy)
+    return true;
+
+  // check to see if the aSomewhere position is more accurate
+  if (oldAccuracy >= newAccuracy)
+    return true;
+
+  return false;
+}
+
 void
 nsGeolocationService::SetCachedPosition(nsIDOMGeoPosition* aPosition)
 {
@@ -639,8 +918,9 @@ nsGeolocationService::GetCachedPosition()
 nsresult
 nsGeolocationService::StartDevice()
 {
-  if (!sGeoEnabled)
+  if (!sGeoEnabled || sGeoInitPending) {
     return NS_ERROR_NOT_AVAILABLE;
+  }
 
   // we do not want to keep the geolocation devices online
   // indefinitely.  Close them down after a reasonable period of
@@ -655,10 +935,11 @@ nsGeolocationService::StartDevice()
 
   // Start them up!
   nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
-  if (!obs)
+  if (!obs) {
     return NS_ERROR_FAILURE;
+  }
 
-  for (PRInt32 i = 0; i < mProviders.Count(); i++) {
+  for (int32_t i = 0; i < mProviders.Count(); i++) {
     mProviders[i]->Startup();
     mProviders[i]->Watch(this);
     obs->NotifyObservers(mProviders[i],
@@ -672,10 +953,11 @@ nsGeolocationService::StartDevice()
 void
 nsGeolocationService::SetDisconnectTimer()
 {
-  if (!mDisconnectTimer)
+  if (!mDisconnectTimer) {
     mDisconnectTimer = do_CreateInstance("@mozilla.org/timer;1");
-  else
+  } else {
     mDisconnectTimer->Cancel();
+  }
 
   mDisconnectTimer->Init(this,
                          sProviderTimeout,
@@ -685,27 +967,27 @@ nsGeolocationService::SetDisconnectTimer()
 void
 nsGeolocationService::SetHigherAccuracy(bool aEnable)
 {
-    if (!mHigherAccuracy && aEnable) {
-	  for (PRInt32 i = 0; i < mProviders.Count(); i++) {
-	    mProviders[i]->SetHighAccuracy(true);
-	  }
-    }
-	
-    if (mHigherAccuracy && !aEnable) {
-	  for (PRInt32 i = 0; i < mProviders.Count(); i++) {
-	    mProviders[i]->SetHighAccuracy(false);
-	  }
+  if (!mHigherAccuracy && aEnable) {
+    for (int32_t i = 0; i < mProviders.Count(); i++) {
+      mProviders[i]->SetHighAccuracy(true);
     }
+  }
 
-    mHigherAccuracy = aEnable;
+  if (mHigherAccuracy && !aEnable) {
+    for (int32_t i = 0; i < mProviders.Count(); i++) {
+      mProviders[i]->SetHighAccuracy(false);
+    }
+  }
+
+  mHigherAccuracy = aEnable;
 }
 
-void 
+void
 nsGeolocationService::StopDevice()
 {
   if(mDisconnectTimer) {
     mDisconnectTimer->Cancel();
-    mDisconnectTimer = nsnull;
+    mDisconnectTimer = nullptr;
   }
 
   if (XRE_GetProcessType() == GeckoProcessType_Content) {
@@ -715,10 +997,11 @@ nsGeolocationService::StopDevice()
   }
 
   nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
-  if (!obs)
+  if (!obs) {
     return;
+  }
 
-  for (PRInt32 i = 0; i < mProviders.Count(); i++) {
+  for (int32_t i = 0; i < mProviders.Count(); i++) {
     mProviders[i]->Shutdown();
     obs->NotifyObservers(mProviders[i],
                          "geolocation-device-events",
@@ -726,31 +1009,24 @@ nsGeolocationService::StopDevice()
   }
 }
 
-nsGeolocationService* nsGeolocationService::gService = nsnull;
+nsRefPtr<nsGeolocationService> nsGeolocationService::sService;
 
-nsGeolocationService*
-nsGeolocationService::GetInstance()
-{
-  if (!nsGeolocationService::gService) {
-    nsGeolocationService::gService = new nsGeolocationService();
-    NS_ASSERTION(nsGeolocationService::gService, "null nsGeolocationService.");
-
-    if (nsGeolocationService::gService) {
-      if (NS_FAILED(nsGeolocationService::gService->Init())) {
-        delete nsGeolocationService::gService;
-        nsGeolocationService::gService = nsnull;
-      }        
-    }
-  }
-  return nsGeolocationService::gService;
-}
-
-nsGeolocationService*
+already_AddRefed<nsGeolocationService>
 nsGeolocationService::GetGeolocationService()
 {
-  nsGeolocationService* inst = nsGeolocationService::GetInstance();
-  NS_IF_ADDREF(inst);
-  return inst;
+  nsRefPtr<nsGeolocationService> result;
+  if (nsGeolocationService::sService) {
+    result = nsGeolocationService::sService;
+    return result.forget();
+  }
+
+  result = new nsGeolocationService();
+  if (NS_FAILED(result->Init())) {
+    return nullptr;
+  }
+  ClearOnShutdown(&nsGeolocationService::sService);
+  nsGeolocationService::sService = result;
+  return result.forget();
 }
 
 void
@@ -782,12 +1058,16 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE(nsGeolocation)
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsGeolocation)
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsGeolocation)
+  tmp->mPendingRequests.Clear();
   tmp->mPendingCallbacks.Clear();
   tmp->mWatchingCallbacks.Clear();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsGeolocation)
-  PRUint32 i; 
+  uint32_t i;
+  for (i = 0; i < tmp->mPendingRequests.Length(); ++i)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR_AMBIGUOUS(mPendingRequests[i].request, nsIContentPermissionRequest)
+
   for (i = 0; i < tmp->mPendingCallbacks.Length(); ++i)
     NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR_AMBIGUOUS(mPendingCallbacks[i], nsIContentPermissionRequest)
 
@@ -795,14 +1075,16 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsGeolocation)
     NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR_AMBIGUOUS(mWatchingCallbacks[i], nsIContentPermissionRequest)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
-nsGeolocation::nsGeolocation() 
+nsGeolocation::nsGeolocation()
+: mLastWatchId(0)
 {
 }
 
 nsGeolocation::~nsGeolocation()
 {
-  if (mService)
+  if (mService) {
     Shutdown();
+  }
 }
 
 nsresult
@@ -811,33 +1093,33 @@ nsGeolocation::Init(nsIDOMWindow* aContentDom)
   // Remember the window
   if (aContentDom) {
     nsCOMPtr<nsPIDOMWindow> window = do_QueryInterface(aContentDom);
-    if (!window)
+    if (!window) {
       return NS_ERROR_FAILURE;
+    }
 
     mOwner = do_GetWeakReference(window->GetCurrentInnerWindow());
-    if (!mOwner)
+    if (!mOwner) {
       return NS_ERROR_FAILURE;
+    }
 
-    // Grab the uri of the document
+    // Grab the principal of the document
     nsCOMPtr<nsIDOMDocument> domdoc;
     aContentDom->GetDocument(getter_AddRefs(domdoc));
     nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
-    if (!doc)
+    if (!doc) {
       return NS_ERROR_FAILURE;
+    }
 
-    doc->NodePrincipal()->GetURI(getter_AddRefs(mURI));
-    
-    if (!mURI)
-      return NS_ERROR_FAILURE;
+    mPrincipal = doc->NodePrincipal();
   }
 
   // If no aContentDom was passed into us, we are being used
-  // by chrome/c++ and have no mOwner, no mURI, and no need
+  // by chrome/c++ and have no mOwner, no mPrincipal, and no need
   // to prompt.
-  mService = nsGeolocationService::GetInstance();
-  if (mService)
+  mService = nsGeolocationService::GetGeolocationService();
+  if (mService) {
     mService->AddLocator(this);
-
+  }
   return NS_OK;
 }
 
@@ -845,27 +1127,30 @@ void
 nsGeolocation::Shutdown()
 {
   // Shutdown and release all callbacks
-  for (PRUint32 i = 0; i< mPendingCallbacks.Length(); i++)
+  for (uint32_t i = 0; i< mPendingCallbacks.Length(); i++)
     mPendingCallbacks[i]->Shutdown();
   mPendingCallbacks.Clear();
 
-  for (PRUint32 i = 0; i< mWatchingCallbacks.Length(); i++)
+  for (uint32_t i = 0; i< mWatchingCallbacks.Length(); i++)
     mWatchingCallbacks[i]->Shutdown();
   mWatchingCallbacks.Clear();
 
-  if (mService)
+  if (mService) {
     mService->RemoveLocator(this);
+  }
 
-  mService = nsnull;
-  mURI = nsnull;
+  mService = nullptr;
+  mPrincipal = nullptr;
 }
 
 bool
 nsGeolocation::HasActiveCallbacks()
 {
-  for (PRUint32 i = 0; i < mWatchingCallbacks.Length(); i++)
-    if (mWatchingCallbacks[i]->IsActive())
+  for (uint32_t i = 0; i < mWatchingCallbacks.Length(); i++) {
+    if (mWatchingCallbacks[i]->IsActive()) {
       return true;
+    }
+  }
 
   return mPendingCallbacks.Length() != 0;
 }
@@ -885,19 +1170,21 @@ nsGeolocation::RemoveRequest(nsGeolocationRequest* aRequest)
 }
 
 void
-nsGeolocation::Update(nsIDOMGeoPosition *aSomewhere)
+nsGeolocation::Update(nsIDOMGeoPosition *aSomewhere, bool aIsBetter)
 {
-  if (!WindowOwnerStillExists())
+  if (!WindowOwnerStillExists()) {
     return Shutdown();
+  }
 
-  for (PRUint32 i = mPendingCallbacks.Length(); i> 0; i--) {
-    if (mPendingCallbacks[i-1]->Update(aSomewhere))
+  for (uint32_t i = mPendingCallbacks.Length(); i> 0; i--) {
+    if (mPendingCallbacks[i-1]->Update(aSomewhere, aIsBetter)) {
       mPendingCallbacks.RemoveElementAt(i-1);
+    }
   }
 
   // notify everyone that is watching
-  for (PRUint32 i = 0; i< mWatchingCallbacks.Length(); i++) {
-    mWatchingCallbacks[i]->Update(aSomewhere);
+  for (uint32_t i = 0; i< mWatchingCallbacks.Length(); i++) {
+    mWatchingCallbacks[i]->Update(aSomewhere, aIsBetter);
   }
 }
 
@@ -909,36 +1196,58 @@ nsGeolocation::GetCurrentPosition(nsIDOMGeoPositionCallback *callback,
 {
   NS_ENSURE_ARG_POINTER(callback);
 
-  if (!sGeoEnabled)
-    return NS_ERROR_NOT_AVAILABLE;
-
-  if (mPendingCallbacks.Length() > MAX_GEO_REQUESTS_PER_WINDOW)
+  if (mPendingCallbacks.Length() > MAX_GEO_REQUESTS_PER_WINDOW) {
     return NS_ERROR_NOT_AVAILABLE;
+  }
 
   nsRefPtr<nsGeolocationRequest> request = new nsGeolocationRequest(this,
-								    callback,
-								    errorCallback,
-								    false);
-  if (!request)
+                                                                    callback,
+                                                                    errorCallback,
+                                                                    false);
+  if (!request) {
     return NS_ERROR_OUT_OF_MEMORY;
+  }
 
   nsresult rv = request->Init(cx, options);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (mOwner) {
-    if (!RegisterRequestWithPrompt(request))
-      return NS_ERROR_NOT_AVAILABLE;
-    
-    mPendingCallbacks.AppendElement(request);
+  if (!sGeoEnabled) {
+    nsCOMPtr<nsIRunnable> ev = new RequestAllowEvent(false, request);
+    NS_DispatchToMainThread(ev);
     return NS_OK;
   }
 
-  if (!nsContentUtils::IsCallerChrome())
+  if (!mOwner && !nsContentUtils::IsCallerChrome()) {
     return NS_ERROR_FAILURE;
+  }
 
   mPendingCallbacks.AppendElement(request);
 
-  nsCOMPtr<nsIRunnable> ev = new RequestAllowEvent(true, request);
+  if (sGeoInitPending) {
+    PendingRequest req = { request, PendingRequest::GetCurrentPosition };
+    mPendingRequests.AppendElement(req);
+    return NS_OK;
+  }
+
+  return GetCurrentPositionReady(request);
+}
+
+nsresult
+nsGeolocation::GetCurrentPositionReady(nsGeolocationRequest* aRequest)
+{
+  if (mOwner) {
+    if (!RegisterRequestWithPrompt(aRequest)) {
+      return NS_ERROR_NOT_AVAILABLE;
+    }
+
+    return NS_OK;
+  }
+
+  if (!nsContentUtils::IsCallerChrome()) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIRunnable> ev = new RequestAllowEvent(true, aRequest);
   NS_DispatchToMainThread(ev);
 
   return NS_OK;
@@ -949,82 +1258,131 @@ nsGeolocation::WatchPosition(nsIDOMGeoPositionCallback *callback,
                              nsIDOMGeoPositionErrorCallback *errorCallback,
                              const jsval& options,
                              JSContext* cx,
-                             PRInt32 *_retval NS_OUTPARAM)
+                             int32_t *_retval)
 {
-
   NS_ENSURE_ARG_POINTER(callback);
 
-  if (!sGeoEnabled)
+  if (mPendingCallbacks.Length() > MAX_GEO_REQUESTS_PER_WINDOW) {
     return NS_ERROR_NOT_AVAILABLE;
+  }
 
-  if (mPendingCallbacks.Length() > MAX_GEO_REQUESTS_PER_WINDOW)
-    return NS_ERROR_NOT_AVAILABLE;
+  // The watch ID:
+  *_retval = mLastWatchId++;
 
   nsRefPtr<nsGeolocationRequest> request = new nsGeolocationRequest(this,
-								    callback,
-								    errorCallback,
-								    true);
-  if (!request)
+                                                                    callback,
+                                                                    errorCallback,
+                                                                    true,
+                                                                    *_retval);
+  if (!request) {
     return NS_ERROR_OUT_OF_MEMORY;
+  }
 
   nsresult rv = request->Init(cx, options);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (mOwner) {
-    if (!RegisterRequestWithPrompt(request))
-      return NS_ERROR_NOT_AVAILABLE;
+  if (!sGeoEnabled) {
+    nsCOMPtr<nsIRunnable> ev = new RequestAllowEvent(false, request);
 
     // need to hand back an index/reference.
     mWatchingCallbacks.AppendElement(request);
-    *_retval = mWatchingCallbacks.Length() - 1;
+
+    NS_DispatchToMainThread(ev);
+    return NS_OK;
+  }
+
+  if (!mOwner && !nsContentUtils::IsCallerChrome()) {
+    return NS_ERROR_FAILURE;
+  }
+
+  mWatchingCallbacks.AppendElement(request);
+
+  if (sGeoInitPending) {
+    PendingRequest req = { request, PendingRequest::WatchPosition };
+    mPendingRequests.AppendElement(req);
+    return NS_OK;
+  }
+
+  return WatchPositionReady(request);
+}
+
+nsresult
+nsGeolocation::WatchPositionReady(nsGeolocationRequest* aRequest)
+{
+  if (mOwner) {
+    if (!RegisterRequestWithPrompt(aRequest))
+      return NS_ERROR_NOT_AVAILABLE;
 
     return NS_OK;
   }
 
-  if (!nsContentUtils::IsCallerChrome())
+  if (!nsContentUtils::IsCallerChrome()) {
     return NS_ERROR_FAILURE;
+  }
 
-  request->Allow();
-
-  // need to hand back an index/reference.
-  mWatchingCallbacks.AppendElement(request);
-  *_retval = mWatchingCallbacks.Length() - 1;
+  aRequest->Allow();
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsGeolocation::ClearWatch(PRInt32 aWatchId)
+nsGeolocation::ClearWatch(int32_t aWatchId)
 {
-  PRUint32 count = mWatchingCallbacks.Length();
-  if (aWatchId < 0 || count == 0 || PRUint32(aWatchId) >= count)
+  if (aWatchId < 0) {
     return NS_OK;
+  }
+
+  for (uint32_t i = 0, length = mWatchingCallbacks.Length(); i < length; ++i) {
+    if (mWatchingCallbacks[i]->WatchId() == aWatchId) {
+      mWatchingCallbacks[i]->MarkCleared();
+      break;
+    }
+  }
 
-  mWatchingCallbacks[aWatchId]->MarkCleared();
   return NS_OK;
 }
 
+void
+nsGeolocation::ServiceReady()
+{
+  for (uint32_t length = mPendingRequests.Length(); length > 0; --length) {
+    switch (mPendingRequests[0].type) {
+      case PendingRequest::GetCurrentPosition:
+        GetCurrentPositionReady(mPendingRequests[0].request);
+        break;
+
+      case PendingRequest::WatchPosition:
+        WatchPositionReady(mPendingRequests[0].request);
+        break;
+    }
+
+    mPendingRequests.RemoveElementAt(0);
+  }
+}
+
 bool
 nsGeolocation::WindowOwnerStillExists()
 {
   // an owner was never set when nsGeolocation
   // was created, which means that this object
   // is being used without a window.
-  if (mOwner == nsnull)
+  if (mOwner == nullptr) {
     return true;
+  }
 
   nsCOMPtr<nsPIDOMWindow> window = do_QueryReferent(mOwner);
 
-  if (window)
-  {
+  if (window) {
     bool closed = false;
     window->GetClosed(&closed);
-    if (closed)
+    if (closed) {
       return false;
+    }
 
     nsPIDOMWindow* outer = window->GetOuterWindow();
-    if (!outer || outer->GetCurrentInnerWindow() != window)
+    if (!outer || outer->GetCurrentInnerWindow() != window) {
       return false;
+    }
   }
 
   return true;
@@ -1034,31 +1392,34 @@ bool
 nsGeolocation::RegisterRequestWithPrompt(nsGeolocationRequest* request)
 {
   if (Preferences::GetBool("geo.prompt.testing", false)) {
-    nsCOMPtr<nsIRunnable> ev =
-        new RequestAllowEvent(Preferences::GetBool("geo.prompt.testing.allow",
-                                                   false), request);
+    bool allow = Preferences::GetBool("geo.prompt.testing.allow", false);
+    nsCOMPtr<nsIRunnable> ev = new RequestAllowEvent(allow,
+						     request);
     NS_DispatchToMainThread(ev);
     return true;
   }
 
   if (XRE_GetProcessType() == GeckoProcessType_Content) {
     nsCOMPtr<nsPIDOMWindow> window = do_QueryReferent(mOwner);
-    if (!window)
+    if (!window) {
       return true;
+    }
 
     // because owner implements nsITabChild, we can assume that it is
     // the one and only TabChild.
     TabChild* child = GetTabChildFrom(window->GetDocShell());
-    if (!child)
+    if (!child) {
       return false;
-    
+    }
+
     // Retain a reference so the object isn't deleted without IPDL's knowledge.
     // Corresponding release occurs in DeallocPContentPermissionRequest.
     request->AddRef();
+    child->SendPContentPermissionRequestConstructor(request,
+                                                    NS_LITERAL_CSTRING("geolocation"),
+                                                    NS_LITERAL_CSTRING("unused"),
+                                                    IPC::Principal(mPrincipal));
 
-    nsCString type = NS_LITERAL_CSTRING("geolocation");
-    child->SendPContentPermissionRequestConstructor(request, type, IPC::URI(mURI));
-    
     request->Sendprompt();
     return true;
   }
@@ -1067,4 +1428,3 @@ nsGeolocation::RegisterRequestWithPrompt(nsGeolocationRequest* request)
   NS_DispatchToMainThread(ev);
   return true;
 }
-
