@@ -22,9 +22,9 @@
 using namespace mozilla;
 
 // font info loader constants
-static const PRUint32 kDelayBeforeLoadingCmaps = 8 * 1000; // 8secs
-static const PRUint32 kIntervalBetweenLoadingCmaps = 150; // 150ms
-static const PRUint32 kNumFontsPerSlice = 10; // read in info 10 fonts at a time
+static const uint32_t kDelayBeforeLoadingCmaps = 8 * 1000; // 8secs
+static const uint32_t kIntervalBetweenLoadingCmaps = 150; // 150ms
+static const uint32_t kNumFontsPerSlice = 10; // read in info 10 fonts at a time
 
 #ifdef PR_LOGGING
 
@@ -36,14 +36,14 @@ static const PRUint32 kNumFontsPerSlice = 10; // read in info 10 fonts at a time
 
 #endif // PR_LOGGING
 
-gfxPlatformFontList *gfxPlatformFontList::sPlatformFontList = nsnull;
+gfxPlatformFontList *gfxPlatformFontList::sPlatformFontList = nullptr;
 
 
 static const char* kObservedPrefs[] = {
     "font.",
     "font.name-list.",
     "intl.accept_languages",  // hmmmm...
-    nsnull
+    nullptr
 };
 
 class gfxFontListPrefObserver MOZ_FINAL : public nsIObserver {
@@ -52,7 +52,7 @@ public:
     NS_DECL_NSIOBSERVER
 };
 
-static gfxFontListPrefObserver* gFontListPrefObserver = nsnull;
+static gfxFontListPrefObserver* gFontListPrefObserver = nullptr;
 
 NS_IMPL_ISUPPORTS1(gfxFontListPrefObserver, nsIObserver)
 
@@ -120,7 +120,7 @@ gfxPlatformFontList::MemoryReporter::CollectReports
 }
 
 NS_IMETHODIMP
-gfxPlatformFontList::MemoryReporter::GetExplicitNonHeap(PRInt64* aAmount)
+gfxPlatformFontList::MemoryReporter::GetExplicitNonHeap(int64_t* aAmount)
 {
     // This reporter only measures heap memory.
     *aAmount = 0;
@@ -207,7 +207,7 @@ gfxPlatformFontList::InitOtherFamilyNames()
     mFontFamilies.Enumerate(gfxPlatformFontList::InitOtherFamilyNamesProc, this);
 }
                                                          
-PLDHashOperator PR_CALLBACK
+PLDHashOperator
 gfxPlatformFontList::InitOtherFamilyNamesProc(nsStringHashKey::KeyType aKey,
                                               nsRefPtr<gfxFontFamily>& aFamilyEntry,
                                               void* userArg)
@@ -227,7 +227,7 @@ gfxPlatformFontList::InitFaceNameLists()
     mFontFamilies.Enumerate(gfxPlatformFontList::InitFaceNameListsProc, this);
 }
 
-PLDHashOperator PR_CALLBACK
+PLDHashOperator
 gfxPlatformFontList::InitFaceNameListsProc(nsStringHashKey::KeyType aKey,
                                            nsRefPtr<gfxFontFamily>& aFamilyEntry,
                                            void* userArg)
@@ -243,8 +243,8 @@ gfxPlatformFontList::PreloadNamesList()
     nsAutoTArray<nsString, 10> preloadFonts;
     gfxFontUtils::GetPrefsFontList("font.preload-names-list", preloadFonts);
 
-    PRUint32 numFonts = preloadFonts.Length();
-    for (PRUint32 i = 0; i < numFonts; i++) {
+    uint32_t numFonts = preloadFonts.Length();
+    for (uint32_t i = 0; i < numFonts; i++) {
         nsAutoString key;
         GenerateFontListKey(preloadFonts[i], key);
         
@@ -266,7 +266,7 @@ gfxPlatformFontList::SetFixedPitch(const nsAString& aFamilyName)
     family->FindStyleVariations();
     nsTArray<nsRefPtr<gfxFontEntry> >& fontlist = family->GetFontList();
 
-    PRUint32 i, numFonts = fontlist.Length();
+    uint32_t i, numFonts = fontlist.Length();
 
     for (i = 0; i < numFonts; i++) {
         fontlist[i]->mFixedPitch = 1;
@@ -278,8 +278,8 @@ gfxPlatformFontList::LoadBadUnderlineList()
 {
     nsAutoTArray<nsString, 10> blacklist;
     gfxFontUtils::GetPrefsFontList("font.blacklist.underline_offset", blacklist);
-    PRUint32 numFonts = blacklist.Length();
-    for (PRUint32 i = 0; i < numFonts; i++) {
+    uint32_t numFonts = blacklist.Length();
+    for (uint32_t i = 0; i < numFonts; i++) {
         nsAutoString key;
         GenerateFontListKey(blacklist[i], key);
         mBadUnderlineFamilyNames.PutEntry(key);
@@ -308,7 +308,7 @@ struct FontListData {
     nsTArray<nsString>& mListOfFonts;
 };
 
-PLDHashOperator PR_CALLBACK
+PLDHashOperator
 gfxPlatformFontList::HashEnumFuncForFamilies(nsStringHashKey::KeyType aKey,
                                              nsRefPtr<gfxFontFamily>& aFamilyEntry,
                                              void *aUserArg)
@@ -358,9 +358,9 @@ struct FontFamilyListData {
         : mFamilyArray(aFamilyArray)
     {}
 
-    static PLDHashOperator PR_CALLBACK AppendFamily(nsStringHashKey::KeyType aKey,
-                                                    nsRefPtr<gfxFontFamily>& aFamilyEntry,
-                                                    void *aUserArg)
+    static PLDHashOperator AppendFamily(nsStringHashKey::KeyType aKey,
+                                        nsRefPtr<gfxFontFamily>& aFamilyEntry,
+                                        void *aUserArg)
     {
         FontFamilyListData *data = static_cast<FontFamilyListData*>(aUserArg);
         data->mFamilyArray.AppendElement(aFamilyEntry);
@@ -378,15 +378,15 @@ gfxPlatformFontList::GetFontFamilyList(nsTArray<nsRefPtr<gfxFontFamily> >& aFami
 }
 
 gfxFontEntry*
-gfxPlatformFontList::SystemFindFontForChar(const PRUint32 aCh,
-                                           PRInt32 aRunScript,
+gfxPlatformFontList::SystemFindFontForChar(const uint32_t aCh,
+                                           int32_t aRunScript,
                                            const gfxFontStyle* aStyle)
  {
-    gfxFontEntry* fontEntry = nsnull;
+    gfxFontEntry* fontEntry = nullptr;
 
     // is codepoint with no matching font? return null immediately
     if (mCodepointsWithNoFonts.test(aCh)) {
-        return nsnull;
+        return nullptr;
     }
 
     // try to short-circuit font fallback for U+FFFD, used to represent
@@ -411,7 +411,7 @@ gfxPlatformFontList::SystemFindFontForChar(const PRUint32 aCh,
     fontEntry = CommonFontFallback(aCh, aRunScript, aStyle);
  
     // if didn't find a font, do system-wide fallback (except for specials)
-    PRUint32 cmapCount = 0;
+    uint32_t cmapCount = 0;
     if (!fontEntry) {
         common = false;
         fontEntry = GlobalFontFallback(aCh, aRunScript, aStyle, cmapCount);
@@ -422,9 +422,9 @@ gfxPlatformFontList::SystemFindFontForChar(const PRUint32 aCh,
     PRLogModuleInfo *log = gfxPlatform::GetLog(eGfxLog_textrun);
 
     if (NS_UNLIKELY(log)) {
-        PRUint32 charRange = gfxFontUtils::CharRangeBit(aCh);
-        PRUint32 unicodeRange = FindCharUnicodeRange(aCh);
-        PRInt32 script = mozilla::unicode::GetScriptCode(aCh);
+        uint32_t charRange = gfxFontUtils::CharRangeBit(aCh);
+        uint32_t unicodeRange = FindCharUnicodeRange(aCh);
+        int32_t script = mozilla::unicode::GetScriptCode(aCh);
         PR_LOG(log, PR_LOG_WARNING,\
                ("(textrun-systemfallback-%s) char: u+%6.6x "
                  "char-range: %d unicode-range: %d script: %d match: [%s]"
@@ -433,7 +433,7 @@ gfxPlatformFontList::SystemFindFontForChar(const PRUint32 aCh,
                  charRange, unicodeRange, script,
                 (fontEntry ? NS_ConvertUTF16toUTF8(fontEntry->Name()).get() :
                     "<none>"),
-                PRInt32(elapsed.ToMicroseconds()),
+                int32_t(elapsed.ToMicroseconds()),
                 cmapCount));
     }
 #endif
@@ -447,7 +447,7 @@ gfxPlatformFontList::SystemFindFontForChar(const PRUint32 aCh,
  
     // track system fallback time
     static bool first = true;
-    PRInt32 intElapsed = PRInt32(first ? elapsed.ToMilliseconds() :
+    int32_t intElapsed = int32_t(first ? elapsed.ToMilliseconds() :
                                          elapsed.ToMicroseconds());
     Telemetry::Accumulate((first ? Telemetry::SYSTEM_FONT_FALLBACK_FIRST :
                                    Telemetry::SYSTEM_FONT_FALLBACK),
@@ -461,7 +461,7 @@ gfxPlatformFontList::SystemFindFontForChar(const PRUint32 aCh,
     return fontEntry;
 }
 
-PLDHashOperator PR_CALLBACK 
+PLDHashOperator 
 gfxPlatformFontList::FindFontForCharProc(nsStringHashKey::KeyType aKey, nsRefPtr<gfxFontFamily>& aFamilyEntry,
      void *userArg)
 {
@@ -476,12 +476,12 @@ gfxPlatformFontList::FindFontForCharProc(nsStringHashKey::KeyType aKey, nsRefPtr
 #define NUM_FALLBACK_FONTS        8
 
 gfxFontEntry*
-gfxPlatformFontList::CommonFontFallback(const PRUint32 aCh,
-                                        PRInt32 aRunScript,
+gfxPlatformFontList::CommonFontFallback(const uint32_t aCh,
+                                        int32_t aRunScript,
                                         const gfxFontStyle* aMatchStyle)
 {
     nsAutoTArray<const char*,NUM_FALLBACK_FONTS> defaultFallbacks;
-    PRUint32 i, numFallbacks;
+    uint32_t i, numFallbacks;
 
     gfxPlatform::GetPlatform()->GetCommonFallbackFonts(aCh, aRunScript,
                                                        defaultFallbacks);
@@ -506,14 +506,14 @@ gfxPlatformFontList::CommonFontFallback(const PRUint32 aCh,
         }
     }
 
-    return nsnull;
+    return nullptr;
 }
 
 gfxFontEntry*
-gfxPlatformFontList::GlobalFontFallback(const PRUint32 aCh,
-                                        PRInt32 aRunScript,
+gfxPlatformFontList::GlobalFontFallback(const uint32_t aCh,
+                                        int32_t aRunScript,
                                         const gfxFontStyle* aMatchStyle,
-                                        PRUint32& aCmapCount)
+                                        uint32_t& aCmapCount)
 {
     // otherwise, try to find it among local fonts
     GlobalFontMatch data(aCh, aRunScript, aMatchStyle);
@@ -559,7 +559,7 @@ gfxPlatformFontList::FindFamily(const nsAString& aFamily)
     }
 
     // lookup in other family names list (mostly localized names)
-    if ((familyEntry = mOtherFamilyNames.GetWeak(key)) != nsnull) {
+    if ((familyEntry = mOtherFamilyNames.GetWeak(key)) != nullptr) {
         return familyEntry;
     }
 
@@ -570,12 +570,12 @@ gfxPlatformFontList::FindFamily(const nsAString& aFamily)
     // in practice so avoid pulling in names at startup
     if (!mOtherFamilyNamesInitialized && !IsASCII(aFamily)) {
         InitOtherFamilyNames();
-        if ((familyEntry = mOtherFamilyNames.GetWeak(key)) != nsnull) {
+        if ((familyEntry = mOtherFamilyNames.GetWeak(key)) != nullptr) {
             return familyEntry;
         }
     }
 
-    return nsnull;
+    return nullptr;
 }
 
 gfxFontEntry*
@@ -588,19 +588,19 @@ gfxPlatformFontList::FindFontForFamily(const nsAString& aFamily, const gfxFontSt
     if (familyEntry)
         return familyEntry->FindFontForStyle(*aStyle, aNeedsBold);
 
-    return nsnull;
+    return nullptr;
 }
 
 bool
 gfxPlatformFontList::GetPrefFontFamilyEntries(eFontPrefLang aLangGroup, nsTArray<nsRefPtr<gfxFontFamily> > *array)
 {
-    return mPrefFonts.Get(PRUint32(aLangGroup), array);
+    return mPrefFonts.Get(uint32_t(aLangGroup), array);
 }
 
 void
 gfxPlatformFontList::SetPrefFontFamilyEntries(eFontPrefLang aLangGroup, nsTArray<nsRefPtr<gfxFontFamily> >& array)
 {
-    mPrefFonts.Put(PRUint32(aLangGroup), array);
+    mPrefFonts.Put(uint32_t(aLangGroup), array);
 }
 
 void 
@@ -702,7 +702,7 @@ gfxPlatformFontList::InitLoader()
 bool
 gfxPlatformFontList::RunLoader()
 {
-    PRUint32 i, endIndex = (mStartIndex + mIncrement < mNumFamilies ? mStartIndex + mIncrement : mNumFamilies);
+    uint32_t i, endIndex = (mStartIndex + mIncrement < mNumFamilies ? mStartIndex + mIncrement : mNumFamilies);
     bool loadCmaps = !UsesSystemFallback() ||
         gfxPlatform::GetPlatform()->UseCmapsDuringSystemFallback();
 
@@ -788,7 +788,7 @@ SizeOfFontNameEntryExcludingThis(const nsAString&              aKey,
 
 static size_t
 SizeOfPrefFontEntryExcludingThis
-    (const PRUint32&                           aKey,
+    (const uint32_t&                           aKey,
      const nsTArray<nsRefPtr<gfxFontFamily> >& aList,
      nsMallocSizeOfFun                         aMallocSizeOf,
      void*                                     aUserArg)
@@ -814,7 +814,7 @@ SizeOfSharedCmapExcludingThis(CharMapHashKey*   aHashEntry,
 {
     FontListSizes *sizes = static_cast<FontListSizes*>(aUserArg);
 
-    PRUint32 size = aHashEntry->GetKey()->SizeOfIncludingThis(aMallocSizeOf);
+    uint32_t size = aHashEntry->GetKey()->SizeOfIncludingThis(aMallocSizeOf);
     sizes->mCharMapsSize += size;
 
     // we return zero here because the measurements have been added directly
