@@ -21,12 +21,8 @@
 
 #include "js/Vector.h"
 
-#define JS_KEYWORD(keyword, type, op, version) \
-    extern const char js_##keyword##_str[];
-#include "jskeyword.tbl"
-#undef JS_KEYWORD
-
 namespace js {
+namespace frontend {
 
 enum TokenKind {
     TOK_ERROR = -1,                /* well-known as the only code < EOF */
@@ -103,6 +99,8 @@ enum TokenKind {
     TOK_YIELD,                     /* yield from generator function */
     TOK_LEXICALSCOPE,              /* block scope AST node label */
     TOK_LET,                       /* let keyword */
+    TOK_EXPORT,                    /* export keyword */
+    TOK_IMPORT,                    /* import keyword */
     TOK_RESERVED,                  /* reserved keywords */
     TOK_STRICT_RESERVED,           /* reserved keywords in strict mode */
 
@@ -266,6 +264,10 @@ struct TokenPos {
     bool operator >=(const TokenPos& bpos) const {
         return !(*this < bpos);
     }
+
+    bool encloses(const TokenPos& pos) const {
+        return begin <= pos.begin && pos.end <= end;
+    }
 };
 
 struct Token {
@@ -388,6 +390,7 @@ enum TokenStreamFlags
     TSF_XMLTEXTMODE = 0x200,    /* scanning XMLText terminal from E4X */
     TSF_XMLONLYMODE = 0x400,    /* don't scan {expr} within text/tag */
     TSF_OCTAL_CHAR = 0x800,     /* observed a octal character escape */
+    TSF_HAD_ERROR = 0x1000,     /* returned TOK_ERROR from getToken */
 
     /*
      * To handle the hard case of contiguous HTML comments, we want to clear the
@@ -408,7 +411,7 @@ enum TokenStreamFlags
      * It does not cope with malformed comment hiding hacks where --> is hidden
      * by C-style comments, or on a dirty line.  Such cases are already broken.
      */
-    TSF_IN_HTML_COMMENT = 0x1000
+    TSF_IN_HTML_COMMENT = 0x2000
 };
 
 struct Parser;
@@ -427,16 +430,14 @@ struct CompileError {
     void throwError();
 };
 
-namespace StrictMode {
 /* For an explanation of how these are used, see the comment in the FunctionBox definition. */
-enum StrictModeState {
+MOZ_BEGIN_ENUM_CLASS(StrictMode, uint8_t)
     NOTSTRICT,
     UNKNOWN,
     STRICT
-};
-}
+MOZ_END_ENUM_CLASS(StrictMode)
 
-inline StrictMode::StrictModeState
+inline StrictMode
 StrictModeFromContext(JSContext *cx)
 {
     return cx->hasRunOption(JSOPTION_STRICT_MODE) ? StrictMode::STRICT : StrictMode::UNKNOWN;
@@ -449,7 +450,7 @@ StrictModeFromContext(JSContext *cx)
 // This class is a tiny back-channel from TokenStream to the strict mode flag
 // that avoids exposing the rest of SharedContext to TokenStream. get()
 // returns the current strict mode state. The other two methods get and set
-// the queuedStrictModeError member of TreeContext. StrictModeGetter's
+// the queuedStrictModeError member of ParseContext. StrictModeGetter's
 // non-inline methods are implemented in Parser.cpp.
 //
 class StrictModeGetter {
@@ -457,7 +458,7 @@ class StrictModeGetter {
   public:
     StrictModeGetter(Parser *p) : parser(p) { }
 
-    StrictMode::StrictModeState get() const;
+    StrictMode get() const;
     CompileError *queuedStrictModeError() const;
     void setQueuedStrictModeError(CompileError *e);
 };
@@ -477,9 +478,8 @@ class TokenStream
   public:
     typedef Vector<jschar, 32> CharBuffer;
 
-    TokenStream(JSContext *cx, JSPrincipals *principals, JSPrincipals *originPrincipals,
-                const jschar *base, size_t length, const char *filename, unsigned lineno,
-                JSVersion version, StrictModeGetter *smg);
+    TokenStream(JSContext *cx, const CompileOptions &options,
+                const jschar *base, size_t length, StrictModeGetter *smg);
 
     ~TokenStream();
 
@@ -494,6 +494,9 @@ class TokenStream
         TokenKind type = currentToken().type;
         return type == type1 || type == type2;
     }
+    size_t offsetOfToken(const Token &tok) const {
+        return tok.ptr - userbuf.base();
+    }
     const CharBuffer &getTokenbuf() const { return tokenbuf; }
     const char *getFilename() const { return filename; }
     unsigned getLineno() const { return lineno; }
@@ -505,6 +508,7 @@ class TokenStream
     bool allowsXML() const { return allowXML && strictModeState() != StrictMode::STRICT; }
     bool hasMoarXML() const { return moarXML || VersionShouldParseXML(versionNumber()); }
     void setMoarXML(bool enabled) { moarXML = enabled; }
+    bool hadError() const { return !!(flags & TSF_HAD_ERROR); }
 
     bool isCurrentTokenEquality() const {
         return TokenKindIsEquality(currentToken().type);
@@ -527,9 +531,9 @@ class TokenStream
     void setXMLOnlyMode(bool enabled = true) { setFlag(enabled, TSF_XMLONLYMODE); }
     void setUnexpectedEOF(bool enabled = true) { setFlag(enabled, TSF_UNEXPECTED_EOF); }
 
-    StrictMode::StrictModeState strictModeState() const
+    StrictMode strictModeState() const
     {
-        return strictModeGetter ? strictModeGetter->get() : StrictMode::NOTSTRICT;
+        return strictModeGetter ? strictModeGetter->get() : StrictMode(StrictMode::NOTSTRICT);
     }
     bool isXMLTagMode() const { return !!(flags & TSF_XMLTAGMODE); }
     bool isXMLOnlyMode() const { return !!(flags & TSF_XMLONLYMODE); }
@@ -550,6 +554,7 @@ class TokenStream
     bool reportStrictModeErrorNumberVA(ParseNode *pn, unsigned errorNumber, va_list args);
 
   private:
+    void onError();
     static JSAtom *atomize(JSContext *cx, CharBuffer &cb);
     bool putIdentInTokenbuf(const jschar *identStart);
 
@@ -667,11 +672,22 @@ class TokenStream
         JS_ALWAYS_TRUE(matchToken(tt));
     }
 
+
+    /*
+     * Return the offset into the source buffer of the end of the token.
+     */
+    size_t endOffset(const Token &tok);
+
+    bool hasSourceMap() const {
+        return sourceMap != NULL;
+    }
+
     /*
      * Give up responsibility for managing the sourceMap filename's memory.
      */
-    const jschar *releaseSourceMap() {
-        const jschar* sm = sourceMap;
+    jschar *releaseSourceMap() {
+        JS_ASSERT(hasSourceMap());
+        jschar *sm = sourceMap;
         sourceMap = NULL;
         return sm;
     }
@@ -703,14 +719,22 @@ class TokenStream
     class TokenBuf {
       public:
         TokenBuf(const jschar *buf, size_t length)
-          : base(buf), limit(buf + length), ptr(buf) { }
+          : base_(buf), limit_(buf + length), ptr(buf) { }
 
         bool hasRawChars() const {
-            return ptr < limit;
+            return ptr < limit_;
         }
 
         bool atStart() const {
-            return ptr == base;
+            return ptr == base_;
+        }
+
+        const jschar *base() const {
+            return base_;
+        }
+
+        const jschar *limit() const {
+            return limit_;
         }
 
         jschar getRawChar() {
@@ -770,8 +794,8 @@ class TokenStream
         const jschar *findEOLMax(const jschar *p, size_t max);
 
       private:
-        const jschar *base;             /* base of buffer */
-        const jschar *limit;            /* limit for quick bounds check */
+        const jschar *base_;            /* base of buffer */
+        const jschar *limit_;           /* limit for quick bounds check */
         const jschar *ptr;              /* next char to get */
     };
 
@@ -821,17 +845,17 @@ class TokenStream
     void updateFlagsForEOL();
 
     Token               tokens[ntokens];/* circular token buffer */
-    JS::SkipRoot        tokensRoot;     /* prevent overwriting of token buffer */
+    js::SkipRoot        tokensRoot;     /* prevent overwriting of token buffer */
     unsigned            cursor;         /* index of last parsed token */
     unsigned            lookahead;      /* count of lookahead tokens */
     unsigned            lineno;         /* current line number */
     unsigned            flags;          /* flags -- see above */
     const jschar        *linebase;      /* start of current line;  points into userbuf */
     const jschar        *prevLinebase;  /* start of previous line;  NULL if on the first line */
-    JS::SkipRoot        linebaseRoot;
-    JS::SkipRoot        prevLinebaseRoot;
+    js::SkipRoot        linebaseRoot;
+    js::SkipRoot        prevLinebaseRoot;
     TokenBuf            userbuf;        /* user input buffer */
-    JS::SkipRoot        userbufRoot;
+    js::SkipRoot        userbufRoot;
     const char          *filename;      /* input filename or null */
     jschar              *sourceMap;     /* source map's filename or null */
     void                *listenerTSData;/* listener data for this TokenStream */
@@ -874,6 +898,7 @@ IsIdentifier(JSLinearString *str);
  */
 #define JSREPORT_UC 0x100
 
+} /* namespace frontend */
 } /* namespace js */
 
 extern JS_FRIEND_API(int)
@@ -881,7 +906,7 @@ js_fgets(char *buf, int size, FILE *file);
 
 #ifdef DEBUG
 extern const char *
-TokenKindToString(js::TokenKind tt);
+TokenKindToString(js::frontend::TokenKind tt);
 #endif
 
 #endif /* TokenStream_h__ */
