@@ -8,11 +8,18 @@
 #include "jscntxt.h"
 #include "gc/Marking.h"
 #include "methodjit/MethodJIT.h"
+#ifdef JS_ION
+#include "ion/IonFrames.h"
+#include "ion/IonCompartment.h"
+#include "ion/Bailouts.h"
+#endif
 #include "Stack.h"
 
 #include "jsgcinlines.h"
 #include "jsobjinlines.h"
 
+#include "jsopcode.h"
+
 #include "Stack-inl.h"
 
 /* Includes to get to low-level memory-mapping functionality. */
@@ -83,16 +90,6 @@ StackFrame::initExecuteFrame(JSScript *script, StackFrame *prev, FrameRegs *regs
         setAnnotation(prev->annotation());
 }
 
-void
-StackFrame::initDummyFrame(JSContext *cx, JSObject &chain)
-{
-    PodZero(this);
-    flags_ = DUMMY | HAS_PREVPC | HAS_SCOPECHAIN;
-    initPrev(cx);
-    JS_ASSERT(chain.isGlobal());
-    scopeChain_ = &chain;
-}
-
 template <StackFrame::TriggerPostBarriers doPostBarrier>
 void
 StackFrame::copyFrameAndValues(JSContext *cx, Value *vp, StackFrame *otherfp,
@@ -143,8 +140,6 @@ StackFrame::writeBarrierPost()
     /* This needs to follow the same rules as in StackFrame::mark. */
     if (scopeChain_)
         JSObject::writeBarrierPost(scopeChain_, (void *)&scopeChain_);
-    if (isDummyFrame())
-        return;
     if (flags_ & HAS_ARGS_OBJ)
         JSObject::writeBarrierPost(argsObj_, (void *)&argsObj_);
     if (isFunctionFrame()) {
@@ -185,7 +180,7 @@ StackFrame::prevpcSlow(InlinedSite **pinlined)
     JS_ASSERT(!(flags_ & HAS_PREVPC));
 #if defined(JS_METHODJIT) && defined(JS_MONOIC)
     StackFrame *p = prev();
-    mjit::JITScript *jit = p->script()->getJIT(p->isConstructing(), p->compartment()->needsBarrier());
+    mjit::JITScript *jit = p->script()->getJIT(p->isConstructing(), p->compartment()->compileBarriers());
     prevpc_ = jit->nativeToPC(ncode_, &prevInline_);
     flags_ |= HAS_PREVPC;
     if (pinlined)
@@ -222,6 +217,16 @@ StackFrame::pcQuadratic(const ContextStack &stack, size_t maxDepth)
     return regs.fp()->script()->code;
 }
 
+bool
+StackFrame::copyRawFrameSlots(CopyVector *vec)
+{
+    if (!vec->resize(numFormalArgs() + script()->nfixed))
+        return false;
+    PodCopy(vec->begin(), formals(), numFormalArgs());
+    PodCopy(vec->begin() + numFormalArgs(), slots(), script()->nfixed);
+    return true;
+}
+
 static inline void
 AssertDynamicScopeMatchesStaticScope(JSScript *script, JSObject *scope)
 {
@@ -258,10 +263,20 @@ AssertDynamicScopeMatchesStaticScope(JSScript *script, JSObject *scope)
 #endif
 }
 
+bool
+StackFrame::initCallObject(JSContext *cx)
+{
+    CallObject *callobj = CallObject::createForFunction(cx, this);
+    if (!callobj)
+        return false;
+    pushOnScopeChain(*callobj);
+    flags_ |= HAS_CALL_OBJ;
+    return true;
+}
+
 bool
 StackFrame::prologue(JSContext *cx, bool newType)
 {
-    JS_ASSERT(!isDummyFrame());
     JS_ASSERT(!isGeneratorFrame());
     JS_ASSERT(cx->regs().pc == script()->code);
 
@@ -285,13 +300,8 @@ StackFrame::prologue(JSContext *cx, bool newType)
     JS_ASSERT(isNonEvalFunctionFrame());
     AssertDynamicScopeMatchesStaticScope(script(), scopeChain());
 
-    if (fun()->isHeavyweight()) {
-        CallObject *callobj = CallObject::createForFunction(cx, this);
-        if (!callobj)
-            return false;
-        pushOnScopeChain(*callobj);
-        flags_ |= HAS_CALL_OBJ;
-    }
+    if (fun()->isHeavyweight() && !initCallObject(cx))
+        return false;
 
     if (isConstructing()) {
         RootedObject callee(cx, &this->callee());
@@ -308,7 +318,6 @@ StackFrame::prologue(JSContext *cx, bool newType)
 void
 StackFrame::epilogue(JSContext *cx)
 {
-    JS_ASSERT(!isDummyFrame());
     JS_ASSERT(!isYielding());
     JS_ASSERT(!hasBlockChain());
 
@@ -325,7 +334,17 @@ StackFrame::epilogue(JSContext *cx)
             else
                 JS_ASSERT(scopeChain() == prev()->scopeChain());
         } else {
-            JS_ASSERT(scopeChain()->isGlobal());
+            /*
+             * Debugger.Object.prototype.evalInGlobal creates indirect eval
+             * frames scoped to the given global;
+             * Debugger.Object.prototype.evalInGlobalWithBindings creates
+             * indirect eval frames scoped to an object carrying the introduced
+             * bindings.
+             */
+            if (isDebuggerFrame())
+                JS_ASSERT(scopeChain()->isGlobal() || scopeChain()->enclosingScope()->isGlobal());
+            else
+                JS_ASSERT(scopeChain()->isGlobal());
         }
         return;
     }
@@ -421,8 +440,6 @@ StackFrame::mark(JSTracer *trc)
      */
     if (flags_ & HAS_SCOPECHAIN)
         gc::MarkObjectUnbarriered(trc, &scopeChain_, "scope chain");
-    if (isDummyFrame())
-        return;
     if (flags_ & HAS_ARGS_OBJ)
         gc::MarkObjectUnbarriered(trc, &argsObj_, "arguments");
     if (isFunctionFrame()) {
@@ -618,13 +635,6 @@ StackSpace::markAndClobberFrame(JSTracer *trc, StackFrame *fp, Value *slotsEnd,
 {
     Value *slotsBegin = fp->slots();
 
-    if (!fp->isScriptFrame()) {
-        JS_ASSERT(fp->isDummyFrame());
-        if (trc)
-            gc::MarkValueRootRange(trc, slotsBegin, slotsEnd, "vm_stack");
-        return;
-    }
-
     /* If it's a scripted frame, we should have a pc. */
     JS_ASSERT(pc);
 
@@ -680,7 +690,7 @@ StackSpace::markAndClobberFrame(JSTracer *trc, StackFrame *fp, Value *slotsEnd,
                 else if (type == JSVAL_TYPE_BOOLEAN)
                     *vp = BooleanValue(false);
                 else if (type == JSVAL_TYPE_STRING)
-                    *vp = StringValue(rt->atomState.nullAtom);
+                    *vp = StringValue(rt->atomState.null);
                 else if (type == JSVAL_TYPE_NULL)
                     *vp = NullValue();
                 else if (type == JSVAL_TYPE_OBJECT)
@@ -739,16 +749,12 @@ StackSpace::markActiveCompartments()
 }
 
 JS_FRIEND_API(bool)
-StackSpace::ensureSpaceSlow(JSContext *cx, MaybeReportError report, Value *from, ptrdiff_t nvals,
-                            JSCompartment *dest) const
+StackSpace::ensureSpaceSlow(JSContext *cx, MaybeReportError report, Value *from, ptrdiff_t nvals) const
 {
     assertInvariants();
 
-    /* See CX_COMPARTMENT comment. */
-    if (dest == (JSCompartment *)CX_COMPARTMENT)
-        dest = cx->compartment;
-
-    bool trusted = !dest || dest->principals == cx->runtime->trustedPrincipals();
+    JSCompartment *dest = cx->compartment;
+    bool trusted = dest->principals == cx->runtime->trustedPrincipals();
     Value *end = trusted ? trustedEnd_ : defaultEnd_;
 
     /*
@@ -805,11 +811,57 @@ StackSpace::tryBumpLimit(JSContext *cx, Value *from, unsigned nvals, Value **lim
 }
 
 size_t
-StackSpace::sizeOfCommitted()
+StackSpace::sizeOf()
 {
-#ifdef XP_WIN
+#if defined(XP_UNIX)
+    /*
+     * Measure how many of our pages are resident in RAM using mincore, and
+     * return that as our size.  This is slow, but hopefully nobody expects
+     * this method to be fast.
+     *
+     * Note that using mincore means that we don't count pages of the stack
+     * which are swapped out to disk.  We really should, but what we have here
+     * is better than counting the whole stack!
+     */
+
+    const int pageSize = getpagesize();
+    size_t numBytes = (trustedEnd_ - base_) * sizeof(Value);
+    size_t numPages = (numBytes + pageSize - 1) / pageSize;
+
+    // On Linux, mincore's third argument has type unsigned char*.
+#ifdef __linux__
+    typedef unsigned char MincoreArgType;
+#else
+    typedef char MincoreArgType;
+#endif
+
+    MincoreArgType *vec = (MincoreArgType *) js_malloc(numPages);
+    int result = mincore(base_, numBytes, vec);
+    if (result) {
+        js_free(vec);
+        /*
+         * If mincore fails us, return the vsize (like we do below if we're not
+         * on Windows or Unix).
+         */
+        return (trustedEnd_ - base_) * sizeof(Value);
+    }
+
+    size_t residentBytes = 0;
+    for (size_t i = 0; i < numPages; i++) {
+        /* vec[i] has its least-significant bit set iff page i is in RAM. */
+        if (vec[i] & 0x1)
+            residentBytes += pageSize;
+    }
+    js_free(vec);
+    return residentBytes;
+
+#elif defined(XP_WIN)
     return (commitEnd_ - base_) * sizeof(Value);
 #else
+    /*
+     * Return the stack's virtual size, which is at least an upper bound on its
+     * resident size.
+     */
     return (trustedEnd_ - base_) * sizeof(Value);
 #endif
 }
@@ -842,7 +894,7 @@ ContextStack::~ContextStack()
 ptrdiff_t
 ContextStack::spIndexOf(const Value *vp)
 {
-    if (!hasfp() || !fp()->isScriptFrame())
+    if (!hasfp())
         return JSDVG_SEARCH_STACK;
 
     Value *base = fp()->base();
@@ -880,9 +932,10 @@ ContextStack::containsSlow(const StackFrame *target) const
  */
 Value *
 ContextStack::ensureOnTop(JSContext *cx, MaybeReportError report, unsigned nvars,
-                          MaybeExtend extend, bool *pushedSeg, JSCompartment *dest)
+                          MaybeExtend extend, bool *pushedSeg)
 {
     Value *firstUnused = space().firstUnused();
+    FrameRegs *regs = cx->maybeRegs();
 
 #ifdef JS_METHODJIT
     /*
@@ -893,9 +946,13 @@ ContextStack::ensureOnTop(JSContext *cx, MaybeReportError report, unsigned nvars
      * To avoid pathological behavior here, make sure to mark any topmost
      * function as uninlineable, which will expand inline frames if there are
      * any and prevent the function from being inlined in the future.
+     *
+     * Note: When called from pushBailoutFrame, error = DONT_REPORT_ERROR. Use
+     * this to deny potential invalidation, which would read from
+     * runtime->ionTop.
      */
-    if (FrameRegs *regs = cx->maybeRegs()) {
-        JSFunction *fun = NULL;
+    if (regs && report != DONT_REPORT_ERROR) {
+        RootedFunction fun(cx);
         if (InlinedSite *site = regs->inlined()) {
             mjit::JITChunk *chunk = regs->fp()->jit()->chunk(regs->pc);
             fun = chunk->inlineFrames()[site->inlineIndex].fun;
@@ -909,6 +966,7 @@ ContextStack::ensureOnTop(JSContext *cx, MaybeReportError report, unsigned nvars
         }
 
         if (fun) {
+            AutoCompartment ac(cx, fun);
             fun->script()->uninlineable = true;
             types::MarkTypeObjectFlags(cx, fun, types::OBJECT_FLAG_UNINLINEABLE);
         }
@@ -917,15 +975,14 @@ ContextStack::ensureOnTop(JSContext *cx, MaybeReportError report, unsigned nvars
 #endif
 
     if (onTop() && extend) {
-        if (!space().ensureSpace(cx, report, firstUnused, nvars, dest))
+        if (!space().ensureSpace(cx, report, firstUnused, nvars))
             return NULL;
         return firstUnused;
     }
 
-    if (!space().ensureSpace(cx, report, firstUnused, VALUES_PER_STACK_SEGMENT + nvars, dest))
+    if (!space().ensureSpace(cx, report, firstUnused, VALUES_PER_STACK_SEGMENT + nvars))
         return NULL;
 
-    FrameRegs *regs;
     CallArgsList *calls;
     if (seg_ && extend) {
         regs = seg_->maybeRegs();
@@ -952,12 +1009,13 @@ ContextStack::popSegment()
 }
 
 bool
-ContextStack::pushInvokeArgs(JSContext *cx, unsigned argc, InvokeArgsGuard *iag)
+ContextStack::pushInvokeArgs(JSContext *cx, unsigned argc, InvokeArgsGuard *iag,
+                             MaybeReportError report)
 {
     JS_ASSERT(argc <= StackSpace::ARGS_LENGTH_MAX);
 
     unsigned nvars = 2 + argc;
-    Value *firstUnused = ensureOnTop(cx, REPORT_ERROR, nvars, CAN_EXTEND, &iag->pushedSeg_);
+    Value *firstUnused = ensureOnTop(cx, report, nvars, CAN_EXTEND, &iag->pushedSeg_);
     if (!firstUnused)
         return false;
 
@@ -987,28 +1045,38 @@ ContextStack::popInvokeArgs(const InvokeArgsGuard &iag)
     Debug_SetValueRangeToCrashOnTouch(space().firstUnused(), oldend);
 }
 
-bool
-ContextStack::pushInvokeFrame(JSContext *cx, const CallArgs &args,
-                              InitialFrameFlags initial, InvokeFrameGuard *ifg)
+StackFrame *
+ContextStack::pushInvokeFrame(JSContext *cx, MaybeReportError report,
+                              const CallArgs &args, JSFunction *fun,
+                              InitialFrameFlags initial, FrameGuard *fg)
 {
     JS_ASSERT(onTop());
     JS_ASSERT(space().firstUnused() == args.end());
 
-    JSObject &callee = args.callee();
-    JSFunction *fun = callee.toFunction();
     JSScript *script = fun->script();
 
     StackFrame::Flags flags = ToFrameFlags(initial);
-    StackFrame *fp = getCallFrame(cx, REPORT_ERROR, args, fun, script, &flags);
+    StackFrame *fp = getCallFrame(cx, report, args, fun, script, &flags);
     if (!fp)
-        return false;
+        return NULL;
 
     fp->initCallFrame(cx, *fun, script, args.length(), flags);
-    ifg->regs_.prepareToRun(*fp, script);
+    fg->regs_.prepareToRun(*fp, script);
 
-    ifg->prevRegs_ = seg_->pushRegs(ifg->regs_);
-    JS_ASSERT(space().firstUnused() == ifg->regs_.sp);
-    ifg->setPushed(*this);
+    fg->prevRegs_ = seg_->pushRegs(fg->regs_);
+    JS_ASSERT(space().firstUnused() == fg->regs_.sp);
+    fg->setPushed(*this);
+    return fp;
+}
+
+bool
+ContextStack::pushInvokeFrame(JSContext *cx, const CallArgs &args,
+                              InitialFrameFlags initial, InvokeFrameGuard *ifg)
+{
+    JSObject &callee = args.callee();
+    JSFunction *fun = callee.toFunction();
+    if (!pushInvokeFrame(cx, REPORT_ERROR, args, fun, initial, ifg))
+        return false;
     return true;
 }
 
@@ -1064,27 +1132,35 @@ ContextStack::pushExecuteFrame(JSContext *cx, JSScript *script, const Value &thi
     return true;
 }
 
+#ifdef JS_ION
 bool
-ContextStack::pushDummyFrame(JSContext *cx, JSCompartment *dest, JSObject &scopeChain, DummyFrameGuard *dfg)
+ContextStack::pushBailoutArgs(JSContext *cx, const ion::IonBailoutIterator &it, InvokeArgsGuard *iag)
 {
-    JS_ASSERT(dest == scopeChain.compartment());
+    unsigned argc = it.numActualArgs();
+    ion::SnapshotIterator s(it);
 
-    unsigned nvars = VALUES_PER_STACK_FRAME;
-    Value *firstUnused = ensureOnTop(cx, REPORT_ERROR, nvars, CAN_EXTEND, &dfg->pushedSeg_, dest);
-    if (!firstUnused)
+    if (!pushInvokeArgs(cx, argc, iag, DONT_REPORT_ERROR))
         return false;
 
-    StackFrame *fp = reinterpret_cast<StackFrame *>(firstUnused);
-    fp->initDummyFrame(cx, scopeChain);
-    dfg->regs_.initDummyFrame(*fp);
+    JSFunction *fun = it.callee();
+    iag->setCallee(ObjectValue(*fun));
 
-    cx->setCompartment(dest);
-    dfg->prevRegs_ = seg_->pushRegs(dfg->regs_);
-    JS_ASSERT(space().firstUnused() == dfg->regs_.sp);
-    dfg->setPushed(*this);
+    CopyTo dst(iag->array());
+    Value *src = it.actualArgs();
+    Value thisv = iag->thisv();
+    s.readFrameArgs(dst, src, NULL, &thisv, 0, fun->nargs, argc);
     return true;
 }
 
+StackFrame *
+ContextStack::pushBailoutFrame(JSContext *cx, const ion::IonBailoutIterator &it,
+                               const CallArgs &args, BailoutFrameGuard *bfg)
+{
+    JSFunction *fun = it.callee();
+    return pushInvokeFrame(cx, DONT_REPORT_ERROR, args, fun, INITIAL_NONE, bfg);
+}
+#endif
+
 void
 ContextStack::popFrame(const FrameGuard &fg)
 {
@@ -1100,13 +1176,6 @@ ContextStack::popFrame(const FrameGuard &fg)
         popSegment();
 
     Debug_SetValueRangeToCrashOnTouch(space().firstUnused(), oldend);
-
-    /*
-     * NB: this code can call out and observe the stack (e.g., through GC), so
-     * it should only be called from a consistent stack state.
-     */
-    if (!hasfp())
-        cx_->resetCompartment();
 }
 
 bool
@@ -1181,27 +1250,25 @@ ContextStack::popGeneratorFrame(const GeneratorFrameGuard &gfg)
 bool
 ContextStack::saveFrameChain()
 {
-    JSCompartment *dest = NULL;
-
     bool pushedSeg;
-    if (!ensureOnTop(cx_, REPORT_ERROR, 0, CANT_EXTEND, &pushedSeg, dest))
+    if (!ensureOnTop(cx_, REPORT_ERROR, 0, CANT_EXTEND, &pushedSeg))
         return false;
 
     JS_ASSERT(pushedSeg);
     JS_ASSERT(!hasfp());
-    JS_ASSERT(onTop() && seg_->isEmpty());
-
-    cx_->resetCompartment();
+    JS_ASSERT(onTop());
+    JS_ASSERT(seg_->isEmpty());
     return true;
 }
 
 void
 ContextStack::restoreFrameChain()
 {
-    JS_ASSERT(onTop() && seg_->isEmpty());
+    JS_ASSERT(!hasfp());
+    JS_ASSERT(onTop());
+    JS_ASSERT(seg_->isEmpty());
 
     popSegment();
-    cx_->resetCompartment();
 }
 
 /*****************************************************************************/
@@ -1219,11 +1286,12 @@ StackIter::popFrame()
     StackFrame *oldfp = fp_;
     JS_ASSERT(seg_->contains(oldfp));
     fp_ = fp_->prev();
+
     if (seg_->contains(fp_)) {
         InlinedSite *inline_;
         pc_ = oldfp->prevpc(&inline_);
         JS_ASSERT(!inline_);
-        script_ = fp_->maybeScript();
+        script_ = fp_->script();
     } else {
         poisonRegs();
     }
@@ -1245,7 +1313,7 @@ StackIter::settleOnNewSegment()
     if (FrameRegs *regs = seg_->maybeRegs()) {
         pc_ = regs->pc;
         if (fp_)
-            script_ = fp_->maybeScript();
+            script_ = fp_->script();
     } else {
         poisonRegs();
     }
@@ -1328,11 +1396,33 @@ StackIter::settleOnNewState()
          * ordering to decide which was the most recent.
          */
         if (containsFrame && (!containsCall || (Value *)fp_ >= calls_->array())) {
-            /* Nobody wants to see dummy frames. */
-            if (fp_->isDummyFrame()) {
-                popFrame();
-                continue;
+#ifdef JS_ION
+            if (fp_->beginsIonActivation()) {
+                ionFrames_ = ion::IonFrameIterator(ionActivations_);
+
+                if (ionFrames_.isNative()) {
+                    state_ = ION;
+                    return;
+                }
+
+                while (!ionFrames_.isScripted() && !ionFrames_.done())
+                    ++ionFrames_;
+
+                // When invoked from JM, we don't re-use the entryfp, so we
+                // may have an empty Ion activation.
+                if (ionFrames_.done()) {
+                    state_ = SCRIPTED;
+                    script_ = fp_->script();
+                    return;
+                }
+
+                state_ = ION;
+                ionInlineFrames_ = ion::InlineFrameIterator(&ionFrames_);
+                pc_ = ionInlineFrames_.pc();
+                script_ = ionInlineFrames_.script();
+                return;
             }
+#endif /* JS_ION */
 
             state_ = SCRIPTED;
             script_ = fp_->script();
@@ -1363,6 +1453,11 @@ StackIter::settleOnNewState()
 StackIter::StackIter(JSContext *cx, SavedOption savedOption)
   : maybecx_(cx),
     savedOption_(savedOption)
+#ifdef JS_ION
+    , ionActivations_(cx),
+    ionFrames_((uint8_t *)NULL),
+    ionInlineFrames_((js::ion::IonFrameIterator*) NULL)
+#endif
 {
 #ifdef JS_METHODJIT
     CompartmentVector &v = cx->runtime->compartments;
@@ -1380,6 +1475,11 @@ StackIter::StackIter(JSContext *cx, SavedOption savedOption)
 
 StackIter::StackIter(JSRuntime *rt, StackSegment &seg)
   : maybecx_(NULL), savedOption_(STOP_AT_SAVED)
+#ifdef JS_ION
+    , ionActivations_(rt),
+    ionFrames_((uint8_t *)NULL),
+    ionInlineFrames_((js::ion::IonFrameIterator*) NULL)
+#endif
 {
 #ifdef JS_METHODJIT
     CompartmentVector &v = rt->compartments;
@@ -1390,6 +1490,54 @@ StackIter::StackIter(JSRuntime *rt, StackSegment &seg)
     settleOnNewState();
 }
 
+#ifdef JS_ION
+void
+StackIter::popIonFrame()
+{
+    // Keep fp which describes all ion frames.
+    poisonRegs();
+    if (ionFrames_.isScripted() && ionInlineFrames_.more()) {
+        ++ionInlineFrames_;
+        pc_ = ionInlineFrames_.pc();
+        script_ = ionInlineFrames_.script();
+    } else {
+        ++ionFrames_;
+        while (!ionFrames_.done() && !ionFrames_.isScripted())
+            ++ionFrames_;
+
+        if (!ionFrames_.done()) {
+            ionInlineFrames_ = ion::InlineFrameIterator(&ionFrames_);
+            pc_ = ionInlineFrames_.pc();
+            script_ = ionInlineFrames_.script();
+            return;
+        }
+
+        // The activation has no other frames. If entryfp is NULL, it was invoked
+        // by a native written in C++, using FastInvoke, on top of another activation.
+        ion::IonActivation *activation = ionActivations_.activation();
+        if (!activation->entryfp()) {
+            JS_ASSERT(activation->prevpc());
+            JS_ASSERT(fp_->beginsIonActivation());
+            ++ionActivations_;
+            settleOnNewState();
+            return;
+        }
+
+        if (fp_->runningInIon()) {
+            ++ionActivations_;
+            popFrame();
+            settleOnNewState();
+        } else {
+            JS_ASSERT(fp_->callingIntoIon());
+            state_ = SCRIPTED;
+            script_ = fp_->script();
+            pc_ = ionActivations_.activation()->prevpc();
+            ++ionActivations_;
+        }
+    }
+}
+#endif
+
 StackIter &
 StackIter::operator++()
 {
@@ -1404,6 +1552,13 @@ StackIter::operator++()
         popCall();
         settleOnNewState();
         break;
+      case ION:
+#ifdef JS_ION      
+        popIonFrame();
+        break;
+#else
+        JS_NOT_REACHED("Unexpected state");
+#endif
     }
     return *this;
 }
@@ -1426,6 +1581,12 @@ StackIter::isFunctionFrame() const
         break;
       case SCRIPTED:
         return fp()->isFunctionFrame();
+      case ION:
+#ifdef  JS_ION    
+        return ionInlineFrames_.isFunctionFrame();
+#else
+        break;
+#endif
       case NATIVE:
         return false;
     }
@@ -1441,6 +1602,7 @@ StackIter::isEvalFrame() const
         break;
       case SCRIPTED:
         return fp()->isEvalFrame();
+      case ION:
       case NATIVE:
         return false;
     }
@@ -1457,6 +1619,7 @@ StackIter::isNonEvalFunctionFrame() const
         break;
       case SCRIPTED:
         return fp()->isNonEvalFunctionFrame();
+      case ION:
       case NATIVE:
         return !isEvalFrame() && isFunctionFrame();
     }
@@ -1471,6 +1634,13 @@ StackIter::isConstructing() const
       case DONE:
         JS_NOT_REACHED("Unexpected state");
         return false;
+      case ION:
+#ifdef JS_ION      
+        return ionInlineFrames_.isConstructing();
+#else
+        JS_NOT_REACHED("Unexpected state");
+        return false;
+#endif        
       case SCRIPTED:
       case NATIVE:
         return fp()->isConstructing();
@@ -1487,6 +1657,15 @@ StackIter::callee() const
       case SCRIPTED:
         JS_ASSERT(isFunctionFrame());
         return &fp()->callee();
+      case ION:
+#ifdef JS_ION      
+        if (ionFrames_.isScripted())
+            return ionInlineFrames_.callee();
+        JS_ASSERT(ionFrames_.isNative());
+        return ionFrames_.callee();
+#else
+        break;
+#endif        
       case NATIVE:
         return nativeArgs().callee().toFunction();
     }
@@ -1503,6 +1682,12 @@ StackIter::calleev() const
       case SCRIPTED:
         JS_ASSERT(isFunctionFrame());
         return fp()->calleev();
+      case ION:
+#ifdef JS_ION
+        return ObjectValue(*callee());
+#else
+        break;
+#endif
       case NATIVE:
         return nativeArgs().calleev();
     }
@@ -1510,19 +1695,92 @@ StackIter::calleev() const
     return Value();
 }
 
+unsigned
+StackIter::numActualArgs() const
+{
+    switch (state_) {
+      case DONE:
+        break;
+      case SCRIPTED:
+        JS_ASSERT(isFunctionFrame());
+        return fp()->numActualArgs();
+      case ION:
+#ifdef JS_ION
+        return ionInlineFrames_.numActualArgs();
+#else
+        break;
+#endif
+      case NATIVE:
+        return nativeArgs().length();
+    }
+    JS_NOT_REACHED("Unexpected state");
+    return 0;
+}
+
 Value
 StackIter::thisv() const
 {
     switch (state_) {
       case DONE:
-        MOZ_NOT_REACHED("Unexpected state");
-        return Value();
+        break;
+      case ION:
+#ifdef JS_ION
+        return ObjectValue(*ionInlineFrames_.thisObject());
+#else
+        break;
+#endif
       case SCRIPTED:
       case NATIVE:
         return fp()->thisValue();
     }
-    MOZ_NOT_REACHED("unexpected state");
-    return Value();
+    JS_NOT_REACHED("Unexpected state");
+    return NullValue();
+}
+
+size_t
+StackIter::numFrameSlots() const
+{
+    switch (state_) {
+      case DONE:
+      case NATIVE:
+        break;
+      case ION:
+#ifdef JS_ION
+        return ionInlineFrames_.snapshotIterator().slots() - ionInlineFrames_.script()->nfixed;
+#else
+        break;
+#endif
+      case SCRIPTED:
+        JS_ASSERT(maybecx_);
+        JS_ASSERT(maybecx_->regs().spForStackDepth(0) == fp()->base());
+        return maybecx_->regs().sp - fp()->base();
+    }
+    JS_NOT_REACHED("Unexpected state");
+    return 0;
+}
+
+Value
+StackIter::frameSlotValue(size_t index) const
+{
+    switch (state_) {
+      case DONE:
+      case NATIVE:
+        break;
+      case ION:
+#ifdef JS_ION
+      {
+        ion::SnapshotIterator si(ionInlineFrames_.snapshotIterator());
+        index += ionInlineFrames_.script()->nfixed;
+        return si.maybeReadSlotByIndex(index);
+      }
+#else
+        break;
+#endif
+      case SCRIPTED:
+          return fp()->base()[index];
+    }
+    JS_NOT_REACHED("Unexpected state");
+    return NullValue();
 }
 
 /*****************************************************************************/
