@@ -17,71 +17,87 @@
 using namespace mozilla;
 using namespace js;
 
+#ifdef DEBUG
 bool
 JSString::isShort() const
 {
     bool is_short = (getAllocKind() == gc::FINALIZE_SHORT_STRING);
-    JS_ASSERT_IF(is_short, isFixed());
+    JS_ASSERT_IF(is_short, isFlat());
     return is_short;
 }
-
-bool
-JSString::isFixed() const
-{
-    return isFlat() && !isExtensible();
-}
-
-bool
-JSString::isInline() const
-{
-    return isFixed() && (d.u1.chars == d.inlineStorage || isShort());
-}
+#endif
 
 bool
 JSString::isExternal() const
 {
     bool is_external = (getAllocKind() == gc::FINALIZE_EXTERNAL_STRING);
-    JS_ASSERT_IF(is_external, isFixed());
+    JS_ASSERT_IF(is_external, isFlat());
     return is_external;
 }
 
 size_t
 JSString::sizeOfExcludingThis(JSMallocSizeOfFun mallocSizeOf)
 {
-    /* JSRope: do nothing, we'll count all children chars when we hit the leaf strings. */
+    // JSRope: do nothing, we'll count all children chars when we hit the leaf strings.
     if (isRope())
         return 0;
 
     JS_ASSERT(isLinear());
 
-    /* JSDependentString: do nothing, we'll count the chars when we hit the base string. */
+    // JSDependentString: do nothing, we'll count the chars when we hit the base string.
     if (isDependent())
         return 0;
 
     JS_ASSERT(isFlat());
 
-    /* JSExtensibleString: count the full capacity, not just the used space. */
+    // JSExtensibleString: count the full capacity, not just the used space.
     if (isExtensible()) {
         JSExtensibleString &extensible = asExtensible();
         return mallocSizeOf(extensible.chars());
     }
 
-    JS_ASSERT(isFixed());
-
-    /* JSExternalString: don't count, the chars could be stored anywhere. */
+    // JSExternalString: don't count, the chars could be stored anywhere.
     if (isExternal())
         return 0;
 
-    /* JSInlineString, JSShortString, JSInlineAtom, JSShortAtom: the chars are inline. */
+    // JSInlineString, JSShortString [JSInlineAtom, JSShortAtom]: the chars are inline.
     if (isInline())
         return 0;
 
-    /* JSAtom, JSFixedString: count the chars. +1 for the null char. */
-    JSFixedString &fixed = asFixed();
-    return mallocSizeOf(fixed.chars());
+    // JSAtom, JSStableString, JSUndependedString: measure the space for the
+    // chars.  For JSUndependedString, there is no need to count the base
+    // string, for the same reason as JSDependentString above.
+    JSFlatString &flat = asFlat();
+    return mallocSizeOf(flat.chars());
 }
 
 #ifdef DEBUG
+
+void
+JSString::dumpChars(const jschar *s, size_t n)
+{
+    if (n == SIZE_MAX) {
+        n = 0;
+        while (s[n])
+            n++;
+    }
+
+    fputc('"', stderr);
+    for (size_t i = 0; i < n; i++) {
+        if (s[i] == '\n')
+            fprintf(stderr, "\\n");
+        else if (s[i] == '\t')
+            fprintf(stderr, "\\t");
+        else if (s[i] >= 32 && s[i] < 127)
+            fputc(s[i], stderr);
+        else if (s[i] <= 255)
+            fprintf(stderr, "\\x%02x", (unsigned int) s[i]);
+        else
+            fprintf(stderr, "\\u%04x", (unsigned int) s[i]);
+    }
+    fputc('"', stderr);
+}
+
 void
 JSString::dump()
 {
@@ -90,7 +106,7 @@ JSString::dump()
                 (void *) this, (void *) chars);
 
         extern void DumpChars(const jschar *s, size_t n);
-        DumpChars(chars, length());
+        dumpChars(chars, length());
     } else {
         fprintf(stderr, "(oom in JSString::dump)");
     }
@@ -138,7 +154,7 @@ AllocChars(JSContext *maybecx, size_t length, jschar **chars, size_t *capacity)
 
     JS_STATIC_ASSERT(JSString::MAX_LENGTH * sizeof(jschar) < UINT32_MAX);
     size_t bytes = numChars * sizeof(jschar);
-    *chars = (jschar *)(maybecx ? maybecx->malloc_(bytes) : OffTheBooks::malloc_(bytes));
+    *chars = (jschar *)(maybecx ? maybecx->malloc_(bytes) : js_malloc(bytes));
     return *chars != NULL;
 }
 
@@ -183,6 +199,7 @@ JSRope::flattenInternal(JSContext *maybecx)
     jschar *wholeChars;
     JSString *str = this;
     jschar *pos;
+    JSCompartment *comp = compartment();
 
     if (this->leftChild()->isExtensible()) {
         JSExtensibleString &left = this->leftChild()->asExtensible();
@@ -200,7 +217,8 @@ JSRope::flattenInternal(JSContext *maybecx)
             JS_STATIC_ASSERT(!(EXTENSIBLE_FLAGS & DEPENDENT_FLAGS));
             left.d.lengthAndFlags = bits ^ (EXTENSIBLE_FLAGS | DEPENDENT_FLAGS);
             left.d.s.u2.base = (JSLinearString *)this;  /* will be true on exit */
-            JSString::writeBarrierPost(left.d.s.u2.base, &left.d.s.u2.base);
+            StringWriteBarrierPostRemove(comp, &left.d.u1.left);
+            StringWriteBarrierPost(comp, (JSString **)&left.d.s.u2.base);
             goto visit_right_child;
         }
     }
@@ -217,6 +235,7 @@ JSRope::flattenInternal(JSContext *maybecx)
 
         JSString &left = *str->d.u1.left;
         str->d.u1.chars = pos;
+        StringWriteBarrierPostRemove(comp, &str->d.u1.left);
         if (left.isRope()) {
             left.d.s.u3.parent = str;          /* Return to this when 'left' done, */
             left.d.lengthAndFlags = 0x200;     /* but goto visit_right_child. */
@@ -246,12 +265,14 @@ JSRope::flattenInternal(JSContext *maybecx)
             str->d.lengthAndFlags = buildLengthAndFlags(wholeLength, EXTENSIBLE_FLAGS);
             str->d.u1.chars = wholeChars;
             str->d.s.u2.capacity = wholeCapacity;
+            StringWriteBarrierPostRemove(comp, &str->d.u1.left);
+            StringWriteBarrierPostRemove(comp, &str->d.s.u2.right);
             return &this->asFlat();
         }
         size_t progress = str->d.lengthAndFlags;
         str->d.lengthAndFlags = buildLengthAndFlags(pos - str->d.u1.chars, DEPENDENT_FLAGS);
         str->d.s.u2.base = (JSLinearString *)this;       /* will be true on exit */
-        JSString::writeBarrierPost(str->d.s.u2.base, &str->d.s.u2.base);
+        StringWriteBarrierPost(comp, (JSString **)&str->d.s.u2.base);
         str = str->d.s.u3.parent;
         if (progress == 0x200)
             goto visit_right_child;
@@ -273,7 +294,7 @@ JSRope::flatten(JSContext *maybecx)
 #endif
 }
 
-JSString * JS_FASTCALL
+JSString *
 js_ConcatStrings(JSContext *cx, HandleString left, HandleString right)
 {
     JS_ASSERT_IF(!left->isAtom(), left->compartment() == cx->compartment);
@@ -312,7 +333,7 @@ js_ConcatStrings(JSContext *cx, HandleString left, HandleString right)
     return JSRope::new_(cx, left, right, wholeLength);
 }
 
-JSFixedString *
+JSFlatString *
 JSDependentString::undepend(JSContext *cx)
 {
     JS_ASSERT(JSString::isDependent());
@@ -340,7 +361,22 @@ JSDependentString::undepend(JSContext *cx)
      */
     d.lengthAndFlags = buildLengthAndFlags(n, UNDEPENDED_FLAGS);
 
-    return &this->asFixed();
+    return &this->asFlat();
+}
+
+JSStableString *
+JSInlineString::uninline(JSContext *maybecx)
+{
+    JS_ASSERT(isInline());
+    size_t n = length();
+    jschar *news = maybecx ? maybecx->pod_malloc<jschar>(n + 1) : js_pod_malloc<jschar>(n + 1);
+    if (!news)
+        return NULL;
+    js_strncpy(news, d.inlineStorage, n);
+    news[n] = 0;
+    d.u1.chars = news;
+    JS_ASSERT(!isInline());
+    return &asStable();
 }
 
 bool
@@ -429,11 +465,11 @@ const StaticStrings::SmallChar StaticStrings::toSmallChar[] = { R7(0) };
 bool
 StaticStrings::init(JSContext *cx)
 {
-    SwitchToCompartment sc(cx, cx->runtime->atomsCompartment);
+    AutoEnterAtomsCompartment ac(cx);
 
     for (uint32_t i = 0; i < UNIT_STATIC_LIMIT; i++) {
         jschar buffer[] = { jschar(i), '\0' };
-        JSFixedString *s = js_NewStringCopyN(cx, buffer, 1);
+        JSFlatString *s = js_NewStringCopyN(cx, buffer, 1);
         if (!s)
             return false;
         unitStaticTable[i] = s->morphAtomizedStringIntoAtom();
@@ -441,7 +477,7 @@ StaticStrings::init(JSContext *cx)
 
     for (uint32_t i = 0; i < NUM_SMALL_CHARS * NUM_SMALL_CHARS; i++) {
         jschar buffer[] = { FROM_SMALL_CHAR(i >> 6), FROM_SMALL_CHAR(i & 0x3F), '\0' };
-        JSFixedString *s = js_NewStringCopyN(cx, buffer, 2);
+        JSFlatString *s = js_NewStringCopyN(cx, buffer, 2);
         if (!s)
             return false;
         length2StaticTable[i] = s->morphAtomizedStringIntoAtom();
@@ -459,7 +495,7 @@ StaticStrings::init(JSContext *cx)
                                 jschar('0' + ((i / 10) % 10)),
                                 jschar('0' + (i % 10)),
                                 '\0' };
-            JSFixedString *s = js_NewStringCopyN(cx, buffer, 3);
+            JSFlatString *s = js_NewStringCopyN(cx, buffer, 3);
             if (!s)
                 return false;
             intStaticTable[i] = s->morphAtomizedStringIntoAtom();
