@@ -39,7 +39,7 @@
 // The bidi indicator hangs off the caret to one side, to show which
 // direction the typing is in. It needs to be at least 2x2 to avoid looking like 
 // an insignificant dot
-static const PRInt32 kMinBidiIndicatorPixels = 2;
+static const int32_t kMinBidiIndicatorPixels = 2;
 
 #ifdef IBMBIDI
 #include "nsIBidiKeyboard.h"
@@ -62,7 +62,7 @@ CheckForTrailingTextFrameRecursive(nsIFrame* aFrame, nsIFrame* aStopAtFrame)
        (static_cast<nsTextFrame*>(aFrame))->IsAtEndOfLine())))
     return aFrame;
   if (!aFrame->IsFrameOfType(nsIFrame::eLineParticipant))
-    return nsnull;
+    return nullptr;
 
   for (nsIFrame* f = aFrame->GetFirstPrincipalChild(); f; f = f->GetNextSibling())
   {
@@ -70,7 +70,7 @@ CheckForTrailingTextFrameRecursive(nsIFrame* aFrame, nsIFrame* aStopAtFrame)
     if (r)
       return r;
   }
-  return nsnull;
+  return nullptr;
 }
 
 static nsLineBox*
@@ -84,20 +84,20 @@ FindContainingLine(nsIFrame* aFrame)
     {
       bool isValid;
       nsBlockInFlowLineIterator iter(blockParent, aFrame, &isValid);
-      return isValid ? iter.GetLine().get() : nsnull;
+      return isValid ? iter.GetLine().get() : nullptr;
     }
     aFrame = parent;
   }
-  return nsnull;
+  return nullptr;
 }
 
 static void
-AdjustCaretFrameForLineEnd(nsIFrame** aFrame, PRInt32* aOffset)
+AdjustCaretFrameForLineEnd(nsIFrame** aFrame, int32_t* aOffset)
 {
   nsLineBox* line = FindContainingLine(*aFrame);
   if (!line)
     return;
-  PRInt32 count = line->GetChildCount();
+  int32_t count = line->GetChildCount();
   for (nsIFrame* f = line->mFirstChild; count > 0; --count, f = f->GetNextSibling())
   {
     nsIFrame* r = CheckForTrailingTextFrameRecursive(f, *aFrame);
@@ -116,7 +116,7 @@ AdjustCaretFrameForLineEnd(nsIFrame** aFrame, PRInt32* aOffset)
 //-----------------------------------------------------------------------------
 
 nsCaret::nsCaret()
-: mPresShell(nsnull)
+: mPresShell(nullptr)
 , mBlinkRate(500)
 , mVisible(false)
 , mDrawn(false)
@@ -153,7 +153,7 @@ nsresult nsCaret::Init(nsIPresShell *inPresShell)
   mCaretAspectRatio =
     LookAndFeel::GetFloat(LookAndFeel::eFloatID_CaretAspectRatio, 0.0f);
 
-  mBlinkRate = static_cast<PRUint32>(
+  mBlinkRate = static_cast<uint32_t>(
     LookAndFeel::GetInt(LookAndFeel::eIntID_CaretBlinkTime, mBlinkRate));
   mShowDuringSelection =
     LookAndFeel::GetInt(LookAndFeel::eIntID_ShowCaretDuringSelection,
@@ -192,19 +192,19 @@ nsresult nsCaret::Init(nsIPresShell *inPresShell)
 }
 
 static bool
-DrawCJKCaret(nsIFrame* aFrame, PRInt32 aOffset)
+DrawCJKCaret(nsIFrame* aFrame, int32_t aOffset)
 {
   nsIContent* content = aFrame->GetContent();
   const nsTextFragment* frag = content->GetText();
   if (!frag)
     return false;
-  if (aOffset < 0 || PRUint32(aOffset) >= frag->GetLength())
+  if (aOffset < 0 || uint32_t(aOffset) >= frag->GetLength())
     return false;
   PRUnichar ch = frag->CharAt(aOffset);
   return 0x2e80 <= ch && ch <= 0xd7ff;
 }
 
-nsCaret::Metrics nsCaret::ComputeMetrics(nsIFrame* aFrame, PRInt32 aOffset, nscoord aCaretHeight)
+nsCaret::Metrics nsCaret::ComputeMetrics(nsIFrame* aFrame, int32_t aOffset, nscoord aCaretHeight)
 {
   // Compute nominal sizes in appunits
   nscoord caretWidth = (aCaretHeight * mCaretAspectRatio) +
@@ -218,7 +218,7 @@ nsCaret::Metrics nsCaret::ComputeMetrics(nsIFrame* aFrame, PRInt32 aOffset, nsco
 
   // Round them to device pixels. Always round down, except that anything
   // between 0 and 1 goes up to 1 so we don't let the caret disappear.
-  PRUint32 tpp = aFrame->PresContext()->AppUnitsPerDevPixel();
+  uint32_t tpp = aFrame->PresContext()->AppUnitsPerDevPixel();
   Metrics result;
   result.mCaretWidth = NS_ROUND_BORDER_TO_PIXELS(caretWidth, tpp);
   result.mBidiIndicatorSize = NS_ROUND_BORDER_TO_PIXELS(bidiIndicatorSize, tpp);
@@ -232,17 +232,17 @@ void nsCaret::Terminate()
   // a good drawing environment during teardown.
   
   KillTimer();
-  mBlinkTimer = nsnull;
+  mBlinkTimer = nullptr;
 
   // unregiser ourselves as a selection listener
   nsCOMPtr<nsISelection> domSelection = do_QueryReferent(mDomSelectionWeak);
   nsCOMPtr<nsISelectionPrivate> privateSelection(do_QueryInterface(domSelection));
   if (privateSelection)
     privateSelection->RemoveSelectionListener(this);
-  mDomSelectionWeak = nsnull;
-  mPresShell = nsnull;
+  mDomSelectionWeak = nullptr;
+  mPresShell = nullptr;
 
-  mLastContent = nsnull;
+  mLastContent = nullptr;
 }
 
 //-----------------------------------------------------------------------------
@@ -302,7 +302,7 @@ void nsCaret::SetCaretReadOnly(bool inMakeReadonly)
 
 nsresult
 nsCaret::GetGeometryForFrame(nsIFrame* aFrame,
-                             PRInt32   aFrameOffset,
+                             int32_t   aFrameOffset,
                              nsRect*   aRect,
                              nscoord*  aBidiIndicatorSize)
 {
@@ -358,28 +358,28 @@ nsIFrame* nsCaret::GetGeometry(nsISelection* aSelection, nsRect* aRect,
   nsCOMPtr<nsIDOMNode> focusNode;
   nsresult rv = aSelection->GetFocusNode(getter_AddRefs(focusNode));
   if (NS_FAILED(rv) || !focusNode)
-    return nsnull;
+    return nullptr;
 
-  PRInt32 focusOffset;
+  int32_t focusOffset;
   rv = aSelection->GetFocusOffset(&focusOffset);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
     
   nsCOMPtr<nsIContent> contentNode = do_QueryInterface(focusNode);
   if (!contentNode)
-    return nsnull;
+    return nullptr;
 
   nsRefPtr<nsFrameSelection> frameSelection = GetFrameSelection();
   if (!frameSelection)
-    return nsnull;
-  PRUint8 bidiLevel = frameSelection->GetCaretBidiLevel();
+    return nullptr;
+  uint8_t bidiLevel = frameSelection->GetCaretBidiLevel();
   nsIFrame* frame;
-  PRInt32 frameOffset;
+  int32_t frameOffset;
   rv = GetCaretFrameForNodeOffset(contentNode, focusOffset,
                                   frameSelection->GetHint(), bidiLevel,
                                   &frame, &frameOffset);
   if (NS_FAILED(rv) || !frame)
-    return nsnull;
+    return nullptr;
 
   GetGeometryForFrame(frame, frameOffset, aRect, aBidiIndicatorSize);
   return frame;
@@ -418,7 +418,7 @@ void nsCaret::SetVisibilityDuringSelection(bool aVisibility)
 }
 
 static
-nsFrameSelection::HINT GetHintForPosition(nsIDOMNode* aNode, PRInt32 aOffset)
+nsFrameSelection::HINT GetHintForPosition(nsIDOMNode* aNode, int32_t aOffset)
 {
   nsFrameSelection::HINT hint = nsFrameSelection::HINTLEFT;
   nsCOMPtr<nsIContent> node = do_QueryInterface(aNode);
@@ -433,11 +433,11 @@ nsFrameSelection::HINT GetHintForPosition(nsIDOMNode* aNode, PRInt32 aOffset)
   return hint;
 }
 
-nsresult nsCaret::DrawAtPosition(nsIDOMNode* aNode, PRInt32 aOffset)
+nsresult nsCaret::DrawAtPosition(nsIDOMNode* aNode, int32_t aOffset)
 {
   NS_ENSURE_ARG(aNode);
 
-  PRUint8 bidiLevel;
+  uint8_t bidiLevel;
   nsRefPtr<nsFrameSelection> frameSelection = GetFrameSelection();
   if (!frameSelection)
     return NS_ERROR_FAILURE;
@@ -456,21 +456,21 @@ nsresult nsCaret::DrawAtPosition(nsIDOMNode* aNode, PRInt32 aOffset)
   return rv;
 }
 
-nsIFrame * nsCaret::GetCaretFrame(PRInt32 *aOffset)
+nsIFrame * nsCaret::GetCaretFrame(int32_t *aOffset)
 {
   // Return null if we're not drawn to prevent anybody from trying to draw us.
   if (!mDrawn)
-    return nsnull;
+    return nullptr;
 
   // Recompute the frame that we're supposed to draw in to guarantee that
   // we're not going to try to draw into a stale (dead) frame.
-  PRInt32 offset;
-  nsIFrame *frame = nsnull;
+  int32_t offset;
+  nsIFrame *frame = nullptr;
   nsresult rv = GetCaretFrameForNodeOffset(mLastContent, mLastContentOffset,
                                            mLastHint, mLastBidiLevel, &frame,
                                            &offset);
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
   if (aOffset) {
     *aOffset = offset;
@@ -483,8 +483,9 @@ void nsCaret::InvalidateOutsideCaret()
   nsIFrame *frame = GetCaretFrame();
 
   // Only invalidate if we are not fully contained by our frame's rect.
-  if (frame && !frame->GetVisualOverflowRect().Contains(GetCaretRect()))
-    InvalidateRects(mCaretRect, GetHookRect(), frame);
+  if (frame && !frame->GetVisualOverflowRect().Contains(GetCaretRect())) {
+    frame->SchedulePaint();
+  }
 }
 
 void nsCaret::UpdateCaretPosition()
@@ -507,7 +508,7 @@ void nsCaret::PaintCaret(nsDisplayListBuilder *aBuilder,
   NS_ASSERTION(mDrawn, "The caret shouldn't be drawing");
 
   const nsRect drawCaretRect = mCaretRect + aOffset;
-  PRInt32 contentOffset;
+  int32_t contentOffset;
 
 #ifdef DEBUG
   nsIFrame* frame =
@@ -515,7 +516,7 @@ void nsCaret::PaintCaret(nsDisplayListBuilder *aBuilder,
     GetCaretFrame(&contentOffset);
   NS_ASSERTION(frame == aForFrame, "We're referring different frame");
   // If the offset falls outside of the frame, then don't paint the caret.
-  PRInt32 startOffset, endOffset;
+  int32_t startOffset, endOffset;
   if (aForFrame->GetType() == nsGkAtoms::textFrame &&
       (NS_FAILED(aForFrame->GetOffsets(startOffset, endOffset)) ||
       startOffset > contentOffset ||
@@ -552,7 +553,7 @@ void nsCaret::PaintCaret(nsDisplayListBuilder *aBuilder,
 
 
 //-----------------------------------------------------------------------------
-NS_IMETHODIMP nsCaret::NotifySelectionChanged(nsIDOMDocument *, nsISelection *aDomSel, PRInt16 aReason)
+NS_IMETHODIMP nsCaret::NotifySelectionChanged(nsIDOMDocument *, nsISelection *aDomSel, int16_t aReason)
 {
   if (aReason & nsISelectionListener::MOUSEUP_REASON)//this wont do
     return NS_OK;
@@ -613,31 +614,9 @@ nsresult nsCaret::PrimeTimer()
   return NS_OK;
 }
 
-void nsCaret::InvalidateTextOverflowBlock()
-{
-  // If the nearest block has a potential 'text-overflow' marker then
-  // invalidate it.
-  if (mLastContent) {
-    nsIFrame* caretFrame = mLastContent->GetPrimaryFrame();
-    if (caretFrame) {
-      nsIFrame* block = nsLayoutUtils::GetAsBlock(caretFrame) ? caretFrame :
-        nsLayoutUtils::FindNearestBlockAncestor(caretFrame);
-      if (block) {
-        const nsStyleTextReset* style = block->GetStyleTextReset();
-        if (style->mTextOverflow.mLeft.mType != NS_STYLE_TEXT_OVERFLOW_CLIP ||
-            style->mTextOverflow.mRight.mType != NS_STYLE_TEXT_OVERFLOW_CLIP) {
-          block->InvalidateOverflowRect();
-        }
-      }
-    }
-  }
-}
-
 //-----------------------------------------------------------------------------
 void nsCaret::StartBlinking()
 {
-  InvalidateTextOverflowBlock();
-
   if (mReadOnly) {
     // Make sure the one draw command we use for a readonly caret isn't
     // done until the selection is set
@@ -661,8 +640,6 @@ void nsCaret::StartBlinking()
 //-----------------------------------------------------------------------------
 void nsCaret::StopBlinking()
 {
-  InvalidateTextOverflowBlock();
-
   if (mDrawn)     // erase the caret if necessary
     DrawCaret(true);
 
@@ -672,17 +649,17 @@ void nsCaret::StopBlinking()
 
 bool
 nsCaret::DrawAtPositionWithHint(nsIDOMNode*             aNode,
-                                PRInt32                 aOffset,
+                                int32_t                 aOffset,
                                 nsFrameSelection::HINT  aFrameHint,
-                                PRUint8                 aBidiLevel,
+                                uint8_t                 aBidiLevel,
                                 bool                    aInvalidate)
 {
   nsCOMPtr<nsIContent> contentNode = do_QueryInterface(aNode);
   if (!contentNode)
     return false;
 
-  nsIFrame* theFrame = nsnull;
-  PRInt32   theFrameOffset = 0;
+  nsIFrame* theFrame = nullptr;
+  int32_t   theFrameOffset = 0;
 
   nsresult rv = GetCaretFrameForNodeOffset(contentNode, aOffset, aFrameHint, aBidiLevel,
                                            &theFrame, &theFrameOffset);
@@ -721,18 +698,18 @@ nsCaret::DrawAtPositionWithHint(nsIDOMNode*             aNode,
   }
 
   if (aInvalidate)
-    InvalidateRects(mCaretRect, mHookRect, theFrame);
+    theFrame->SchedulePaint();
 
   return true;
 }
 
 nsresult 
 nsCaret::GetCaretFrameForNodeOffset(nsIContent*             aContentNode,
-                                    PRInt32                 aOffset,
+                                    int32_t                 aOffset,
                                     nsFrameSelection::HINT aFrameHint,
-                                    PRUint8                 aBidiLevel,
+                                    uint8_t                 aBidiLevel,
                                     nsIFrame**              aReturnFrame,
-                                    PRInt32*                aReturnOffset)
+                                    int32_t*                aReturnOffset)
 {
 
   //get frame selection and find out what frame to use...
@@ -748,8 +725,8 @@ nsCaret::GetCaretFrameForNodeOffset(nsIContent*             aContentNode,
   if (!frameSelection)
     return NS_ERROR_FAILURE;
 
-  nsIFrame* theFrame = nsnull;
-  PRInt32   theFrameOffset = 0;
+  nsIFrame* theFrame = nullptr;
+  int32_t   theFrameOffset = 0;
 
   theFrame = frameSelection->GetFrameForNodeOffset(aContentNode, aOffset,
                                                    aFrameHint, &theFrameOffset);
@@ -776,12 +753,12 @@ nsCaret::GetCaretFrameForNodeOffset(nsIContent*             aContentNode,
     if (aBidiLevel & BIDI_LEVEL_UNDEFINED)
       aBidiLevel = NS_GET_EMBEDDING_LEVEL(theFrame);
 
-    PRInt32 start;
-    PRInt32 end;
+    int32_t start;
+    int32_t end;
     nsIFrame* frameBefore;
     nsIFrame* frameAfter;
-    PRUint8 levelBefore;     // Bidi level of the character before the caret
-    PRUint8 levelAfter;      // Bidi level of the character after the caret
+    uint8_t levelBefore;     // Bidi level of the character before the caret
+    uint8_t levelAfter;      // Bidi level of the character after the caret
 
     theFrame->GetOffsets(start, end);
     if (start == 0 || end == 0 || start == theFrameOffset || end == theFrameOffset)
@@ -819,7 +796,7 @@ nsCaret::GetCaretFrameForNodeOffset(nsIContent*             aContentNode,
                 // so we stay with the current frame.
                 // Exception: when the first frame on the line has a different Bidi level from the paragraph level, there is no
                 // real frame for the caret to be in. We have to find the visually first frame on the line.
-                PRUint8 baseLevel = NS_GET_BASE_LEVEL(frameAfter);
+                uint8_t baseLevel = NS_GET_BASE_LEVEL(frameAfter);
                 if (baseLevel != levelAfter)
                 {
                   nsPeekOffsetStruct pos(eSelectBeginLine, eDirPrevious, 0, 0, false, true, false, true);
@@ -850,7 +827,7 @@ nsCaret::GetCaretFrameForNodeOffset(nsIContent*             aContentNode,
                 // so we stay with the current frame.
                 // Exception: when the last frame on the line has a different Bidi level from the paragraph level, there is no
                 // real frame for the caret to be in. We have to find the visually last frame on the line.
-                PRUint8 baseLevel = NS_GET_BASE_LEVEL(frameBefore);
+                uint8_t baseLevel = NS_GET_BASE_LEVEL(frameBefore);
                 if (baseLevel != levelBefore)
                 {
                   nsPeekOffsetStruct pos(eSelectEndLine, eDirNext, 0, 0, false, true, false, true);
@@ -977,7 +954,7 @@ bool nsCaret::IsMenuPopupHidingCaret()
 
   // If there's a menu popup open before the popup with
   // the caret, don't show the caret.
-  for (PRUint32 i=0; i<popups.Length(); i++) {
+  for (uint32_t i=0; i<popups.Length(); i++) {
     nsMenuPopupFrame* popupFrame = static_cast<nsMenuPopupFrame*>(popups[i]);
     nsIContent* popupContent = popupFrame->GetContent();
 
@@ -1022,9 +999,9 @@ void nsCaret::DrawCaret(bool aInvalidate)
   }
 
   nsCOMPtr<nsIDOMNode> node;
-  PRInt32 offset;
+  int32_t offset;
   nsFrameSelection::HINT hint;
-  PRUint8 bidiLevel;
+  uint8_t bidiLevel;
 
   if (!mDrawn)
   {
@@ -1066,7 +1043,7 @@ void nsCaret::DrawCaret(bool aInvalidate)
     if (!mLastContent->IsInDoc() ||
         presShell->GetDocument() != mLastContent->GetCurrentDoc())
     {
-      mLastContent = nsnull;
+      mLastContent = nullptr;
       mDrawn = false;
       return;
     }
@@ -1081,7 +1058,7 @@ void nsCaret::DrawCaret(bool aInvalidate)
 }
 
 bool
-nsCaret::UpdateCaretRects(nsIFrame* aFrame, PRInt32 aFrameOffset)
+nsCaret::UpdateCaretRects(nsIFrame* aFrame, int32_t aFrameOffset)
 {
   NS_ASSERTION(aFrame, "Should have a frame here");
 
@@ -1137,16 +1114,6 @@ nsCaret::UpdateCaretRects(nsIFrame* aFrame, PRInt32 aFrameOffset)
   return true;
 }
 
-// static
-void nsCaret::InvalidateRects(const nsRect &aRect, const nsRect &aHook,
-                              nsIFrame *aFrame)
-{
-  NS_ASSERTION(aFrame, "Must have a frame to invalidate");
-  nsRect rect;
-  rect.UnionRect(aRect, aHook);
-  aFrame->Invalidate(rect);
-}
-
 //-----------------------------------------------------------------------------
 /* static */
 void nsCaret::CaretBlinkCallback(nsITimer *aTimer, void *aClosure)
@@ -1164,8 +1131,8 @@ nsCaret::GetFrameSelection()
 {
   nsCOMPtr<nsISelectionPrivate> privateSelection(do_QueryReferent(mDomSelectionWeak));
   if (!privateSelection)
-    return nsnull;
-  nsFrameSelection* frameSelection = nsnull;
+    return nullptr;
+  nsFrameSelection* frameSelection = nullptr;
   privateSelection->GetFrameSelection(&frameSelection);
   return frameSelection;
 }
