@@ -75,7 +75,7 @@ var gClearingForAssertionCheck = false;
 
 const TYPE_SCRIPT = 'script'; // test contains individual test results
 
-function markupDocumentViewer() { 
+function markupDocumentViewer() {
     return docShell.contentViewer.QueryInterface(CI.nsIMarkupDocumentViewer);
 }
 
@@ -119,7 +119,9 @@ function PaintWaitFinishedListener(event)
 
 function OnInitialLoad()
 {
+#ifndef REFTEST_B2G
     removeEventListener("load", OnInitialLoad, true);
+#endif
 
     gDebug = CC[DEBUG_CONTRACTID].getService(CI.nsIDebug2);
 
@@ -132,7 +134,7 @@ function OnInitialLoad()
 
     addEventListener("MozPaintWait", PaintWaitListener, true);
     addEventListener("MozPaintWaitFinished", PaintWaitFinishedListener, true);
- 
+
     LogWarning("Using browser remote="+ gBrowserIsRemote +"\n");
 }
 
@@ -170,11 +172,16 @@ function resetZoom() {
 }
 
 function doPrintMode(contentRootElement) {
+#if REFTEST_B2G
+    // nsIPrintSettings not available in B2G
+    return false;
+#else
     // use getAttribute because className works differently in HTML and SVG
     return contentRootElement &&
            contentRootElement.hasAttribute('class') &&
            contentRootElement.getAttribute('class').split(/\s+/)
                              .indexOf("reftest-print") != -1;
+#endif
 }
 
 function setupPrintMode() {
@@ -289,7 +296,7 @@ function WaitForTestEnd(contentRootElement, inPrintMode) {
             } catch (e) {
                 LogWarning("flushWindow failed: " + e + "\n");
             }
-            
+
             if (!afterPaintWasPending && utils.isMozAfterPaintPending) {
                 LogInfo("FlushRendering generated paint for window " + win.location.href);
                 anyPendingPaintsGeneratedInDescendants = true;
@@ -380,7 +387,7 @@ function WaitForTestEnd(contentRootElement, inPrintMode) {
             }
 
             state = STATE_WAITING_FOR_REFTEST_WAIT_REMOVAL;
-            var hasReftestWait = shouldWaitForReftestWaitRemoval(contentRootElement);            
+            var hasReftestWait = shouldWaitForReftestWaitRemoval(contentRootElement);
             // Notify the test document that now is a good time to test some invalidation
             LogInfo("MakeProgress: dispatching MozReftestInvalidate");
             if (contentRootElement) {
@@ -750,7 +757,7 @@ function SendInitCanvasWithSnapshot()
     // browser though, it doesn't wrt correctness whether this request
     // is sync or async.
     var ret = sendSyncMessage("reftest:InitCanvasWithSnapshot")[0];
- 
+
     gHaveCanvasSnapshot = ret.painted;
     return ret.painted;
 }
@@ -780,9 +787,10 @@ function SendUpdateCanvasForEvent(event)
 {
     var win = content;
     var scale = markupDocumentViewer().fullZoom;
- 
+
     var rects = [ ];
     var rectList = event.clientRects;
+    LogInfo("SendUpdateCanvasForEvent with " + rectList.length + " rects");
     for (var i = 0; i < rectList.length; ++i) {
         var r = rectList[i];
         // Set left/top/right/bottom to "device pixel" boundaries
@@ -790,6 +798,7 @@ function SendUpdateCanvasForEvent(event)
         var top = Math.floor(roundTo(r.top*scale, 0.001));
         var right = Math.ceil(roundTo(r.right*scale, 0.001));
         var bottom = Math.ceil(roundTo(r.bottom*scale, 0.001));
+        LogInfo("Rect: " + left + " " + top + " " + right + " " + bottom);
 
         rects.push({ left: left, top: top, right: right, bottom: bottom });
     }
@@ -803,5 +812,8 @@ function SendUpdateCanvasForEvent(event)
         sendAsyncMessage("reftest:UpdateCanvasForInvalidation", { rects: rects });
     }
 }
-
+#if REFTEST_B2G
+OnInitialLoad();
+#else
 addEventListener("load", OnInitialLoad, true);
+#endif
