@@ -7,7 +7,6 @@
 #include "nsMenuPopupFrame.h"
 #include "nsGkAtoms.h"
 #include "nsIContent.h"
-#include "prtypes.h"
 #include "nsIAtom.h"
 #include "nsPresContext.h"
 #include "nsStyleContext.h"
@@ -53,7 +52,7 @@
 
 using namespace mozilla;
 
-PRInt8 nsMenuPopupFrame::sDefaultLevelIsTop = -1;
+int8_t nsMenuPopupFrame::sDefaultLevelIsTop = -1;
 
 // NS_NewMenuPopupFrame
 //
@@ -76,7 +75,7 @@ NS_QUERYFRAME_TAIL_INHERITING(nsBoxFrame)
 //
 nsMenuPopupFrame::nsMenuPopupFrame(nsIPresShell* aShell, nsStyleContext* aContext)
   :nsBoxFrame(aShell, aContext),
-  mCurrentMenu(nsnull),
+  mCurrentMenu(nullptr),
   mPrefSize(-1, -1),
   mLastClientOffset(0, 0),
   mPopupType(ePopupTypePanel),
@@ -131,7 +130,7 @@ nsMenuPopupFrame::Init(nsIContent*      aContent,
 
   mPopupType = ePopupTypePanel;
   nsIDocument* doc = aContent->OwnerDoc();
-  PRInt32 namespaceID;
+  int32_t namespaceID;
   nsCOMPtr<nsIAtom> tag = doc->BindingManager()->ResolveTag(aContent, &namespaceID);
   if (namespaceID == kNameSpaceID_XUL) {
     if (tag == nsGkAtoms::menupopup || tag == nsGkAtoms::popup)
@@ -148,7 +147,7 @@ nsMenuPopupFrame::Init(nsIContent*      aContent,
 
   nsCOMPtr<nsISupports> cont = PresContext()->GetContainer();
   nsCOMPtr<nsIDocShellTreeItem> dsti = do_QueryInterface(cont);
-  PRInt32 type = -1;
+  int32_t type = -1;
   if (dsti && NS_SUCCEEDED(dsti->GetItemType(&type)) &&
       type == nsIDocShellTreeItem::typeChrome)
     mInContentShell = false;
@@ -170,6 +169,8 @@ nsMenuPopupFrame::Init(nsIContent*      aContent,
     }
   }
 
+  AddStateBits(NS_FRAME_IN_POPUP);
+
   return rv;
 }
 
@@ -200,7 +201,7 @@ nsMenuPopupFrame::PopupLevel(bool aIsNoAutoHide) const
 
   // If the level attribute has been set, use that.
   static nsIContent::AttrValuesArray strings[] =
-    {&nsGkAtoms::top, &nsGkAtoms::parent, &nsGkAtoms::floating, nsnull};
+    {&nsGkAtoms::top, &nsGkAtoms::parent, &nsGkAtoms::floating, nullptr};
   switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsGkAtoms::level,
                                     strings, eCaseMatters)) {
     case 0:
@@ -267,9 +268,10 @@ nsMenuPopupFrame::CreateWidgetForView(nsIView* aView)
                                      (eTransparencyTransparent ==
                                       mode);
   nsIContent* parentContent = GetContent()->GetParent();
-  nsIAtom *tag = nsnull;
+  nsIAtom *tag = nullptr;
   if (parentContent)
     tag = parentContent->Tag();
+  widgetData.mSupportTranslucency = mode == eTransparencyTransparent;
   widgetData.mDropShadow = !(viewHasTransparentContent || tag == nsGkAtoms::menulist);
   widgetData.mPopupLevel = PopupLevel(widgetData.mNoAutoHide);
 
@@ -310,10 +312,10 @@ nsMenuPopupFrame::CreateWidgetForView(nsIView* aView)
   return NS_OK;
 }
 
-PRUint8
+uint8_t
 nsMenuPopupFrame::GetShadowStyle()
 {
-  PRUint8 shadow = GetStyleUIReset()->mWindowShadow;
+  uint8_t shadow = GetStyleUIReset()->mWindowShadow;
   if (shadow != NS_STYLE_WINDOW_SHADOW_DEFAULT)
     return shadow;
 
@@ -337,7 +339,7 @@ public:
 
   NS_IMETHOD Run()
   {
-    nsMouseEvent event(true, NS_XUL_POPUP_SHOWN, nsnull, nsMouseEvent::eReal);
+    nsMouseEvent event(true, NS_XUL_POPUP_SHOWN, nullptr, nsMouseEvent::eReal);
     return nsEventDispatcher::Dispatch(mPopup, mPresContext, &event);                 
   }
 
@@ -438,23 +440,31 @@ nsMenuPopupFrame::LayoutPopup(nsBoxLayoutState& aState, nsIFrame* aParentMenu, b
       // so set the preferred size accordingly
       mPrefSize = newsize;
       if (isOpen) {
-        SetPopupPosition(nsnull, false);
+        SetPopupPosition(nullptr, false);
       }
     }
   }
 
   nsPresContext* pc = PresContext();
+  nsIView* view = GetView();
+
+  if ((isOpen && mIsOpenChanged) || sizeChanged) {
+    // If the size of the popup changed, apply any size constraints.
+    nsIWidget* widget = view->GetWidget();
+    if (widget) {
+      SetSizeConstraints(pc, widget, minSize, maxSize);
+    }
+  }
+
   if (isOpen) {
-    nsIView* view = GetView();
     nsIViewManager* viewManager = view->GetViewManager();
     nsRect rect = GetRect();
     rect.x = rect.y = 0;
-
     viewManager->ResizeView(view, rect);
 
     viewManager->SetViewVisibility(view, nsViewVisibility_kShow);
     mPopupState = ePopupOpenAndVisible;
-    nsContainerFrame::SyncFrameViewProperties(pc, this, nsnull, view, 0);
+    nsContainerFrame::SyncFrameViewProperties(pc, this, nullptr, view, 0);
   }
 
   // finally, if the popup just opened, send a popupshown event
@@ -473,7 +483,7 @@ nsMenuPopupFrame::GetTriggerContent(nsMenuPopupFrame* aMenuPopupFrame)
       return aMenuPopupFrame->mTriggerContent;
 
     // check up the menu hierarchy until a popup with a trigger node is found
-    nsMenuFrame* menuFrame = aMenuPopupFrame->GetParentMenu();
+    nsMenuFrame* menuFrame = do_QueryFrame(aMenuPopupFrame->GetParent());
     if (!menuFrame)
       break;
 
@@ -484,14 +494,14 @@ nsMenuPopupFrame::GetTriggerContent(nsMenuPopupFrame* aMenuPopupFrame)
     aMenuPopupFrame = static_cast<nsMenuPopupFrame *>(parentPopup);
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 void
 nsMenuPopupFrame::InitPositionFromAnchorAlign(const nsAString& aAnchor,
                                               const nsAString& aAlign)
 {
-  mTriggerContent = nsnull;
+  mTriggerContent = nullptr;
 
   if (aAnchor.EqualsLiteral("topleft"))
     mPopupAnchor = POPUPALIGNMENT_TOPLEFT;
@@ -528,7 +538,7 @@ void
 nsMenuPopupFrame::InitializePopup(nsIContent* aAnchorContent,
                                   nsIContent* aTriggerContent,
                                   const nsAString& aPosition,
-                                  PRInt32 aXPos, PRInt32 aYPos,
+                                  int32_t aXPos, int32_t aYPos,
                                   bool aAttributesOverride)
 {
   EnsureWidget();
@@ -565,7 +575,7 @@ nsMenuPopupFrame::InitializePopup(nsIContent* aAnchorContent,
     mFlipBoth = flip.EqualsLiteral("both");
 
     position.CompressWhitespace();
-    PRInt32 spaceIdx = position.FindChar(' ');
+    int32_t spaceIdx = position.FindChar(' ');
     // if there is a space in the position, assume it is the anchor and
     // alignment as two separate tokens.
     if (spaceIdx >= 0) {
@@ -629,14 +639,14 @@ nsMenuPopupFrame::InitializePopup(nsIContent* aAnchorContent,
     mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::left, left);
     mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::top, top);
 
-    PRInt32 err;
+    nsresult err;
     if (!left.IsEmpty()) {
-      PRInt32 x = left.ToInteger(&err);
+      int32_t x = left.ToInteger(&err);
       if (NS_SUCCEEDED(err))
         mScreenXPos = x;
     }
     if (!top.IsEmpty()) {
-      PRInt32 y = top.ToInteger(&err);
+      int32_t y = top.ToInteger(&err);
       if (NS_SUCCEEDED(err))
         mScreenYPos = y;
     }
@@ -645,13 +655,13 @@ nsMenuPopupFrame::InitializePopup(nsIContent* aAnchorContent,
 
 void
 nsMenuPopupFrame::InitializePopupAtScreen(nsIContent* aTriggerContent,
-                                          PRInt32 aXPos, PRInt32 aYPos,
+                                          int32_t aXPos, int32_t aYPos,
                                           bool aIsContextMenu)
 {
   EnsureWidget();
 
   mPopupState = ePopupShowing;
-  mAnchorContent = nsnull;
+  mAnchorContent = nullptr;
   mTriggerContent = aTriggerContent;
   mScreenXPos = aXPos;
   mScreenYPos = aYPos;
@@ -666,7 +676,7 @@ void
 nsMenuPopupFrame::InitializePopupWithAnchorAlign(nsIContent* aAnchorContent,
                                                  nsAString& aAnchor,
                                                  nsAString& aAlign,
-                                                 PRInt32 aXPos, PRInt32 aYPos)
+                                                 int32_t aXPos, int32_t aYPos)
 {
   EnsureWidget();
 
@@ -686,7 +696,7 @@ nsMenuPopupFrame::InitializePopupWithAnchorAlign(nsIContent* aAnchorContent,
     InitPositionFromAnchorAlign(aAnchor, aAlign);
   }
   else {
-    mAnchorContent = nsnull;
+    mAnchorContent = nullptr;
     mPopupAnchor = POPUPALIGNMENT_NONE;
     mPopupAlignment = POPUPALIGNMENT_NONE;
     mScreenXPos = aXPos;
@@ -705,7 +715,7 @@ nsMenuPopupFrame::ShowPopup(bool aIsContextMenu, bool aSelectFirstItem)
     mPopupState = ePopupOpen;
     mIsOpenChanged = true;
 
-    nsMenuFrame* menuFrame = GetParentMenu();
+    nsMenuFrame* menuFrame = do_QueryFrame(GetParent());
     if (menuFrame) {
       nsWeakFrame weakFrame(this);
       menuFrame->PopupOpened();
@@ -751,13 +761,13 @@ nsMenuPopupFrame::HidePopup(bool aDeselectMenu, nsPopupState aNewState)
         if (win) {
           nsCOMPtr<nsPIWindowRoot> root = win->GetTopWindowRoot();
           if (root) {
-            root->SetPopupNode(nsnull);
+            root->SetPopupNode(nullptr);
           }
         }
       }
     }
-    mTriggerContent = nsnull;
-    mAnchorContent = nsnull;
+    mTriggerContent = nullptr;
+    mAnchorContent = nullptr;
   }
 
   // when invisible and about to be closed, HidePopup has already been called,
@@ -771,19 +781,25 @@ nsMenuPopupFrame::HidePopup(bool aDeselectMenu, nsPopupState aNewState)
   mPopupState = aNewState;
 
   if (IsMenu())
-    SetCurrentMenuItem(nsnull);
+    SetCurrentMenuItem(nullptr);
 
   mIncrementalString.Truncate();
 
   LockMenuUntilClosed(false);
 
   mIsOpenChanged = false;
-  mCurrentMenu = nsnull; // make sure no current menu is set
+  mCurrentMenu = nullptr; // make sure no current menu is set
   mHFlip = mVFlip = false;
 
   nsIView* view = GetView();
   nsIViewManager* viewManager = view->GetViewManager();
   viewManager->SetViewVisibility(view, nsViewVisibility_kHide);
+  nsIWidget* widget = view->GetWidget();
+  if (widget) {
+    widget::SizeConstraints sizeConstraints = widget->GetSizeConstraints();
+    sizeConstraints.mMinSize.width = sizeConstraints.mMinSize.height = 0;
+    widget->SetSizeConstraints(sizeConstraints);
+  }
   viewManager->ResizeView(view, nsRect(0, 0, 0, 0));
 
   FireDOMEvent(NS_LITERAL_STRING("DOMMenuInactive"), mContent);
@@ -796,25 +812,17 @@ nsMenuPopupFrame::HidePopup(bool aDeselectMenu, nsPopupState aNewState)
 
   if (state.HasState(NS_EVENT_STATE_HOVER)) {
     nsEventStateManager *esm = PresContext()->EventStateManager();
-    esm->SetContentState(nsnull, NS_EVENT_STATE_HOVER);
+    esm->SetContentState(nullptr, NS_EVENT_STATE_HOVER);
   }
 
-  nsMenuFrame* menuFrame = GetParentMenu();
+  nsMenuFrame* menuFrame = do_QueryFrame(GetParent());
   if (menuFrame) {
     menuFrame->PopupClosed(aDeselectMenu);
   }
 }
 
 void
-nsMenuPopupFrame::InvalidateInternal(const nsRect& aDamageRect,
-                                     nscoord aX, nscoord aY, nsIFrame* aForChild,
-                                     PRUint32 aFlags)
-{
-  InvalidateRoot(aDamageRect + nsPoint(aX, aY), aFlags);
-}
-
-void
-nsMenuPopupFrame::GetLayoutFlags(PRUint32& aFlags)
+nsMenuPopupFrame::GetLayoutFlags(uint32_t& aFlags)
 {
   aFlags = NS_FRAME_NO_SIZE_VIEW | NS_FRAME_NO_MOVE_VIEW | NS_FRAME_NO_VISIBILITY;
 }
@@ -851,7 +859,7 @@ nsMenuPopupFrame::GetRootViewForPopup(nsIFrame* aStartFrame)
     view = temp;
   }
 
-  return nsnull;
+  return nullptr;
 }
 
 nsPoint
@@ -859,8 +867,8 @@ nsMenuPopupFrame::AdjustPositionForAnchorAlign(nsRect& anchorRect,
                                                FlipStyle& aHFlip, FlipStyle& aVFlip)
 {
   // flip the anchor and alignment for right-to-left
-  PRInt8 popupAnchor(mPopupAnchor);
-  PRInt8 popupAlign(mPopupAlignment);
+  int8_t popupAnchor(mPopupAnchor);
+  int8_t popupAlign(mPopupAlignment);
   if (IsDirectionRTL()) {
     // no need to flip the centered anchor types
     if (popupAnchor < POPUPALIGNMENT_LEFTCENTER) {
@@ -1187,14 +1195,14 @@ nsMenuPopupFrame::SetPopupPosition(nsIFrame* aAnchorFrame, bool aIsMove)
     // first convert the screen position in mScreenXPos and mScreenYPos from
     // CSS pixels into device pixels, ignoring any scaling as mScreenXPos and
     // mScreenYPos are unscaled screen coordinates.
-    PRInt32 factor = devContext->UnscaledAppUnitsPerDevPixel();
+    int32_t factor = devContext->UnscaledAppUnitsPerDevPixel();
 
     // context menus should be offset by two pixels so that they don't appear
     // directly where the cursor is. Otherwise, it is too easy to have the
     // context menu close up again.
     if (mAdjustOffsetForContextMenu) {
-      PRInt32 offsetForContextMenuDev =
-        nsPresContext::CSSPixelsToAppUnits(2) / factor;
+      int32_t offsetForContextMenuDev =
+        nsPresContext::CSSPixelsToAppUnits(CONTEXT_MENU_OFFSET_PIXELS) / factor;
       offsetForContextMenu = presContext->DevPixelsToAppUnits(offsetForContextMenuDev);
     }
 
@@ -1318,10 +1326,11 @@ nsMenuPopupFrame::GetConstraintRect(const nsRect& aAnchorRect,
     // so we should use the same screen. Otherwise, use the screen where the
     // anchor is located.
     nsRect rect = mInContentShell ? aRootScreenRect : aAnchorRect;
-    PRInt32 width = rect.width > 0 ? presContext->AppUnitsToDevPixels(rect.width) : 1;
-    PRInt32 height = rect.height > 0 ? presContext->AppUnitsToDevPixels(rect.height) : 1;
-    sm->ScreenForRect(presContext->AppUnitsToDevPixels(rect.x),
-                      presContext->AppUnitsToDevPixels(rect.y),
+    // nsIScreenManager::ScreenForRect wants the coordinates in CSS pixels
+    int32_t width = NS_MAX(1, nsPresContext::AppUnitsToIntCSSPixels(rect.width));
+    int32_t height = NS_MAX(1, nsPresContext::AppUnitsToIntCSSPixels(rect.height));
+    sm->ScreenForRect(nsPresContext::AppUnitsToIntCSSPixels(rect.x),
+                      nsPresContext::AppUnitsToIntCSSPixels(rect.y),
                       width, height, getter_AddRefs(screen));
     if (screen) {
       // get the total screen area if the popup is allowed to overlap it.
@@ -1346,9 +1355,9 @@ nsMenuPopupFrame::GetConstraintRect(const nsRect& aAnchorRect,
   return screenRect;
 }
 
-void nsMenuPopupFrame::CanAdjustEdges(PRInt8 aHorizontalSide, PRInt8 aVerticalSide, nsIntPoint& aChange)
+void nsMenuPopupFrame::CanAdjustEdges(int8_t aHorizontalSide, int8_t aVerticalSide, nsIntPoint& aChange)
 {
-  PRInt8 popupAlign(mPopupAlignment);
+  int8_t popupAlign(mPopupAlignment);
   if (IsDirectionRTL()) {
     popupAlign = -popupAlign;
   }
@@ -1424,7 +1433,7 @@ bool nsMenuPopupFrame::ConsumeOutsideClicks()
 nsIScrollableFrame* nsMenuPopupFrame::GetScrollFrame(nsIFrame* aStart)
 {
   if (!aStart)
-    return nsnull;  
+    return nullptr;  
 
   // try start frame and siblings
   nsIFrame* currFrame = aStart;
@@ -1445,7 +1454,7 @@ nsIScrollableFrame* nsMenuPopupFrame::GetScrollFrame(nsIFrame* aStart)
     currFrame = currFrame->GetNextSibling();
   } while (currFrame);
 
-  return nsnull;
+  return nullptr;
 }
 
 void nsMenuPopupFrame::EnsureMenuItemIsVisible(nsMenuFrame* aMenuItem)
@@ -1483,7 +1492,7 @@ NS_IMETHODIMP nsMenuPopupFrame::SetCurrentMenuItem(nsMenuFrame* aMenuItem)
 void
 nsMenuPopupFrame::CurrentMenuIsBeingDestroyed()
 {
-  mCurrentMenu = nsnull;
+  mCurrentMenu = nullptr;
 }
 
 NS_IMETHODIMP
@@ -1531,31 +1540,31 @@ nsMenuPopupFrame::Enter(nsGUIEvent* aEvent)
   if (mCurrentMenu)
     return mCurrentMenu->Enter(aEvent);
 
-  return nsnull;
+  return nullptr;
 }
 
 nsMenuFrame*
 nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction)
 {
-  PRUint32 charCode, keyCode;
+  uint32_t charCode, keyCode;
   aKeyEvent->GetCharCode(&charCode);
   aKeyEvent->GetKeyCode(&keyCode);
 
   doAction = false;
 
   // Enumerate over our list of frames.
-  nsIFrame* immediateParent = nsnull;
+  nsIFrame* immediateParent = nullptr;
   PresContext()->PresShell()->
-    FrameConstructor()->GetInsertionPoint(this, nsnull, &immediateParent);
+    FrameConstructor()->GetInsertionPoint(this, nullptr, &immediateParent);
   if (!immediateParent)
     immediateParent = this;
 
-  PRUint32 matchCount = 0, matchShortcutCount = 0;
+  uint32_t matchCount = 0, matchShortcutCount = 0;
   bool foundActive = false;
   bool isShortcut;
-  nsMenuFrame* frameBefore = nsnull;
-  nsMenuFrame* frameAfter = nsnull;
-  nsMenuFrame* frameShortcut = nsnull;
+  nsMenuFrame* frameBefore = nullptr;
+  nsMenuFrame* frameAfter = nullptr;
+  nsMenuFrame* frameShortcut = nullptr;
 
   nsIContent* parentContent = mContent->GetParent();
 
@@ -1570,7 +1579,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction
     if (keyCode == NS_VK_BACK) {
       if (!isMenu && !mIncrementalString.IsEmpty()) {
         mIncrementalString.SetLength(mIncrementalString.Length() - 1);
-        return nsnull;
+        return nullptr;
       }
       else {
 #ifdef XP_WIN
@@ -1580,7 +1589,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction
 #endif  // #ifdef XP_WIN
       }
     }
-    return nsnull;
+    return nullptr;
   }
   else {
     PRUnichar uniChar = ToLowerCase(static_cast<PRUnichar>(charCode));
@@ -1594,7 +1603,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction
 
   // See bug 188199 & 192346, if all letters in incremental string are same, just try to match the first one
   nsAutoString incrementalString(mIncrementalString);
-  PRUint32 charIndex = 1, stringLength = incrementalString.Length();
+  uint32_t charIndex = 1, stringLength = incrementalString.Length();
   while (charIndex < stringLength && incrementalString[charIndex] == incrementalString[charIndex - 1]) {
     charIndex++;
   }
@@ -1613,7 +1622,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction
   //       <http://bugzilla.mozilla.org/show_bug.cgi?id=126675#c32>
   currFrame = immediateParent->GetFirstPrincipalChild();
 
-  PRInt32 menuAccessKey = -1;
+  int32_t menuAccessKey = -1;
   nsMenuBarListener::GetMenuAccessKey(&menuAccessKey);
 
   // We start searching from first child. This process is divided into two parts
@@ -1640,28 +1649,29 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction
       if (StringBeginsWith(textKey, incrementalString,
                            nsCaseInsensitiveStringComparator())) {
         // mIncrementalString is a prefix of textKey
-        if (currFrame->GetType() == nsGkAtoms::menuFrame) {
+        nsMenuFrame* menu = do_QueryFrame(currFrame);
+        if (menu) {
           // There is one match
           matchCount++;
           if (isShortcut) {
             // There is one shortcut-key match
             matchShortcutCount++;
             // Record the matched item. If there is only one matched shortcut item, do it
-            frameShortcut = static_cast<nsMenuFrame *>(currFrame);
+            frameShortcut = menu;
           }
           if (!foundActive) {
             // It's a first candidate item located before/on the current item
             if (!frameBefore)
-              frameBefore = static_cast<nsMenuFrame *>(currFrame);
+              frameBefore = menu;
           }
           else {
             // It's a first candidate item located after the current item
             if (!frameAfter)
-              frameAfter = static_cast<nsMenuFrame *>(currFrame);
+              frameAfter = menu;
           }
         }
         else
-          return nsnull;
+          return nullptr;
       }
 
       // Get the active status
@@ -1702,7 +1712,7 @@ nsMenuPopupFrame::FindMenuWithShortcut(nsIDOMKeyEvent* aKeyEvent, bool& doAction
   }
 #endif  // #ifdef XP_WIN
 
-  return nsnull;
+  return nullptr;
 }
 
 void
@@ -1711,9 +1721,9 @@ nsMenuPopupFrame::LockMenuUntilClosed(bool aLock)
   mIsMenuLocked = aLock;
 
   // Lock / unlock the parent, too.
-  nsIFrame* parent = GetParent();
-  if (parent && parent->GetType() == nsGkAtoms::menuFrame) {
-    nsMenuParent* parentParent = static_cast<nsMenuFrame*>(parent)->GetMenuParent();
+  nsMenuFrame* menu = do_QueryFrame(GetParent());
+  if (menu) {
+    nsMenuParent* parentParent = menu->GetMenuParent();
     if (parentParent) {
       parentParent->LockMenuUntilClosed(aLock);
     }
@@ -1725,7 +1735,7 @@ nsMenuPopupFrame::GetWidget()
 {
   nsIView * view = GetRootViewForPopup(this);
   if (!view)
-    return nsnull;
+    return nullptr;
 
   return view->GetWidget();
 }
@@ -1752,9 +1762,9 @@ nsMenuPopupFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
 // helpers /////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP 
-nsMenuPopupFrame::AttributeChanged(PRInt32 aNameSpaceID,
+nsMenuPopupFrame::AttributeChanged(int32_t aNameSpaceID,
                                    nsIAtom* aAttribute,
-                                   PRInt32 aModType)
+                                   int32_t aModType)
 
 {
   nsresult rv = nsBoxFrame::AttributeChanged(aNameSpaceID, aAttribute,
@@ -1791,9 +1801,9 @@ nsMenuPopupFrame::MoveToAttributePosition()
   nsAutoString left, top;
   mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::left, left);
   mContent->GetAttr(kNameSpaceID_None, nsGkAtoms::top, top);
-  PRInt32 err1, err2;
-  PRInt32 xpos = left.ToInteger(&err1);
-  PRInt32 ypos = top.ToInteger(&err2);
+  nsresult err1, err2;
+  int32_t xpos = left.ToInteger(&err1);
+  int32_t ypos = top.ToInteger(&err2);
 
   if (NS_SUCCEEDED(err1) && NS_SUCCEEDED(err2))
     MoveTo(xpos, ypos, false);
@@ -1802,11 +1812,11 @@ nsMenuPopupFrame::MoveToAttributePosition()
 void
 nsMenuPopupFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
-  nsIFrame* parent = GetParent();
-  if (parent && parent->GetType() == nsGkAtoms::menuFrame) {
+  nsMenuFrame* menu = do_QueryFrame(GetParent());
+  if (menu) {
     // clear the open attribute on the parent menu
     nsContentUtils::AddScriptRunner(
-      new nsUnsetAttrRunnable(parent->GetContent(), nsGkAtoms::open));
+      new nsUnsetAttrRunnable(menu->GetContent(), nsGkAtoms::open));
   }
 
   nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
@@ -1816,7 +1826,7 @@ nsMenuPopupFrame::DestroyFrom(nsIFrame* aDestructRoot)
   nsIRootBox* rootBox =
     nsIRootBox::GetRootBox(PresContext()->GetPresShell());
   if (rootBox && rootBox->GetDefaultTooltip() == mContent) {
-    rootBox->SetDefaultTooltip(nsnull);
+    rootBox->SetDefaultTooltip(nullptr);
   }
 
   nsBoxFrame::DestroyFrom(aDestructRoot);
@@ -1824,7 +1834,7 @@ nsMenuPopupFrame::DestroyFrom(nsIFrame* aDestructRoot)
 
 
 void
-nsMenuPopupFrame::MoveTo(PRInt32 aLeft, PRInt32 aTop, bool aUpdateAttrs)
+nsMenuPopupFrame::MoveTo(int32_t aLeft, int32_t aTop, bool aUpdateAttrs)
 {
   nsIWidget* widget = GetWidget();
   if ((mScreenXPos == aLeft && mScreenYPos == aTop) &&
@@ -1838,11 +1848,20 @@ nsMenuPopupFrame::MoveTo(PRInt32 aLeft, PRInt32 aTop, bool aUpdateAttrs)
   // added to the position when SetPopupPosition is called.
   nsMargin margin(0, 0, 0, 0);
   GetStyleMargin()->GetMargin(margin);
+
+  // Workaround for bug 788189.  See also bug 708278 comment #25 and following.
+  if (mAdjustOffsetForContextMenu) {
+    nscoord offsetForContextMenu =
+      nsPresContext::CSSPixelsToAppUnits(CONTEXT_MENU_OFFSET_PIXELS);
+    margin.left += offsetForContextMenu;
+    margin.top += offsetForContextMenu;
+  }
+
   nsPresContext* presContext = PresContext();
   mScreenXPos = aLeft - presContext->AppUnitsToIntCSSPixels(margin.left);
   mScreenYPos = aTop - presContext->AppUnitsToIntCSSPixels(margin.top);
 
-  SetPopupPosition(nsnull, true);
+  SetPopupPosition(nullptr, true);
 
   nsCOMPtr<nsIContent> popup = mContent;
   if (aUpdateAttrs && (popup->HasAttr(kNameSpaceID_None, nsGkAtoms::left) ||
@@ -1869,7 +1888,7 @@ nsMenuPopupFrame::SetAutoPosition(bool aShouldAutoPosition)
 }
 
 void
-nsMenuPopupFrame::SetConsumeRollupEvent(PRUint32 aConsumeMode)
+nsMenuPopupFrame::SetConsumeRollupEvent(uint32_t aConsumeMode)
 {
   mConsumeRollupEvent = aConsumeMode;
 }
@@ -1886,12 +1905,12 @@ nsMenuPopupFrame::CreatePopupView()
   }
 
   nsIViewManager* viewManager = PresContext()->GetPresShell()->GetViewManager();
-  NS_ASSERTION(nsnull != viewManager, "null view manager");
+  NS_ASSERTION(nullptr != viewManager, "null view manager");
 
   // Create a view
   nsIView* parentView = viewManager->GetRootView();
   nsViewVisibility visibility = nsViewVisibility_kHide;
-  PRInt32 zIndex = PR_INT32_MAX;
+  int32_t zIndex = INT32_MAX;
   bool    autoZIndex = false;
 
   NS_ASSERTION(parentView, "no parent view");
@@ -1901,7 +1920,7 @@ nsMenuPopupFrame::CreatePopupView()
   if (view) {
     viewManager->SetViewZIndex(view, autoZIndex, zIndex);
     // XXX put view last in document order until we can do better
-    viewManager->InsertChild(parentView, view, nsnull, true);
+    viewManager->InsertChild(parentView, view, nullptr, true);
   }
 
   // Remember our view
