@@ -5,88 +5,81 @@
 
 package org.mozilla.gecko;
 
-import android.util.Log;
-import java.lang.String;
-import java.util.concurrent.SynchronousQueue;
-import java.util.concurrent.TimeUnit;
+import org.mozilla.gecko.gfx.LayerView;
+import org.mozilla.gecko.util.GeckoEventResponder;
+import org.mozilla.gecko.widget.DateTimePicker;
+
+import org.json.JSONArray;
+import org.json.JSONObject;
+
 import android.app.AlertDialog;
 import android.content.Context;
 import android.content.DialogInterface;
-import android.content.DialogInterface.OnClickListener;
 import android.content.DialogInterface.OnCancelListener;
-import android.content.DialogInterface.OnMultiChoiceClickListener;
+import android.content.DialogInterface.OnClickListener;
 import android.content.res.Resources;
 import android.graphics.Bitmap;
-import android.graphics.Color;
 import android.graphics.drawable.BitmapDrawable;
 import android.graphics.drawable.Drawable;
 import android.text.InputType;
-import android.text.method.PasswordTransformationMethod;
-import android.util.TypedValue;
+import android.util.Log;
+import android.view.inputmethod.InputMethodManager;
+import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
-import android.view.LayoutInflater;
 import android.view.ViewGroup.LayoutParams;
 import android.widget.AdapterView;
 import android.widget.AdapterView.OnItemClickListener;
-import android.widget.TextView;
+import android.widget.ArrayAdapter;
 import android.widget.CheckBox;
 import android.widget.CheckedTextView;
+import android.widget.DatePicker;
 import android.widget.EditText;
 import android.widget.LinearLayout;
-import android.widget.Spinner;
-import android.widget.ArrayAdapter;
 import android.widget.ListView;
-import org.json.JSONArray;
-import org.json.JSONObject;
-import android.text.InputType;
+import android.widget.Spinner;
+import android.widget.TextView;
+import android.widget.TimePicker;
+
+import java.text.SimpleDateFormat;
+import java.util.concurrent.SynchronousQueue;
+import java.util.concurrent.TimeUnit;
+import java.util.Calendar;
+import java.util.GregorianCalendar;
 
 public class PromptService implements OnClickListener, OnCancelListener, OnItemClickListener, GeckoEventResponder {
     private static final String LOGTAG = "GeckoPromptService";
 
+    private String[] mButtons;
     private PromptInput[] mInputs;
-    private AlertDialog mDialog = null;
+    private AlertDialog mDialog;
     private static LayoutInflater mInflater;
 
-    private final static int GROUP_PADDING_SIZE = 32; // in dip units
-    private static int mGroupPaddingSize = 0; // calculated from GROUP_PADDING_SIZE. In pixel units
-
-    private final static int LEFT_RIGHT_TEXT_WITH_ICON_PADDING = 10; // in dip units
-    private static int mLeftRightTextWithIconPadding = 0; // calculated from LEFT_RIGHT_TEXT_WITH_ICON_PADDING.
-
-    private final static int TOP_BOTTOM_TEXT_WITH_ICON_PADDING = 8; // in dip units
-    private static int mTopBottomTextWithIconPadding = 0; // calculated from TOP_BOTTOM_TEXT_WITH_ICON_PADDING.
-
-    private final static int ICON_TEXT_PADDING = 10; // in dip units
-    private static int mIconTextPadding = 0; // calculated from ICON_TEXT_PADDING.
-
-    private final static int ICON_SIZE = 72; // in dip units
-    private static int mIconSize = 0; // calculated from ICON_SIZE.
+    private int mGroupPaddingSize;
+    private int mLeftRightTextWithIconPadding;
+    private int mTopBottomTextWithIconPadding;
+    private int mIconTextPadding;
+    private int mIconSize;
 
     PromptService() {
         mInflater = LayoutInflater.from(GeckoApp.mAppContext);
-        Resources res = GeckoApp.mAppContext.getResources();
-        mGroupPaddingSize = (int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP,
-                                                           GROUP_PADDING_SIZE,
-                                                           res.getDisplayMetrics());
-        mLeftRightTextWithIconPadding = (int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP,
-                                                                       LEFT_RIGHT_TEXT_WITH_ICON_PADDING,
-                                                                       res.getDisplayMetrics());
-        mTopBottomTextWithIconPadding = (int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP,
-                                                                       TOP_BOTTOM_TEXT_WITH_ICON_PADDING,
-                                                                       res.getDisplayMetrics());
-        mIconTextPadding = (int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP,
-                                                          ICON_TEXT_PADDING,
-                                                          res.getDisplayMetrics());
-        mIconSize = (int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP,
-                                                   ICON_SIZE,
-                                                   res.getDisplayMetrics());
 
-        GeckoAppShell.registerGeckoEventListener("Prompt:Show", this);
+        Resources res = GeckoApp.mAppContext.getResources();
+        mGroupPaddingSize = (int) (res.getDimension(R.dimen.prompt_service_group_padding_size));
+        mLeftRightTextWithIconPadding = (int) (res.getDimension(R.dimen.prompt_service_left_right_text_with_icon_padding));
+        mTopBottomTextWithIconPadding = (int) (res.getDimension(R.dimen.prompt_service_top_bottom_text_with_icon_padding));
+        mIconTextPadding = (int) (res.getDimension(R.dimen.prompt_service_icon_text_padding));
+        mIconSize = (int) (res.getDimension(R.dimen.prompt_service_icon_size));
+
+        GeckoAppShell.getEventDispatcher().registerEventListener("Prompt:Show", this);
     }
 
     void destroy() {
-        GeckoAppShell.unregisterGeckoEventListener("Prompt:Show", this);
+        GeckoAppShell.getEventDispatcher().unregisterEventListener("Prompt:Show", this);
+    }
+
+    private static String formatDateString(String dateFormat, Calendar calendar) {
+        return new SimpleDateFormat(dateFormat).format(calendar.getTime());
     }
 
     private class PromptButton {
@@ -102,6 +95,8 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
         private String label = "";
         private String type  = "";
         private String hint  = "";
+        private Boolean autofocus = false;
+        private String value = "";
         private JSONObject mJSONInput = null;
         private View view = null;
 
@@ -116,9 +111,15 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
             try {
                 hint  = aJSONInput.getString("hint");
             } catch(Exception ex) { }
+            try {
+                value  = aJSONInput.getString("value");
+            } catch(Exception ex) { }
+            try {
+                autofocus  = aJSONInput.getBoolean("autofocus");
+            } catch(Exception ex) { }
         }
 
-        public View getView() {
+        public View getView() throws UnsupportedOperationException {
             if (type.equals("checkbox")) {
                 CheckBox checkbox = new CheckBox(GeckoApp.mAppContext);
                 checkbox.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
@@ -128,6 +129,53 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
                     checkbox.setChecked(value);
                 } catch(Exception ex) { }
                 view = (View)checkbox;
+            } else if (type.equals("date")) {
+                try {
+                    DateTimePicker input = new DateTimePicker(GeckoApp.mAppContext, "yyyy-MM-dd", value,
+                                                              DateTimePicker.pickersState.DATE);
+                    input.toggleCalendar(true);
+                    view = (View)input;
+                } catch (UnsupportedOperationException ex) {
+                    // We can't use our custom version of the DatePicker widget because the sdk is too old.
+                    // But we can fallback on the native one.
+                    DatePicker input = new DatePicker(GeckoApp.mAppContext);
+                    try {
+                        if (!value.equals("")) {
+                            GregorianCalendar calendar = new GregorianCalendar();
+                            calendar.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(value));
+                            input.updateDate(calendar.get(Calendar.YEAR),
+                                             calendar.get(Calendar.MONTH),
+                                             calendar.get(Calendar.DAY_OF_MONTH));
+                        }
+                    } catch (Exception e) {
+                        Log.e(LOGTAG, "error parsing format string: " + e);
+                    }
+                    view = (View)input;
+                }
+            } else if (type.equals("week")) {
+                DateTimePicker input = new DateTimePicker(GeckoApp.mAppContext, "yyyy-'W'ww", value,
+                                                          DateTimePicker.pickersState.WEEK);
+                view = (View)input;
+            } else if (type.equals("time")) {
+                TimePicker input = new TimePicker(GeckoApp.mAppContext);
+                if (!value.equals("")) {
+                    try {
+                        GregorianCalendar calendar = new GregorianCalendar();
+                        calendar.setTime(new SimpleDateFormat("kk:mm").parse(value));
+                        input.setCurrentHour(calendar.get(GregorianCalendar.HOUR_OF_DAY));
+                        input.setCurrentMinute(calendar.get(GregorianCalendar.MINUTE));
+                    } catch (Exception e) { }
+                }
+                view = (View)input;
+            } else if (type.equals("datetime-local") || type.equals("datetime")) {
+                DateTimePicker input = new DateTimePicker(GeckoApp.mAppContext, "yyyy-MM-dd kk:mm", value,
+                                                          DateTimePicker.pickersState.DATETIME);
+                input.toggleCalendar(true);
+                view = (View)input;
+            } else if (type.equals("month")) {
+                DateTimePicker input = new DateTimePicker(GeckoApp.mAppContext, "yyyy-MM", value,
+                                                          DateTimePicker.pickersState.MONTH);
+                view = (View)input;
             } else if (type.equals("textbox") || this.type.equals("password")) {
                 EditText input = new EditText(GeckoApp.mAppContext);
                 int inputtype = InputType.TYPE_CLASS_TEXT;
@@ -144,6 +192,18 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
                 if (!hint.equals("")) {
                     input.setHint(hint);
                 }
+
+                if (autofocus) {
+                    input.setOnFocusChangeListener(new View.OnFocusChangeListener() {
+                        public void onFocusChange(View v, boolean hasFocus) {
+                            if (hasFocus) {
+                                ((InputMethodManager) GeckoApp.mAppContext.getSystemService(Context.INPUT_METHOD_SERVICE)).showSoftInput(v, 0);
+                            }
+                        }
+                    });
+                    input.requestFocus();
+                }
+
                 view = (View)input;
             } else if (type.equals("menulist")) {
                 Spinner spinner = new Spinner(GeckoApp.mAppContext);
@@ -162,7 +222,7 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
         public String getName() {
             return type;
         }
-    
+
         public String getValue() {
             if (this.type.equals("checkbox")) {
                 CheckBox checkbox = (CheckBox)view;
@@ -173,6 +233,35 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
             } else if (type.equals("menulist")) {
                 Spinner spinner = (Spinner)view;
                 return Integer.toString(spinner.getSelectedItemPosition());
+            } else if (type.equals("time")) {
+                TimePicker tp = (TimePicker)view;
+                GregorianCalendar calendar =
+                    new GregorianCalendar(0,0,0,tp.getCurrentHour(),tp.getCurrentMinute());
+                return formatDateString("kk:mm",calendar);
+            } else if (android.os.Build.VERSION.SDK_INT < 11 && type.equals("date")) {
+                // We can't use the custom DateTimePicker with a sdk older than 11.
+                // Fallback on the native DatePicker.
+                DatePicker dp = (DatePicker)view;
+                GregorianCalendar calendar =
+                    new GregorianCalendar(dp.getYear(),dp.getMonth(),dp.getDayOfMonth());
+                return formatDateString("yyyy-MM-dd",calendar);
+            } else {
+                DateTimePicker dp = (DateTimePicker)view;
+                GregorianCalendar calendar = new GregorianCalendar();
+                calendar.setTimeInMillis(dp.getTimeInMillis());
+                if (type.equals("date")) {
+                    return formatDateString("yyyy-MM-dd",calendar);
+                } else if (type.equals("week")) {
+                    return formatDateString("yyyy-'W'ww",calendar);
+                } else if (type.equals("datetime-local")) {
+                    return formatDateString("yyyy-MM-dd kk:mm",calendar);
+                } else if (type.equals("datetime")) {
+                    calendar.set(GregorianCalendar.ZONE_OFFSET,0);
+                    calendar.setTimeInMillis(dp.getTimeInMillis());
+                    return formatDateString("yyyy-MM-dd kk:mm",calendar);
+                } else if (type.equals("month")) {
+                    return formatDateString("yyyy-MM",calendar);
+                }
             }
             return "";
         }
@@ -195,13 +284,17 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
         String promptServiceResult = "";
         try {
             promptServiceResult = waitForReturn();
-        } catch (InterruptedException e) {
-            Log.i(LOGTAG, "showing prompt ",  e);
-        }
+        } catch (InterruptedException e) { }
         return promptServiceResult;
     }
 
-    public void show(String aTitle, String aText, PromptButton[] aButtons, PromptListItem[] aMenuList, boolean aMultipleSelection) {
+    public void show(String aTitle, String aText, PromptListItem[] aMenuList, boolean aMultipleSelection) {
+        GeckoApp.assertOnUiThread();
+
+        final LayerView layerView = GeckoApp.mAppContext.getLayerView();
+        // treat actions that show a dialog as if preventDefault by content to prevent panning
+        layerView.abortPanning();
+
         final AlertDialog.Builder builder = new AlertDialog.Builder(GeckoApp.mAppContext);
         if (!aTitle.equals("")) {
             builder.setTitle(aTitle);
@@ -216,7 +309,7 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
             int resourceId = android.R.layout.select_dialog_item;
             if (mSelected != null && mSelected.length > 0) {
                 if (aMultipleSelection) {
-                    resourceId = android.R.layout.select_dialog_multichoice;
+                    resourceId = R.layout.select_dialog_multichoice;
                 } else {
                     resourceId = android.R.layout.select_dialog_singlechoice;
                 }
@@ -246,26 +339,40 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
                 mSelected = null;
             }
         } else if (length == 1) {
-            builder.setView(mInputs[0].getView());
+            try {
+                builder.setView(mInputs[0].getView());
+            } catch(UnsupportedOperationException ex) {
+                // We cannot display these input widgets with this sdk version,
+                // do not display any dialog and finish the prompt now.
+                finishDialog("{\"button\": -1}");
+                return;
+            }
         } else if (length > 1) {
             LinearLayout linearLayout = new LinearLayout(GeckoApp.mAppContext);
             linearLayout.setOrientation(LinearLayout.VERTICAL);
-            for (int i = 0; i < length; i++) {
-                View content = mInputs[i].getView();
-                linearLayout.addView(content);
+            try {
+                for (int i = 0; i < length; i++) {
+                    View content = mInputs[i].getView();
+                    linearLayout.addView(content);
+                }
+            } catch(UnsupportedOperationException ex) {
+                // We cannot display these input widgets with this sdk version,
+                // do not display any dialog and finish the prompt now.
+                finishDialog("{\"button\": -1}");
+                return;
             }
             builder.setView((View)linearLayout);
         }
 
-        length = aButtons == null ? 0 : aButtons.length;
+        length = mButtons == null ? 0 : mButtons.length;
         if (length > 0) {
-            builder.setPositiveButton(aButtons[0].label, this);
-        }
-        if (length > 1) {
-            builder.setNeutralButton(aButtons[1].label, this);
-        }
-        if (length > 2) {
-            builder.setNegativeButton(aButtons[2].label, this);
+            builder.setPositiveButton(mButtons[0], this);
+            if (length > 1) {
+                builder.setNeutralButton(mButtons[1], this);
+                if (length > 2) {
+                    builder.setNegativeButton(mButtons[2], this);
+                }
+            }
         }
 
         mDialog = builder.create();
@@ -274,6 +381,7 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
     }
 
     public void onClick(DialogInterface aDialog, int aWhich) {
+        GeckoApp.assertOnUiThread();
         JSONObject ret = new JSONObject();
         try {
             int button = -1;
@@ -314,11 +422,14 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
     }
 
     private boolean[] mSelected = null;
+
     public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
+        GeckoApp.assertOnUiThread();
         mSelected[position] = !mSelected[position];
     }
 
     public void onCancel(DialogInterface aDialog) {
+        GeckoApp.assertOnUiThread();
         JSONObject ret = new JSONObject();
         try {
             ret.put("button", -1);
@@ -340,11 +451,14 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
 
     public void finishDialog(String aReturn) {
         mInputs = null;
+        mButtons = null;
         mDialog = null;
         mSelected = null;
         try {
             mPromptQueue.put(aReturn);
-        } catch(Exception ex) { }
+        } catch(Exception ex) {
+            Log.d(LOGTAG, "mPromptQueue not ready yet");
+        }
     }
 
     private void processMessage(JSONObject geckoObject) {
@@ -362,10 +476,10 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
             buttons = geckoObject.getJSONArray("buttons");
         } catch(Exception ex) { }
         int length = buttons.length();
-        PromptButton[] promptbuttons = new PromptButton[length];
+        mButtons = new String[length];
         for (int i = 0; i < length; i++) {
             try {
-                promptbuttons[i] = new PromptButton(buttons.getJSONObject(i));
+                mButtons[i] = buttons.getJSONObject(i).getString("label");
             } catch(Exception ex) { }
         }
 
@@ -376,6 +490,7 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
         length = inputs.length();
         mInputs = new PromptInput[length];
         for (int i = 0; i < length; i++) {
+            Log.d(LOGTAG, "creating new input");
             try {
                 mInputs[i] = new PromptInput(inputs.getJSONObject(i));
             } catch(Exception ex) { }
@@ -387,7 +502,7 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
         try {
             multiple = geckoObject.getBoolean("multiple");
         } catch(Exception ex) { }
-        show(title, text, promptbuttons, menuitems, multiple);
+        show(title, text, menuitems, multiple);
     }
 
     private String[] getStringArray(JSONObject aObject, String aName) {
@@ -464,24 +579,10 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
         private static final int VIEW_TYPE_COUNT = 2;
 
         public ListView listView = null;
-    	private PromptListItem[] mList;
-    	private int mResourceId = -1;
-    	PromptListAdapter(Context context, int textViewResourceId, PromptListItem[] objects) {
+        private int mResourceId = -1;
+        PromptListAdapter(Context context, int textViewResourceId, PromptListItem[] objects) {
             super(context, textViewResourceId, objects);
-            mList = objects;
             mResourceId = textViewResourceId;
-    	}
-
-        public int getCount() {
-            return mList.length;
-        }
-
-        public PromptListItem getItem(int position) {
-            return mList[position];
-        }
-
-        public long getItemId(int position) {
-            return mList[position].id;
         }
 
         @Override
@@ -545,6 +646,7 @@ public class PromptService implements OnClickListener, OnCancelListener, OnItemC
                           viewHolder.paddingBottom);
         }
 
+        @Override
         public View getView(int position, View convertView, ViewGroup parent) {
             PromptListItem item = getItem(position);
             ViewHolder viewHolder = null;
