@@ -39,6 +39,7 @@ let AboutReader = function(doc, win) {
   this._creditsElementRef = Cu.getWeakReference(doc.getElementById("reader-credits"));
   this._contentElementRef = Cu.getWeakReference(doc.getElementById("reader-content"));
   this._toolbarElementRef = Cu.getWeakReference(doc.getElementById("reader-toolbar"));
+  this._messageElementRef = Cu.getWeakReference(doc.getElementById("reader-message"));
 
   this._toolbarEnabled = false;
 
@@ -136,7 +137,11 @@ AboutReader.prototype = {
     return this._toolbarElementRef.get();
   },
 
-  observe: function(aMessage, aTopic, aData) {
+  get _messageElement() {
+    return this._messageElementRef.get();
+  },
+
+  observe: function Reader_observe(aMessage, aTopic, aData) {
     switch(aTopic) {
       case "Reader:FaviconReturn": {
         let info = JSON.parse(aData);
@@ -318,6 +323,16 @@ AboutReader.prototype = {
       return;
 
     this._toolbarElement.classList.toggle("toolbar-hidden");
+
+    if (!visible && !this._hasUsedToolbar) {
+      this._hasUsedToolbar = Services.prefs.getBoolPref("reader.has_used_toolbar");
+      if (!this._hasUsedToolbar) {
+        gChromeWin.NativeWindow.toast.show(gStrings.GetStringFromName("aboutReader.toolbarTip"), "short");
+
+        Services.prefs.setBoolPref("reader.has_used_toolbar", true);
+        this._hasUsedToolbar = true;
+      }
+    }
   },
 
   _toggleToolbarVisibility: function Reader_toggleToolbarVisibility(visible) {
@@ -325,7 +340,7 @@ AboutReader.prototype = {
   },
 
   _loadFromURL: function Reader_loadFromURL(url) {
-    this._showProgress();
+    this._showProgressDelayed();
 
     gChromeWin.Reader.parseDocumentFromURL(url, function(article) {
       if (article)
@@ -336,7 +351,7 @@ AboutReader.prototype = {
   },
 
   _loadFromTab: function Reader_loadFromTab(tabId, url) {
-    this._showProgress();
+    this._showProgressDelayed();
 
     gChromeWin.Reader.getArticleForTab(tabId, url, function(article) {
       if (article)
@@ -414,13 +429,17 @@ AboutReader.prototype = {
 
   _showError: function Reader_showError(error) {
     this._headerElement.style.display = "none";
-    this._contentElement.innerHTML = error;
-    this._contentElement.style.display = "block";
+    this._contentElement.style.display = "none";
+
+    this._messageElement.innerHTML = error;
+    this._messageElement.style.display = "block";
 
     this._doc.title = error;
   },
 
   _showContent: function Reader_showContent(article) {
+    this._messageElement.style.display = "none";
+
     this._article = article;
 
     let articleUri = Services.io.newURI(article.url, null, null);
@@ -455,10 +474,19 @@ AboutReader.prototype = {
     this._contentElement.style.display = "none";
   },
 
-  _showProgress: function Reader_showProgress() {
-    this._headerElement.style.display = "none";
-    this._contentElement.innerHTML = gStrings.GetStringFromName("aboutReader.loading");
-    this._contentElement.style.display = "block";
+  _showProgressDelayed: function Reader_showProgressDelayed() {
+    this._win.setTimeout(function() {
+      // Article has already been loaded, no need to show
+      // progress anymore.
+      if (this._article)
+        return;
+
+      this._headerElement.style.display = "none";
+      this._contentElement.style.display = "none";
+
+      this._messageElement.innerHTML = gStrings.GetStringFromName("aboutReader.loading");
+      this._messageElement.style.display = "block";
+    }.bind(this), 300);
   },
 
   _decodeQueryString: function Reader_decodeQueryString(url) {
