@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim: set ts=4 sw=4 et cindent: */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et cindent: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -50,6 +50,7 @@
 #include "nsITimedChannel.h"
 #include "nsChannelProperties.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/unused.h"
 
 #include "nsISimpleEnumerator.h"
 #include "nsStringAPI.h"
@@ -57,13 +58,15 @@
 #include "prlog.h"
 #include "prtime.h"
 
+using namespace mozilla;
+
 namespace TestProtocols {
 
 #if defined(PR_LOGGING)
 //
 // set NSPR_LOG_MODULES=Test:5
 //
-static PRLogModuleInfo *gTestLog = nsnull;
+static PRLogModuleInfo *gTestLog = nullptr;
 #endif
 #define LOG(args) PR_LOG(gTestLog, PR_LOG_DEBUG, args)
 
@@ -74,7 +77,7 @@ static int gKeepRunning = 0;
 static bool gVerbose = false;
 static bool gAskUserForInput = false;
 static bool gResume = false;
-static PRUint64 gStartAt = 0;
+static uint64_t gStartAt = 0;
 
 static const char* gEntityID;
 
@@ -97,7 +100,7 @@ SetHttpProxy(const char *proxy)
     NS_WARNING("invalid proxy port; must be an integer");
     return NS_ERROR_UNEXPECTED;
   }
-  nsCAutoString proxyHost;
+  nsAutoCString proxyHost;
   proxyHost = Substring(proxy, colon);
 
   nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
@@ -196,7 +199,7 @@ public:
   NS_DECL_ISUPPORTS
 
   const char* Name() { return mURLString.get(); }
-  PRInt64   mBytesRead;
+  int64_t   mBytesRead;
   PRTime    mTotalTime;
   PRTime    mConnectTime;
   nsCString mURLString;
@@ -243,7 +246,7 @@ NS_IMPL_ISUPPORTS1(TestChannelEventSink, nsIChannelEventSink)
 NS_IMETHODIMP
 TestChannelEventSink::AsyncOnChannelRedirect(nsIChannel *channel,
                                              nsIChannel *newChannel,
-                                             PRUint32 flags,
+                                             uint32_t flags,
                                              nsIAsyncVerifyRedirectCallback *callback)
 {
     LOG(("\n+++ TestChannelEventSink::OnChannelRedirect (with flags %x) +++\n",
@@ -280,7 +283,7 @@ NS_IMETHODIMP
 TestAuthPrompt::Prompt(const PRUnichar *dialogTitle,
                        const PRUnichar *text,
                        const PRUnichar *passwordRealm,
-                       PRUint32 savePassword,
+                       uint32_t savePassword,
                        const PRUnichar *defaultText,
                        PRUnichar **result,
                        bool *_retval)
@@ -293,7 +296,7 @@ NS_IMETHODIMP
 TestAuthPrompt::PromptUsernameAndPassword(const PRUnichar *dialogTitle,
                                           const PRUnichar *dialogText,
                                           const PRUnichar *passwordRealm,
-                                          PRUint32 savePassword,
+                                          uint32_t savePassword,
                                           PRUnichar **user,
                                           PRUnichar **pwd,
                                           bool *_retval)
@@ -307,7 +310,7 @@ TestAuthPrompt::PromptUsernameAndPassword(const PRUnichar *dialogTitle,
     int n;
 
     printf("Enter username: ");
-    fgets(buf, sizeof(buf), stdin);
+    unused << fgets(buf, sizeof(buf), stdin);
     n = strlen(buf);
     buf[n-1] = '\0'; // trim trailing newline
     *user = NS_StringCloneData(NS_ConvertUTF8toUTF16(buf));
@@ -335,7 +338,7 @@ NS_IMETHODIMP
 TestAuthPrompt::PromptPassword(const PRUnichar *dialogTitle,
                                const PRUnichar *text,
                                const PRUnichar *passwordRealm,
-                               PRUint32 savePassword,
+                               uint32_t savePassword,
                                PRUnichar **pwd,
                                bool *_retval)
 {
@@ -378,10 +381,11 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
   if (info)
     info->mConnectTime = PR_Now() - info->mConnectTime;
 
-  if (gVerbose)
+  if (gVerbose) {
     LOG(("\nStarted loading: %s\n", info ? info->Name() : "UNKNOWN URL"));
+  }
 
-  nsCAutoString value;
+  nsAutoCString value;
 
   nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
   if (channel) {
@@ -400,11 +404,12 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
       channel->GetContentCharset(value);
       LOG(("\tContent-Charset: %s\n", value.get()));
 
-      PRInt32 length = -1;
-      if (NS_SUCCEEDED(channel->GetContentLength(&length)))
+      int32_t length = -1;
+      if (NS_SUCCEEDED(channel->GetContentLength(&length))) {
         LOG(("\tContent-Length: %d\n", length));
-      else
+      } else {
         LOG(("\tContent-Length: Unknown\n"));
+      }
     }
 
     nsCOMPtr<nsISupports> owner;
@@ -419,7 +424,7 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
                                     NS_GET_IID(nsIURI),
                                     getter_AddRefs(foo));
       if (foo) {
-          nsCAutoString spec;
+          nsAutoCString spec;
           foo->GetSpec(spec);
           LOG(("\ttest.foo: %s\n", spec.get()));
       }
@@ -427,19 +432,21 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
 
   nsCOMPtr<nsIPropertyBag2> propbag = do_QueryInterface(request);
   if (propbag) {
-      PRInt64 len;
+      int64_t len;
       nsresult rv = propbag->GetPropertyAsInt64(NS_CHANNEL_PROP_CONTENT_LENGTH,
                                                 &len);
-      if (NS_SUCCEEDED(rv))
+      if (NS_SUCCEEDED(rv)) {
           LOG(("\t64-bit length: %lli\n", len));
+      }
   }
 
   nsCOMPtr<nsIHttpChannelInternal> httpChannelInt(do_QueryInterface(request));
   if (httpChannelInt) {
-      PRUint32 majorVer, minorVer;
+      uint32_t majorVer, minorVer;
       nsresult rv = httpChannelInt->GetResponseVersion(&majorVer, &minorVer);
-      if (NS_SUCCEEDED(rv))
+      if (NS_SUCCEEDED(rv)) {
           LOG(("HTTP Response version: %u.%u\n", majorVer, minorVer));
+      }
   }
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
   if (httpChannel) {
@@ -460,7 +467,7 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
   nsCOMPtr<nsIResumableChannel> resChannel = do_QueryInterface(request);
   if (resChannel) {
       LOG(("Resumable entity identification:\n"));
-      nsCAutoString entityID;
+      nsAutoCString entityID;
       nsresult rv = resChannel->GetEntityID(entityID);
       if (NS_SUCCEEDED(rv)) {
           LOG(("\t|%s|\n", entityID.get()));
@@ -477,16 +484,16 @@ NS_IMETHODIMP
 InputTestConsumer::OnDataAvailable(nsIRequest *request, 
                                    nsISupports* context,
                                    nsIInputStream *aIStream, 
-                                   PRUint32 aSourceOffset,
-                                   PRUint32 aLength)
+                                   uint64_t aSourceOffset,
+                                   uint32_t aLength)
 {
   char buf[1025];
-  PRUint32 amt, size;
+  uint32_t amt, size;
   nsresult rv;
   URLLoadInfo* info = (URLLoadInfo*)context;
 
   while (aLength) {
-    size = NS_MIN<PRUint32>(aLength, sizeof(buf));
+    size = NS_MIN<uint32_t>(aLength, sizeof(buf));
 
     rv = aIStream->Read(buf, size, &amt);
     if (NS_FAILED(rv)) {
@@ -518,7 +525,7 @@ InputTestConsumer::OnStopRequest(nsIRequest *request, nsISupports* context,
   if (info) {
     double connectTime;
     double readTime;
-    PRUint32 httpStatus;
+    uint32_t httpStatus;
     bool bHTTPURL = false;
 
     info->mTotalTime = PR_Now() - info->mTotalTime;
@@ -533,18 +540,19 @@ InputTestConsumer::OnStopRequest(nsIRequest *request, nsISupports* context,
     }
 
     LOG(("\nFinished loading: %s  Status Code: %x\n", info->Name(), aStatus));
-    if (bHTTPURL)
-        LOG(("\tHTTP Status: %u\n", httpStatus));
-     if (NS_ERROR_UNKNOWN_HOST == aStatus ||
-         NS_ERROR_UNKNOWN_PROXY_HOST == aStatus) {
-         LOG(("\tDNS lookup failed.\n"));
-     }
+    if (bHTTPURL) {
+      LOG(("\tHTTP Status: %u\n", httpStatus));
+    }
+    if (NS_ERROR_UNKNOWN_HOST == aStatus ||
+        NS_ERROR_UNKNOWN_PROXY_HOST == aStatus) {
+      LOG(("\tDNS lookup failed.\n"));
+    }
     LOG(("\tTime to connect: %.3f seconds\n", connectTime));
     LOG(("\tTime to read: %.3f seconds.\n", readTime));
     LOG(("\tRead: %lld bytes.\n", info->mBytesRead));
-    if (info->mBytesRead == PRInt64(0)) {
+    if (info->mBytesRead == int64_t(0)) {
     } else if (readTime > 0.0) {
-      LOG(("\tThroughput: %.0f bps.\n", (PRFloat64)(info->mBytesRead*PRInt64(8))/readTime));
+      LOG(("\tThroughput: %.0f bps.\n", (double)(info->mBytesRead*int64_t(8))/readTime));
     } else {
       LOG(("\tThroughput: REAL FAST!!\n"));
     }
@@ -579,7 +587,7 @@ public:
           TestChannelEventSink *sink;
 
           sink = new TestChannelEventSink();
-          if (sink == nsnull)
+          if (sink == nullptr)
             return NS_ERROR_OUT_OF_MEMORY;
           NS_ADDREF(sink);
           rv = sink->QueryInterface(iid, result);
@@ -590,7 +598,7 @@ public:
           TestAuthPrompt *prompt;
 
           prompt = new TestAuthPrompt();
-          if (prompt == nsnull)
+          if (prompt == nullptr)
             return NS_ERROR_OUT_OF_MEMORY;
           NS_ADDREF(prompt);
           rv = prompt->QueryInterface(iid, result);
@@ -614,7 +622,7 @@ nsresult StartLoadingURL(const char* aUrlString)
     if (pService) {
         nsCOMPtr<nsIURI> pURL;
 
-        rv = pService->NewURI(nsDependentCString(aUrlString), nsnull, nsnull, getter_AddRefs(pURL));
+        rv = pService->NewURI(nsDependentCString(aUrlString), nullptr, nullptr, getter_AddRefs(pURL));
         if (NS_FAILED(rv)) {
             LOG(("ERROR: NewURI failed for %s [rv=%x]\n", aUrlString));
             return rv;
@@ -630,7 +638,7 @@ nsresult StartLoadingURL(const char* aUrlString)
 
         // Async reading thru the calls of the event sink interface
         rv = NS_NewChannel(getter_AddRefs(pChannel), pURL, pService,
-                           nsnull,     // loadGroup
+                           nullptr,     // loadGroup
                            callbacks); // notificationCallbacks
         NS_RELEASE(callbacks);
         if (NS_FAILED(rv)) {
@@ -646,8 +654,9 @@ nsresult StartLoadingURL(const char* aUrlString)
         if (props) {
             rv = props->SetPropertyAsInterface(NS_LITERAL_STRING("test.foo"),
                                                pURL);
-            if (NS_SUCCEEDED(rv))
+            if (NS_SUCCEEDED(rv)) {
                 LOG(("set prop 'test.foo'\n"));
+            }
         }
 
         /* 
@@ -687,7 +696,7 @@ nsresult StartLoadingURL(const char* aUrlString)
                 NS_ERROR("Channel is not resumable!");
                 return NS_ERROR_UNEXPECTED;
             }
-            nsCAutoString id;
+            nsAutoCString id;
             if (gEntityID)
                 id = gEntityID;
             LOG(("* resuming at %llu bytes, with entity id |%s|\n", gStartAt, id.get()));
@@ -709,13 +718,13 @@ nsresult StartLoadingURL(const char* aUrlString)
     return rv;
 }
 
-static PRInt32
+static int32_t
 FindChar(nsCString& buffer, char c)
 {
     const char *b;
-    PRInt32 len = NS_CStringGetData(buffer, &b);
+    int32_t len = NS_CStringGetData(buffer, &b);
 
-    for (PRInt32 offset = 0; offset < len; ++offset) {
+    for (int32_t offset = 0; offset < len; ++offset) {
         if (b[offset] == c)
             return offset;
     }
@@ -728,7 +737,7 @@ static void
 StripChar(nsCString& buffer, char c)
 {
     const char *b;
-    PRUint32 len = NS_CStringGetData(buffer, &b) - 1;
+    uint32_t len = NS_CStringGetData(buffer, &b) - 1;
 
     for (; len > 0; --len) {
         if (b[len] == c) {
@@ -741,7 +750,7 @@ StripChar(nsCString& buffer, char c)
 nsresult LoadURLsFromFile(char *aFileName)
 {
     nsresult rv = NS_OK;
-    PRInt32 len, offset;
+    int32_t len, offset;
     PRFileDesc* fd;
     char buffer[1024];
     nsCString fileBuffer;
@@ -793,7 +802,7 @@ nsresult LoadURLFromConsole()
 {
     char buffer[1024];
     printf("Enter URL (\"q\" to start): ");
-    scanf("%s", buffer);
+    unused << scanf("%s", buffer);
     if (buffer[0]=='q') 
         gAskUserForInput = false;
     else
@@ -828,8 +837,8 @@ main(int argc, char* argv[])
       up the event queues. Copied from TestSocketIO.cpp
     */
 
-    rv = NS_InitXPCOM2(nsnull, nsnull, nsnull);
-    if (NS_FAILED(rv)) return rv;
+    rv = NS_InitXPCOM2(nullptr, nullptr, nullptr);
+    if (NS_FAILED(rv)) return -1;
 
     {
         int i;
@@ -880,6 +889,6 @@ main(int argc, char* argv[])
         PumpEvents();
     } // this scopes the nsCOMPtrs
     // no nsCOMPtrs are allowed to be alive when you call NS_ShutdownXPCOM
-    NS_ShutdownXPCOM(nsnull);
-    return rv;
+    NS_ShutdownXPCOM(nullptr);
+    return NS_FAILED(rv) ? -1 : 0;
 }
