@@ -1,7 +1,7 @@
 import os
 from setuptools import setup, find_packages
 
-version = '0.2'
+version = '0.5.10'
 
 # get documentation from the README
 try:
@@ -11,9 +11,11 @@ except (OSError, IOError):
     description = ''
 
 # dependencies
-deps = ['manifestdestiny', 'mozhttpd >= 0.3', 'mozrunner', 'datazilla']
+deps = ['manifestdestiny', 'mozhttpd >= 0.3',
+        'mozprocess >= 0.6', 'mozrunner >= 5.11',
+        'mozdevice >= 0.12']
 
-setup(name='marionette',
+setup(name='marionette_client',
       version=version,
       description="Marionette test automation client",
       long_description=description,
@@ -24,8 +26,14 @@ setup(name='marionette',
       url='https://wiki.mozilla.org/Auto-tools/Projects/Marionette',
       license='MPL',
       packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
+      package_data={'marionette': ['touch/*.js']},
       include_package_data=True,
       zip_safe=False,
+      entry_points="""
+      # -*- Entry points: -*-
+      [console_scripts]
+      marionette = marionette.runtests:cli
+      """,
       install_requires=deps,
       )
 
