@@ -23,6 +23,7 @@ utils.window = content;
 loader.loadSubScript("chrome://marionette/content/EventUtils.js", utils);
 loader.loadSubScript("chrome://marionette/content/ChromeUtils.js", utils);
 loader.loadSubScript("chrome://marionette/content/atoms.js", utils);
+loader.loadSubScript("chrome://marionette/content/marionette-sendkeys.js", utils);
 
 loader.loadSubScript("chrome://specialpowers/content/specialpowersAPI.js");
 loader.loadSubScript("chrome://specialpowers/content/specialpowers.js");
@@ -32,7 +33,7 @@ let marionettePerf = new MarionettePerfData();
 
 let isB2G = false;
 
-let marionetteTimeout = null;
+let marionetteTestName;
 let winUtil = content.QueryInterface(Ci.nsIInterfaceRequestor)
                      .getInterface(Ci.nsIDOMWindowUtils);
 let listenerId = null; //unique ID of this listener
@@ -49,6 +50,8 @@ let sandbox;
 let asyncTestRunning = false;
 let asyncTestCommandId;
 let asyncTestTimeoutId;
+//timer for doc changes
+let checkTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
 
 /**
  * Called when listener is first started up. 
@@ -56,8 +59,11 @@ let asyncTestTimeoutId;
  * If the actor returns an ID, we start the listeners. Otherwise, nothing happens.
  */
 function registerSelf() {
-  let register = sendSyncMessage("Marionette:register", {value: winUtil.outerWindowID, href: content.location.href});
-  
+  Services.io.manageOfflineStatus = false;
+  Services.io.offline = false;
+  let msg = {value: winUtil.outerWindowID, href: content.location.href};
+  let register = sendSyncMessage("Marionette:register", msg);
+
   if (register[0]) {
     listenerId = register[0];
     startListeners();
@@ -84,13 +90,13 @@ function removeMessageListenerId(messageName, handler) {
 function startListeners() {
   addMessageListenerId("Marionette:newSession", newSession);
   addMessageListenerId("Marionette:executeScript", executeScript);
-  addMessageListenerId("Marionette:setScriptTimeout", setScriptTimeout);
   addMessageListenerId("Marionette:executeAsyncScript", executeAsyncScript);
   addMessageListenerId("Marionette:executeJSScript", executeJSScript);
   addMessageListenerId("Marionette:setSearchTimeout", setSearchTimeout);
   addMessageListenerId("Marionette:goUrl", goUrl);
   addMessageListenerId("Marionette:getUrl", getUrl);
   addMessageListenerId("Marionette:getTitle", getTitle);
+  addMessageListenerId("Marionette:getPageSource", getPageSource);
   addMessageListenerId("Marionette:goBack", goBack);
   addMessageListenerId("Marionette:goForward", goForward);
   addMessageListenerId("Marionette:refresh", refresh);
@@ -99,16 +105,27 @@ function startListeners() {
   addMessageListenerId("Marionette:clickElement", clickElement);
   addMessageListenerId("Marionette:getElementAttribute", getElementAttribute);
   addMessageListenerId("Marionette:getElementText", getElementText);
+  addMessageListenerId("Marionette:getElementTagName", getElementTagName);
   addMessageListenerId("Marionette:isElementDisplayed", isElementDisplayed);
+  addMessageListenerId("Marionette:getElementSize", getElementSize);
   addMessageListenerId("Marionette:isElementEnabled", isElementEnabled);
   addMessageListenerId("Marionette:isElementSelected", isElementSelected);
   addMessageListenerId("Marionette:sendKeysToElement", sendKeysToElement);
+  addMessageListenerId("Marionette:getElementPosition", getElementPosition);
   addMessageListenerId("Marionette:clearElement", clearElement);
   addMessageListenerId("Marionette:switchToFrame", switchToFrame);
   addMessageListenerId("Marionette:deleteSession", deleteSession);
   addMessageListenerId("Marionette:sleepSession", sleepSession);
   addMessageListenerId("Marionette:emulatorCmdResult", emulatorCmdResult);
   addMessageListenerId("Marionette:importScript", importScript);
+  addMessageListenerId("Marionette:getAppCacheStatus", getAppCacheStatus);
+  addMessageListenerId("Marionette:setTestName", setTestName);
+  addMessageListenerId("Marionette:setState", setState);
+  addMessageListenerId("Marionette:screenShot", screenShot);
+  addMessageListenerId("Marionette:addCookie", addCookie);
+  addMessageListenerId("Marionette:getAllCookies", getAllCookies);
+  addMessageListenerId("Marionette:deleteAllCookies", deleteAllCookies);
+  addMessageListenerId("Marionette:deleteCookie", deleteCookie);
 }
 
 /**
@@ -130,10 +147,25 @@ function sleepSession(msg) {
   addMessageListener("Marionette:restart", restart);
 }
 
+/**
+ * Sets script-wide state variables; used after frame switching.
+ */
+function setState(msg) {
+  marionetteTimeout = msg.json.scriptTimeout;
+  try {
+    elementManager.setSearchTimeout(msg.json.searchTimeout);
+  }
+  catch (e) {
+    sendError(e.message, e.code, e.stack, msg.json.command_id);
+    return;
+  }
+  sendOk(msg.json.command_id);
+}
+
 /**
  * Restarts all our listeners after this listener was put to sleep
  */
-function restart() {
+function restart(msg) {
   removeMessageListener("Marionette:restart", restart);
   registerSelf();
 }
@@ -144,12 +176,12 @@ function restart() {
 function deleteSession(msg) {
   removeMessageListenerId("Marionette:newSession", newSession);
   removeMessageListenerId("Marionette:executeScript", executeScript);
-  removeMessageListenerId("Marionette:setScriptTimeout", setScriptTimeout);
   removeMessageListenerId("Marionette:executeAsyncScript", executeAsyncScript);
   removeMessageListenerId("Marionette:executeJSScript", executeJSScript);
   removeMessageListenerId("Marionette:setSearchTimeout", setSearchTimeout);
   removeMessageListenerId("Marionette:goUrl", goUrl);
   removeMessageListenerId("Marionette:getTitle", getTitle);
+  removeMessageListenerId("Marionette:getPageSource", getPageSource);
   removeMessageListenerId("Marionette:getUrl", getUrl);
   removeMessageListenerId("Marionette:goBack", goBack);
   removeMessageListenerId("Marionette:goForward", goForward);
@@ -158,18 +190,31 @@ function deleteSession(msg) {
   removeMessageListenerId("Marionette:findElementsContent", findElementsContent);
   removeMessageListenerId("Marionette:clickElement", clickElement);
   removeMessageListenerId("Marionette:getElementAttribute", getElementAttribute);
-  removeMessageListenerId("Marionette:getElementText", getElementText);
+  removeMessageListenerId("Marionette:getElementTagName", getElementTagName);
   removeMessageListenerId("Marionette:isElementDisplayed", isElementDisplayed);
+  removeMessageListenerId("Marionette:getElementSize", getElementSize);
   removeMessageListenerId("Marionette:isElementEnabled", isElementEnabled);
   removeMessageListenerId("Marionette:isElementSelected", isElementSelected);
   removeMessageListenerId("Marionette:sendKeysToElement", sendKeysToElement);
+  removeMessageListenerId("Marionette:getElementPosition", getElementPosition);
   removeMessageListenerId("Marionette:clearElement", clearElement);
   removeMessageListenerId("Marionette:switchToFrame", switchToFrame);
   removeMessageListenerId("Marionette:deleteSession", deleteSession);
   removeMessageListenerId("Marionette:sleepSession", sleepSession);
   removeMessageListenerId("Marionette:emulatorCmdResult", emulatorCmdResult);
   removeMessageListenerId("Marionette:importScript", importScript);
+  removeMessageListenerId("Marionette:getAppCacheStatus", getAppCacheStatus);
+  removeMessageListenerId("Marionette:setTestName", setTestName);
+  removeMessageListenerId("Marionette:setState", setState);
+  removeMessageListenerId("Marionette:screenShot", screenShot);
+  removeMessageListenerId("Marionette:addCookie", addCookie);
+  removeMessageListenerId("Marionette:getAllCookies", getAllCookies);
+  removeMessageListenerId("Marionette:deleteAllCookies", deleteAllCookies);
+  removeMessageListenerId("Marionette:deleteCookie", deleteCookie);
   this.elementManager.reset();
+  // reset frame to the top-most frame
+  curWindow = content;
+  curWindow.focus();
   try {
     importedScripts.remove(false);
   }
@@ -225,8 +270,7 @@ function sendError(message, status, trace, command_id) {
  */
 function resetValues() {
   sandbox = null;
-  marionetteTimeout = null;
-  curWin = content;
+  curWindow = content;
 }
 
 /**
@@ -243,7 +287,7 @@ function errUnload() {
 /**
  * Returns a content sandbox that can be used by the execute_foo functions.
  */
-function createExecuteContentSandbox(aWindow) {
+function createExecuteContentSandbox(aWindow, timeout) {
   let sandbox = new Cu.Sandbox(aWindow);
   sandbox.global = sandbox;
   sandbox.window = aWindow;
@@ -252,21 +296,22 @@ function createExecuteContentSandbox(aWindow) {
   sandbox.__proto__ = sandbox.window;
   sandbox.testUtils = utils;
 
-  let marionette = new Marionette(this, aWindow, "content", marionetteLogObj, marionettePerf);
+  let marionette = new Marionette(this, aWindow, "content",
+                                  marionetteLogObj, marionettePerf,
+                                  timeout, marionetteTestName);
   sandbox.marionette = marionette;
   marionette.exports.forEach(function(fn) {
-    sandbox[fn] = marionette[fn].bind(marionette);
+    try {
+      sandbox[fn] = marionette[fn].bind(marionette);
+    }
+    catch(e) {
+      sandbox[fn] = marionette[fn];
+    }
   });
 
   sandbox.SpecialPowers = new SpecialPowers(aWindow);
 
   sandbox.asyncComplete = function sandbox_asyncComplete(value, status) {
-    if (Object.keys(_emu_cbs).length) {
-      _emu_cbs = {};
-      value = "Emulator callback still pending when finish() called";
-      status = 500;
-    }
-
     curWindow.removeEventListener("unload", errUnload, false);
 
     /* clear all timeouts potentially generated by the script*/
@@ -278,8 +323,17 @@ function createExecuteContentSandbox(aWindow) {
                                              perf: elementManager.wrapValue(marionettePerf.getPerfData())});
     marionetteLogObj.clearLogs();
     marionettePerf.clearPerfData();
+
     if (status == 0){
-      sendResponse({value: elementManager.wrapValue(value), status: status}, asyncTestCommandId);
+      if (Object.keys(_emu_cbs).length) {
+        _emu_cbs = {};
+        sendError("Emulator callback still pending when finish() called",
+                  500, null, asyncTestCommandId);
+      }
+      else {
+        sendResponse({value: elementManager.wrapValue(value), status: status},
+                     asyncTestCommandId);
+      }
     }
     else {
       sendError(value, status, null, asyncTestCommandId);
@@ -308,12 +362,13 @@ function createExecuteContentSandbox(aWindow) {
  * or directly (for 'mochitest' like JS Marionette tests)
  */
 function executeScript(msg, directInject) {
+  let asyncTestCommandId = msg.json.command_id;
   let script = msg.json.value;
 
   if (msg.json.newSandbox || !sandbox) {
-    sandbox = createExecuteContentSandbox(curWindow);
+    sandbox = createExecuteContentSandbox(curWindow, msg.json.timeout);
     if (!sandbox) {
-      sendError("Could not create sandbox!");
+      sendError("Could not create sandbox!", asyncTestCommandId);
       return;
     }
   }
@@ -333,10 +388,10 @@ function executeScript(msg, directInject) {
       marionetteLogObj.clearLogs();
       marionettePerf.clearPerfData();
       if (res == undefined || res.passed == undefined) {
-        sendError("Marionette.finish() not called", 17, null);
+        sendError("Marionette.finish() not called", 17, null, asyncTestCommandId);
       }
       else {
-        sendResponse({value: elementManager.wrapValue(res)});
+        sendResponse({value: elementManager.wrapValue(res)}, asyncTestCommandId);
       }
     }
     else {
@@ -345,7 +400,7 @@ function executeScript(msg, directInject) {
           msg.json.args, curWindow);
       }
       catch(e) {
-        sendError(e.message, e.code, e.stack);
+        sendError(e.message, e.code, e.stack, asyncTestCommandId);
         return;
       }
 
@@ -363,20 +418,21 @@ function executeScript(msg, directInject) {
                                                perf: elementManager.wrapValue(marionettePerf.getPerfData())});
       marionetteLogObj.clearLogs();
       marionettePerf.clearPerfData();
-      sendResponse({value: elementManager.wrapValue(res)});
+      sendResponse({value: elementManager.wrapValue(res)}, asyncTestCommandId);
     }
   }
   catch (e) {
     // 17 = JavascriptException
-    sendError(e.name + ': ' + e.message, 17, e.stack);
+    sendError(e.name + ': ' + e.message, 17, e.stack, asyncTestCommandId);
   }
 }
 
 /**
- * Function to set the timeout of asynchronous scripts
+ * Sets the test name, used in logging messages.
  */
-function setScriptTimeout(msg) {
-  marionetteTimeout = msg.json.value;
+function setTestName(msg) {
+  marionetteTestName = msg.json.value;
+  sendOk(msg.json.command_id);
 }
 
 /**
@@ -390,8 +446,8 @@ function executeAsyncScript(msg) {
  * Execute pure JS test. Handles both async and sync cases.
  */
 function executeJSScript(msg) {
-  if (msg.json.timeout) {
-    executeWithCallback(msg, msg.json.timeout);
+  if (msg.json.async) {
+    executeWithCallback(msg, msg.json.async);
   }
   else {
     executeScript(msg, true);
@@ -406,18 +462,19 @@ function executeJSScript(msg) {
  * For executeAsync, it will return a response when marionetteScriptFinished/arguments[arguments.length-1] 
  * method is called, or if it times out.
  */
-function executeWithCallback(msg, timeout) {
+function executeWithCallback(msg, useFinish) {
   curWindow.addEventListener("unload", errUnload, false);
   let script = msg.json.value;
-  asyncTestCommandId = msg.json.id;
+  let asyncTestCommandId = msg.json.command_id;
 
   if (msg.json.newSandbox || !sandbox) {
-    sandbox = createExecuteContentSandbox(curWindow);
+    sandbox = createExecuteContentSandbox(curWindow, msg.json.timeout);
     if (!sandbox) {
       sendError("Could not create sandbox!");
       return;
     }
   }
+  sandbox.tag = script;
 
   // Error code 28 is scriptTimeout, but spec says execute_async should return 21 (Timeout),
   // see http://code.google.com/p/selenium/wiki/JsonWireProtocol#/session/:sessionId/execute_async.
@@ -426,8 +483,7 @@ function executeWithCallback(msg, timeout) {
   // We'll stay compatible with the Selenium code.
   asyncTestTimeoutId = curWindow.setTimeout(function() {
     sandbox.asyncComplete('timed out', 28);
-  }, marionetteTimeout);
-  sandbox.marionette.timeout = marionetteTimeout;
+  }, msg.json.timeout);
 
   curWindow.addEventListener('error', function win__onerror(evt) {
     curWindow.removeEventListener('error', win__onerror, true);
@@ -436,9 +492,9 @@ function executeWithCallback(msg, timeout) {
   }, true);
 
   let scriptSrc;
-  if (timeout) {
-    if (marionetteTimeout == null || marionetteTimeout == 0) {
-      sendError("Please set a timeout", 21, null);
+  if (useFinish) {
+    if (msg.json.timeout == null || msg.json.timeout == 0) {
+      sendError("Please set a timeout", 21, null, asyncTestCommandId);
     }
     scriptSrc = script;
   }
@@ -448,7 +504,7 @@ function executeWithCallback(msg, timeout) {
         msg.json.args, curWindow);
     }
     catch(e) {
-      sendError(e.message, e.code, e.stack);
+      sendError(e.message, e.code, e.stack, asyncTestCommandId);
       return;
     }
 
@@ -460,8 +516,8 @@ function executeWithCallback(msg, timeout) {
   try {
     asyncTestRunning = true;
     if (importedScripts.exists()) {
-      let stream = Components.classes["@mozilla.org/network/file-input-stream;1"].  
-                      createInstance(Components.interfaces.nsIFileInputStream);
+      let stream = Cc["@mozilla.org/network/file-input-stream;1"].
+                      createInstance(Ci.nsIFileInputStream);
       stream.init(importedScripts, -1, 0, 0);
       let data = NetUtil.readInputStreamToString(stream, stream.available());
       scriptSrc = data + scriptSrc;
@@ -469,7 +525,7 @@ function executeWithCallback(msg, timeout) {
     Cu.evalInSandbox(scriptSrc, sandbox, "1.8");
   } catch (e) {
     // 17 = JavascriptException
-    sendError(e.name + ': ' + e.message, 17, e.stack);
+    sandbox.asyncComplete(e.name + ': ' + e.message, 17);
   }
 }
 
@@ -481,10 +537,10 @@ function setSearchTimeout(msg) {
     elementManager.setSearchTimeout(msg.json.value);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, msg.json.command_id);
     return;
   }
-  sendOk();
+  sendOk(msg.json.command_id);
 }
 
 /**
@@ -492,31 +548,48 @@ function setSearchTimeout(msg) {
  * All other navigation is handled by the server (in chrome space).
  */
 function goUrl(msg) {
+  let command_id = msg.json.command_id;
+  addEventListener("DOMContentLoaded", function onDOMContentLoaded(event) {
+    // Prevent DOMContentLoaded events from frames from invoking this code,
+    // unless the event is coming from the frame associated with the current
+    // window (i.e., someone has used switch_to_frame).
+    if (!event.originalTarget.defaultView.frameElement || 
+        event.originalTarget.defaultView.frameElement == curWindow.frameElement) {
+      removeEventListener("DOMContentLoaded", onDOMContentLoaded, false);
+
+      let errorRegex = /about:.+(error)|(blocked)\?/;
+      if (curWindow.document.readyState == "interactive" && errorRegex.exec(curWindow.document.baseURI)) {
+        sendError("Error loading page", 13, null, command_id);
+        return;
+      }
+
+      sendOk(command_id);
+    }
+  }, false);
   curWindow.location = msg.json.value;
-  //TODO: replace this with DOMContentLoaded event listening when Bug 720714 is resolved
-  let checkTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
-  function checkLoad() { 
-    if (curWindow.document.readyState == "complete") { 
-      sendOk();
-      return;
-    } 
-    checkTimer.initWithCallback(checkLoad, 100, Ci.nsITimer.TYPE_ONE_SHOT);
-  }
-  checkTimer.initWithCallback(checkLoad, 100, Ci.nsITimer.TYPE_ONE_SHOT);
 }
 
 /**
  * Get the current URI
  */
 function getUrl(msg) {
-  sendResponse({value: curWindow.location.href});
+  sendResponse({value: curWindow.location.href}, msg.json.command_id);
 }
 
 /**
  * Get the current Title of the window
  */
 function getTitle(msg) {
-  sendResponse({value: curWindow.top.document.title});
+  sendResponse({value: curWindow.top.document.title}, msg.json.command_id);
+}
+
+/**
+ * Get the current page source 
+ */
+function getPageSource(msg) {
+  var XMLSerializer = curWindow.XMLSerializer;
+  var pageSource = new XMLSerializer().serializeToString(curWindow.document);
+  sendResponse({value: pageSource}, msg.json.command_id);
 }
 
 /**
@@ -524,7 +597,7 @@ function getTitle(msg) {
  */
 function goBack(msg) {
   curWindow.history.back();
-  sendOk();
+  sendOk(msg.json.command_id);
 }
 
 /**
@@ -532,15 +605,19 @@ function goBack(msg) {
  */
 function goForward(msg) {
   curWindow.history.forward();
-  sendOk();
+  sendOk(msg.json.command_id);
 }
 
 /**
  * Refresh the page
  */
 function refresh(msg) {
+  let command_id = msg.json.command_id;
   curWindow.location.reload(true);
-  let listen = function() { removeEventListener("DOMContentLoaded", arguments.callee, false); sendOk() } ;
+  let listen = function() {
+    removeEventListener("DOMContentLoaded", arguments.callee, false);
+    sendOk(command_id);
+  };
   addEventListener("DOMContentLoaded", listen, false);
 }
 
@@ -548,13 +625,14 @@ function refresh(msg) {
  * Find an element in the document using requested search strategy 
  */
 function findElementContent(msg) {
-  let id;
+  let command_id = msg.json.command_id;
   try {
-    let notify = function(id) { sendResponse({value:id});};
-    id = elementManager.find(curWindow, msg.json, notify, false);
+    let on_success = function(id, cmd_id) { sendResponse({value:id}, cmd_id); };
+    let on_error = sendError;
+    elementManager.find(curWindow, msg.json, on_success, on_error, false, command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -562,13 +640,14 @@ function findElementContent(msg) {
  * Find elements in the document using requested search strategy 
  */
 function findElementsContent(msg) {
-  let id;
+  let command_id = msg.json.command_id;
   try {
-    let notify = function(id) { sendResponse({value:id});};
-    id = elementManager.find(curWindow, msg.json, notify, true);
+    let on_success = function(id, cmd_id) { sendResponse({value:id}, cmd_id); };
+    let on_error = sendError;
+    elementManager.find(curWindow, msg.json, on_success, on_error, true, command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -576,14 +655,15 @@ function findElementsContent(msg) {
  * Send click event to element
  */
 function clickElement(msg) {
+  let command_id = msg.json.command_id;
   let el;
   try {
     el = elementManager.getKnownElement(msg.json.element, curWindow);
     utils.click(el);
-    sendOk();
+    sendOk(command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -591,12 +671,14 @@ function clickElement(msg) {
  * Get a given attribute of an element
  */
 function getElementAttribute(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
-    sendResponse({value: utils.getElementAttribute(el, msg.json.name)});
+    sendResponse({value: utils.getElementAttribute(el, msg.json.name)},
+                 command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -604,12 +686,27 @@ function getElementAttribute(msg) {
  * Get the text of this element. This includes text from child elements.
  */
 function getElementText(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
-    sendResponse({value: utils.getElementText(el)});
+    sendResponse({value: utils.getElementText(el)}, command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
+  }
+}
+
+/**
+ * Get the tag name of an element.
+ */
+function getElementTagName(msg) {
+  let command_id = msg.json.command_id;
+  try {
+    let el = elementManager.getKnownElement(msg.json.element, curWindow);
+    sendResponse({value: el.tagName.toLowerCase()}, command_id);
+  }
+  catch (e) {
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -617,12 +714,29 @@ function getElementText(msg) {
  * Check if element is displayed
  */
 function isElementDisplayed(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
-    sendResponse({value: utils.isElementDisplayed(el)});
+    sendResponse({value: utils.isElementDisplayed(el)}, command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
+  }
+}
+
+/**
+ * Get the size of the element and return it
+ */
+function getElementSize(msg){
+  let command_id = msg.json.command_id;
+  try {
+    let el = elementManager.getKnownElement(msg.json.element, curWindow);
+    let clientRect = el.getBoundingClientRect();  
+    sendResponse({value: {width: clientRect.width, height: clientRect.height}},
+                 command_id);
+  }
+  catch (e) {
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -630,12 +744,13 @@ function isElementDisplayed(msg) {
  * Check if element is enabled
  */
 function isElementEnabled(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
-    sendResponse({value: utils.isElementEnabled(el)});
+    sendResponse({value: utils.isElementEnabled(el)}, command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -643,12 +758,13 @@ function isElementEnabled(msg) {
  * Check if element is selected
  */
 function isElementSelected(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
-    sendResponse({value: utils.isElementSelected(el)});
+    sendResponse({value: utils.isElementSelected(el)}, command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -656,13 +772,54 @@ function isElementSelected(msg) {
  * Send keys to element
  */
 function sendKeysToElement(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
-    utils.sendKeysToElement(el, msg.json.value);
-    sendOk();
+    utils.type(curWindow.document, el, msg.json.value.join(""), true);
+    sendOk(command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
+  }
+}
+
+/**
+ * Get the position of an element
+ */
+function getElementPosition(msg) {
+  let command_id = msg.json.command_id;
+  try{
+    let el = elementManager.getKnownElement(msg.json.element, curWindow);
+    var x = el.offsetLeft;
+    var y = el.offsetTop;
+    var elementParent = el.offsetParent;
+    while (elementParent != null) {
+      if (elementParent.tagName == "TABLE") {
+        var parentBorder = parseInt(elementParent.border);
+        if (isNaN(parentBorder)) {
+          var parentFrame = elementParent.getAttribute('frame');
+          if (parentFrame != null) {
+            x += 1;
+            y += 1;
+          }
+        } else if (parentBorder > 0) {
+          x += parentBorder;
+          y += parentBorder;
+        }
+      }
+      x += elementParent.offsetLeft;
+      y += elementParent.offsetTop;
+      elementParent = elementParent.offsetParent;
+    }
+
+    let location = {};
+    location.x = x;
+    location.y = y;
+
+    sendResponse({value: location}, command_id);
+  }
+  catch (e) {
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -670,13 +827,14 @@ function sendKeysToElement(msg) {
  * Clear the text of an element
  */
 function clearElement(msg) {
+  let command_id = msg.json.command_id;
   try {
     let el = elementManager.getKnownElement(msg.json.element, curWindow);
     utils.clearElement(el);
-    sendOk();
+    sendOk(command_id);
   }
   catch (e) {
-    sendError(e.message, e.code, e.stack);
+    sendError(e.message, e.code, e.stack, command_id);
   }
 }
 
@@ -685,62 +843,231 @@ function clearElement(msg) {
  * its index in window.frames, or the iframe's name or id.
  */
 function switchToFrame(msg) {
+  let command_id = msg.json.command_id;
+  function checkLoad() { 
+    let errorRegex = /about:.+(error)|(blocked)\?/;
+    if (curWindow.document.readyState == "complete") {
+      sendOk(command_id);
+      return;
+    } 
+    else if (curWindow.document.readyState == "interactive" && errorRegex.exec(curWindow.document.baseURI)) {
+      sendError("Error loading page", 13, null, command_id);
+      return;
+    }
+    checkTimer.initWithCallback(checkLoad, 100, Ci.nsITimer.TYPE_ONE_SHOT);
+  }
   let foundFrame = null;
+  let frames = curWindow.document.getElementsByTagName("iframe");
+  //Until Bug 761935 lands, we won't have multiple nested OOP iframes. We will only have one.
+  //parWindow will refer to the iframe above the nested OOP frame.
+  let parWindow = curWindow.QueryInterface(Ci.nsIInterfaceRequestor)
+                     .getInterface(Ci.nsIDOMWindowUtils).outerWindowID;
   if ((msg.json.value == null) && (msg.json.element == null)) {
     curWindow = content;
     curWindow.focus();
-    sendOk();
+    checkTimer.initWithCallback(checkLoad, 100, Ci.nsITimer.TYPE_ONE_SHOT);
     return;
   }
   if (msg.json.element != undefined) {
     if (elementManager.seenItems[msg.json.element] != undefined) {
       let wantedFrame = elementManager.getKnownElement(msg.json.element, curWindow); //HTMLIFrameElement
-      let numFrames = curWindow.frames.length;
-      for (let i = 0; i < numFrames; i++) {
-        if (curWindow.frames[i].frameElement == wantedFrame) {
-          curWindow = curWindow.frames[i]; 
-          curWindow.focus();
-          sendOk();
-          return;
+      for (let i = 0; i < frames.length; i++) {
+        if (frames[i] == wantedFrame) {
+          curWindow = frames[i]; 
+          foundFrame = i;
         }
       }
     }
   }
-  switch(typeof(msg.json.value)) {
-    case "string" :
-      let foundById = null;
-      let numFrames = curWindow.frames.length;
-      for (let i = 0; i < numFrames; i++) {
-        //give precedence to name
-        let frame = curWindow.frames[i];
-        let frameElement = frame.frameElement;
-        if (frameElement.name == msg.json.value) {
-          foundFrame = i;
-          break;
-        } else if ((foundById == null) && (frameElement.id == msg.json.value)) {
-          foundById = i;
+  if (foundFrame == null) {
+    switch(typeof(msg.json.value)) {
+      case "string" :
+        let foundById = null;
+        for (let i = 0; i < frames.length; i++) {
+          //give precedence to name
+          let frame = frames[i];
+          let name = utils.getElementAttribute(frame, 'name');
+          let id = utils.getElementAttribute(frame, 'id');
+          if (name == msg.json.value) {
+            foundFrame = i;
+            break;
+          } else if ((foundById == null) && (id == msg.json.value)) {
+            foundById = i;
+          }
         }
-      }
-      if ((foundFrame == null) && (foundById != null)) {
-        foundFrame = foundById;
-      }
-      break;
-    case "number":
-      if (curWindow.frames[msg.json.value] != undefined) {
-        foundFrame = msg.json.value;
-      }
-      break;
+        if ((foundFrame == null) && (foundById != null)) {
+          foundFrame = foundById;
+          curWindow = frames[foundFrame];
+        }
+        break;
+      case "number":
+        if (frames[msg.json.value] != undefined) {
+          foundFrame = msg.json.value;
+          curWindow = frames[foundFrame];
+        }
+        break;
+    }
   }
   if (foundFrame == null) {
-    sendError("Unable to locate frame: " + msg.json.value, 8, null);
+    sendError("Unable to locate frame: " + msg.json.value, 8, null, command_id);
     return;
   }
-  curWindow = curWindow.frames[foundFrame];
-  curWindow.focus();
-  sendOk();
 
   sandbox = null;
+
+  if (curWindow.contentWindow == null) {
+    // The frame we want to switch to is a remote frame; notify our parent to handle
+    // the switch.
+    curWindow = content;
+    sendToServer('Marionette:switchToFrame', {frame: foundFrame,
+                                              win: parWindow,
+                                              command_id: command_id});
+  }
+  else {
+    curWindow = curWindow.contentWindow;
+    curWindow.focus();
+    checkTimer.initWithCallback(checkLoad, 100, Ci.nsITimer.TYPE_ONE_SHOT);
+  }
 }
+ /**
+  * Add a cookie to the document
+  */
+function addCookie(msg) {
+  cookie = msg.json.cookie;
+
+  if (!cookie.expiry) {
+    var date = new Date();
+    var thePresent = new Date(Date.now());
+    date.setYear(thePresent.getFullYear() + 20);
+    cookie.expiry = date.getTime() / 1000;  // Stored in seconds.
+  }
+
+  if (!cookie.domain) {
+    var location = curWindow.document.location;
+    cookie.domain = location.hostname;
+  }
+  else {
+    var currLocation = curWindow.location;
+    var currDomain = currLocation.host;
+    if (currDomain.indexOf(cookie.domain) == -1) {
+      sendError("You may only set cookies for the current domain", 24, null, msg.json.command_id);
+    }
+  }
+
+  // The cookie's domain may include a port. Which is bad. Remove it
+  // We'll catch ip6 addresses by mistake. Since no-one uses those
+  // this will be okay for now. See Bug 814416
+  if (cookie.domain.match(/:\d+$/)) {
+    cookie.domain = cookie.domain.replace(/:\d+$/, '');
+  }
+
+  var document = curWindow.document;
+  if (!document || !document.contentType.match(/html/i)) {
+    sendError('You may only set cookies on html documents', 25, null, msg.json.command_id);
+  }
+  var cookieManager = Cc['@mozilla.org/cookiemanager;1'].
+                        getService(Ci.nsICookieManager2);
+  cookieManager.add(cookie.domain, cookie.path, cookie.name, cookie.value,
+                   cookie.secure, false, false, cookie.expiry);
+  sendOk(msg.json.command_id);
+}
+
+/**
+ * Get All the cookies for a location
+ */
+function getAllCookies(msg) {
+  var toReturn = [];
+  var cookies = getVisibleCookies(curWindow.location);
+  for (var i = 0; i < cookies.length; i++) {
+    var cookie = cookies[i];
+    var expires = cookie.expires;
+    if (expires == 0) {  // Session cookie, don't return an expiry.
+      expires = null;
+    } else if (expires == 1) { // Date before epoch time, cap to epoch.
+      expires = 0;
+    }
+    toReturn.push({
+      'name': cookie.name,
+      'value': cookie.value,
+      'path': cookie.path,
+      'domain': cookie.host,
+      'secure': cookie.isSecure,
+      'expiry': expires
+    });
+  }
+
+  sendResponse({value: toReturn}, msg.json.command_id);
+}
+
+/**
+ * Delete a cookie by name
+ */
+function deleteCookie(msg) {
+  var toDelete = msg.json.name;
+  var cookieManager = Cc['@mozilla.org/cookiemanager;1'].
+                        getService(Ci.nsICookieManager);
+
+  var cookies = getVisibleCookies(curWindow.location);
+  for (var i = 0; i < cookies.length; i++) {
+    var cookie = cookies[i];
+    if (cookie.name == toDelete) {
+      cookieManager.remove(cookie.host, cookie.name, cookie.path, false);
+    }
+  }
+
+  sendOk(msg.json.command_id);
+}
+
+/**
+ * Delete all the visibile cookies on a page
+ */
+function deleteAllCookies(msg) {
+  let cookieManager = Cc['@mozilla.org/cookiemanager;1'].
+                        getService(Ci.nsICookieManager);
+  let cookies = getVisibleCookies(curWindow.location);
+  for (let i = 0; i < cookies.length; i++) {
+    let cookie = cookies[i];
+    cookieManager.remove(cookie.host, cookie.name, cookie.path, false);
+  }
+  sendOk(msg.json.command_id);
+}
+
+/**
+ * Get all the visible cookies from a location
+ */
+function getVisibleCookies(location) {
+  let results = [];
+  let currentPath = location.pathname;
+  if (!currentPath) currentPath = '/';
+  let isForCurrentPath = function(aPath) {
+    return currentPath.indexOf(aPath) != -1;
+  }
+
+  let cookieManager = Cc['@mozilla.org/cookiemanager;1'].
+                        getService(Ci.nsICookieManager);
+  let enumerator = cookieManager.enumerator;
+  while (enumerator.hasMoreElements()) {
+    let cookie = enumerator.getNext().QueryInterface(Ci['nsICookie']);
+
+    // Take the hostname and progressively shorten
+    let hostname = location.hostname;
+    do {
+      if ((cookie.host == '.' + hostname || cookie.host == hostname)
+          && isForCurrentPath(cookie.path)) {
+          results.push(cookie);
+          break;
+      }
+      hostname = hostname.replace(/^.*?\./, '');
+    } while (hostname.indexOf('.') != -1);
+  }
+
+  return results;
+}
+
+function getAppCacheStatus(msg) {
+  sendResponse({ value: curWindow.applicationCache.status },
+               msg.json.command_id);
+} 
 
 // emulator callbacks
 let _emu_cb_id = 0;
@@ -774,16 +1101,105 @@ function emulatorCmdResult(msg) {
 }
 
 function importScript(msg) {
+  let command_id = msg.json.command_id;
   let file;
   if (importedScripts.exists()) {
-    file = FileUtils.openFileOutputStream(importedScripts, FileUtils.MODE_APPEND | FileUtils.MODE_WRONLY);
+    file = FileUtils.openFileOutputStream(importedScripts,
+        FileUtils.MODE_APPEND | FileUtils.MODE_WRONLY);
   }
   else {
-    file = FileUtils.openFileOutputStream(importedScripts, FileUtils.MODE_WRONLY | FileUtils.MODE_CREATE);
+    //Note: The permission bits here don't actually get set (bug 804563)
+    importedScripts.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,
+                                 parseInt("0666", 8));
+    file = FileUtils.openFileOutputStream(importedScripts,
+                                          FileUtils.MODE_WRONLY | FileUtils.MODE_CREATE);
+    importedScripts.permissions = parseInt("0666", 8); //actually set permissions
   }
   file.write(msg.json.script, msg.json.script.length);
   file.close();
-  sendOk();
+  sendOk(command_id);
+}
+
+/**
+ * Saves a screenshot and returns a Base64 string
+ */
+function screenShot(msg) {
+  let node = null;
+  if (msg.json.element) {
+    try {
+      node = elementManager.getKnownElement(msg.json.element, curWindow)
+    }
+    catch (e) {
+      sendResponse(e.message, e.code, e.stack);
+      return;
+    }
+  }
+  else {
+      node = curWindow;
+  }
+  let highlights = msg.json.highlights;
+
+  var document = curWindow.document;
+  var rect, win, width, height, left, top, needsOffset;
+  // node can be either a window or an arbitrary DOM node
+  if (node == curWindow) {
+    // node is a window
+    win = node;
+    width = win.innerWidth;
+    height = win.innerHeight;
+    top = 0;
+    left = 0;
+    // offset needed for highlights to take 'outerHeight' of window into account
+    needsOffset = true;
+  }
+  else {
+    // node is an arbitrary DOM node
+    win = node.ownerDocument.defaultView;
+    rect = node.getBoundingClientRect();
+    width = rect.width;
+    height = rect.height;
+    top = rect.top;
+    left = rect.left;
+    // offset for highlights not needed as they will be relative to this node
+    needsOffset = false;
+  }
+
+  var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
+  canvas.width = width;
+  canvas.height = height;
+  var ctx = canvas.getContext("2d");
+  // Draws the DOM contents of the window to the canvas
+  ctx.drawWindow(win, left, top, width, height, 'rgb(255,255,255)');
+
+  // This section is for drawing a red rectangle around each element passed in via the highlights array
+  if (highlights) {
+    ctx.lineWidth = "2";
+    ctx.strokeStyle = "red";
+    ctx.save();
+
+    for (var i = 0; i < highlights.length; ++i) {
+      var elem = highlights[i];
+      rect = elem.getBoundingClientRect();
+
+      var offsetY = 0, offsetX = 0;
+      if (needsOffset) {
+        var offset = getChromeOffset(elem);
+        offsetX = offset.x;
+        offsetY = offset.y;
+      } else {
+        // Don't need to offset the window chrome, just make relative to containing node
+        offsetY = -top;
+        offsetX = -left;
+      }
+
+      // Draw the rectangle
+      ctx.strokeRect(rect.left + offsetX, rect.top + offsetY, rect.width, rect.height);
+    }
+  }
+
+  // Return the Base64 String back to the client bindings and they can manage
+  // saving the file to disk if it is required
+  sendResponse({value:canvas.toDataURL("image/png","")});
 }
 
 //call register self when we get loaded
