@@ -11,7 +11,10 @@ const Cu = Components.utils;
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/NetUtil.jsm");
+#ifndef MOZ_WIDGET_GONK
 Cu.import("resource://gre/modules/LightweightThemeManager.jsm");
+#endif
+Cu.import("resource://gre/modules/ctypes.jsm");
 
 // When modifying the payload in incompatible ways, please bump this version number
 const PAYLOAD_VERSION = 1;
@@ -22,11 +25,15 @@ const PREF_ENABLED = "toolkit.telemetry.enabled";
 const TELEMETRY_INTERVAL = 60000;
 // Delay before intializing telemetry (ms)
 const TELEMETRY_DELAY = 60000;
+// Delete ping files that have been lying around for longer than this.
+const MAX_PING_FILE_AGE = 7 * 24 * 60 * 60 * 1000; // 1 week
 // Constants from prio.h for nsIFileOutputStream.init
 const PR_WRONLY = 0x2;
 const PR_CREATE_FILE = 0x8;
 const PR_TRUNCATE = 0x20;
+const PR_EXCL = 0x80;
 const RW_OWNER = 0600;
+const RWX_OWNER = 0700;
 
 // MEM_HISTOGRAMS lists the memory reporters we turn into histograms.
 //
@@ -79,6 +86,8 @@ XPCOMUtils.defineLazyServiceGetter(this, "Telemetry",
 XPCOMUtils.defineLazyServiceGetter(this, "idleService",
                                    "@mozilla.org/widget/idleservice;1",
                                    "nsIIdleService");
+XPCOMUtils.defineLazyModuleGetter(this, "UpdateChannel",
+                                  "resource://gre/modules/UpdateChannel.jsm");
 
 function generateUUID() {
   let str = Cc["@mozilla.org/uuid-generator;1"].getService(Ci.nsIUUIDGenerator).generateUUID().toString();
@@ -133,44 +142,58 @@ function getSimpleMeasurements() {
            .getService(Ci.nsIJSEngineTelemetryStats)
            .telemetryValue;
 
+  let shutdownDuration = Services.startup.lastShutdownDuration;
+  if (shutdownDuration)
+    ret.shutdownDuration = shutdownDuration;
+
   return ret;
 }
 
 /**
- * Read the update channel from defaults only.  We do this to ensure that
- * the channel is tightly coupled with the application and does not apply
- * to other installations of the application that may use the same profile.
- */
-function getUpdateChannel() {
-  var channel = "default";
-  var prefName;
-  var prefValue;
-
-  var defaults = Services.prefs.getDefaultBranch(null);
-  try {
-    channel = defaults.getCharPref("app.update.channel");
-  } catch (e) {
-    // use default when pref not found
-  }
-
-  try {
-    var partners = Services.prefs.getChildList("app.partner.");
-    if (partners.length) {
-      channel += "-cck";
-      partners.sort();
-
-      for each (prefName in partners) {
-        prefValue = Services.prefs.getCharPref(prefName);
-        channel += "-" + prefValue;
+ * Read current process I/O counters.
+ */            
+let processInfo = {
+  _initialized: false,
+  _IO_COUNTERS: null,
+  _kernel32: null,
+  _GetProcessIoCounters: null,
+  _GetCurrentProcess: null,
+  getCounters: function() {
+    let isWindows = ("@mozilla.org/windows-registry-key;1" in Components.classes);
+    if (isWindows)
+      return this.getCounters_Windows();
+    return null;
+  },
+  getCounters_Windows: function() {
+    if (!this._initialized){
+      this._IO_COUNTERS = new ctypes.StructType("IO_COUNTERS", [
+        {'readOps': ctypes.unsigned_long_long},
+        {'writeOps': ctypes.unsigned_long_long},
+        {'otherOps': ctypes.unsigned_long_long},
+        {'readBytes': ctypes.unsigned_long_long},
+        {'writeBytes': ctypes.unsigned_long_long},
+        {'otherBytes': ctypes.unsigned_long_long} ]);
+      try {
+        this._kernel32 = ctypes.open("Kernel32.dll");
+        this._GetProcessIoCounters = this._kernel32.declare("GetProcessIoCounters", 
+          ctypes.winapi_abi,
+          ctypes.bool, // return
+          ctypes.voidptr_t, // hProcess
+          this._IO_COUNTERS.ptr); // lpIoCounters
+        this._GetCurrentProcess = this._kernel32.declare("GetCurrentProcess", 
+          ctypes.winapi_abi,
+          ctypes.voidptr_t); // return
+        this._initialized = true;
+      } catch (err) { 
+        return null; 
       }
     }
+    let io = new this._IO_COUNTERS();
+    if(!this._GetProcessIoCounters(this._GetCurrentProcess(), io.address()))
+      return null;
+    return [parseInt(io.readBytes), parseInt(io.writeBytes)];  
   }
-  catch (e) {
-    Cu.reportError(e);
-  }
-
-  return channel;
-}
+};
 
 function TelemetryPing() {}
 
@@ -182,12 +205,21 @@ TelemetryPing.prototype = {
   // duplicate submissions.
   _uuid: generateUUID(),
   // Regex that matches histograms we care about during startup.
+  // Keep this in sync with gen-histogram-bucket-ranges.py.
   _startupHistogramRegex: /SQLITE|HTTP|SPDY|CACHE|DNS/,
   _slowSQLStartup: {},
   _prevSession: null,
   _hasWindowRestoredObserver: false,
+  _hasXulWindowVisibleObserver: false,
   _pendingPings: [],
   _doLoadSaveNotifications: false,
+  _startupIO : {},
+  _hashID: Ci.nsICryptoHash.SHA256,
+  // The number of outstanding saved pings that we have issued loading
+  // requests for.
+  _pingsLoaded: 0,
+  // The number of those requests that have actually completed.
+  _pingLoadsCompleted: 0,
 
   /**
    * When reflecting a histogram into JS, Telemetry hands us an object
@@ -298,7 +330,7 @@ TelemetryPing.prototype = {
       appVersion: ai.version,
       appName: ai.name,
       appBuildID: ai.appBuildID,
-      appUpdateChannel: getUpdateChannel(),
+      appUpdateChannel: UpdateChannel.get(),
       platformBuildID: ai.platformBuildID,
       locale: getLocale()
     };
@@ -348,9 +380,11 @@ TelemetryPing.prototype = {
       }
     }
 
+#ifndef MOZ_WIDGET_GONK
     let theme = LightweightThemeManager.currentTheme;
     if (theme)
       ret.persona = theme.id;
+#endif
 
     if (this._addons)
       ret.addons = this._addons;
@@ -452,8 +486,7 @@ TelemetryPing.prototype = {
         Telemetry.histogramFrom("STARTUP_" + name, name);
       }
     }
-    // Bug 777220: Temporarily turn off slowSQL reporting
-    this._slowSQLStartup = {mainThread:{}, otherThreads:{}};
+    this._slowSQLStartup = Telemetry.slowSQL;
   },
 
   getCurrentSessionPayloadAndSlug: function getCurrentSessionPayloadAndSlug(reason) {
@@ -463,8 +496,7 @@ TelemetryPing.prototype = {
       ver: PAYLOAD_VERSION,
       simpleMeasurements: getSimpleMeasurements(),
       histograms: this.getHistograms(Telemetry.histogramSnapshots),
-      // Bug 777220: Temporarily turn off slowSQL reporting
-      slowSQL: {mainThread:{}, otherThreads:{}},
+      slowSQL: Telemetry.slowSQL,
       chromeHangs: Telemetry.chromeHangs,
       addonHistograms: this.getAddonHistograms()
     };
@@ -473,6 +505,9 @@ TelemetryPing.prototype = {
 	|| Object.keys(this._slowSQLStartup.otherThreads).length) {
       payloadObj.slowSQLStartup = this._slowSQLStartup;
     }
+    
+    for (let ioCounter in this._startupIO)
+      payloadObj.simpleMeasurements[ioCounter] = this._startupIO[ioCounter];
 
     let slug = (isTestPing ? reason : this._uuid);
     payloadObj.info = this.getMetadata(reason);
@@ -483,7 +518,7 @@ TelemetryPing.prototype = {
     function payloadIter() {
       yield this.getCurrentSessionPayloadAndSlug(reason);
 
-      if (this._pendingPings.length > 0) {
+      while (this._pendingPings.length > 0) {
         let data = this._pendingPings.pop();
         // Send persisted pings to the test URL too.
         if (reason == "test-ping") {
@@ -497,6 +532,17 @@ TelemetryPing.prototype = {
     return { __iterator__: payloadIterWithThis };
   },
 
+  hashString: function hashString(s) {
+    let digest = Cc["@mozilla.org/security/hash;1"]
+                 .createInstance(Ci.nsICryptoHash);
+    digest.init(this._hashID);
+    let stream = Cc["@mozilla.org/io/string-input-stream;1"]
+                 .createInstance(Ci.nsIStringInputStream);
+    stream.data = s;
+    digest.updateFromStream(stream, stream.available());
+    return digest.finish(/*base64encode=*/true);
+  },
+
   /**
    * Send data to the server. Record success/send-time in histograms
    */
@@ -540,12 +586,28 @@ TelemetryPing.prototype = {
       // Notify that testing is complete, even if we didn't send everything.
       finishPings(reason);
     }
-    this.doPing(server, data.slug, data.payload,
+    this.doPing(server, data,
                 onSuccess.bind(this), onError.bind(this));
   },
 
-  doPing: function doPing(server, slug, payload, onSuccess, onError) {
-    let submitPath = "/submit/telemetry/" + slug;
+  finishPingRequest: function finishPingRequest(success, startTime, ping) {
+    let hping = Telemetry.getHistogramById("TELEMETRY_PING");
+    let hsuccess = Telemetry.getHistogramById("TELEMETRY_SUCCESS");
+
+    hsuccess.add(success);
+    hping.add(new Date() - startTime);
+
+    if (success) {
+      let file = this.saveFileForPing(ping);
+      try {
+        file.remove(true);
+      } catch(e) {
+      }
+    }
+  },
+
+  doPing: function doPing(server, ping, onSuccess, onError) {
+    let submitPath = "/submit/telemetry/" + ping.slug;
     let url = server + submitPath;
     let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
                   .createInstance(Ci.nsIXMLHttpRequest);
@@ -555,30 +617,20 @@ TelemetryPing.prototype = {
     request.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
 
     let startTime = new Date();
-    let file = this.savedHistogramsFile();
-
-    function finishRequest(success) {
-      let hping = Telemetry.getHistogramById("TELEMETRY_PING");
-      let hsuccess = Telemetry.getHistogramById("TELEMETRY_SUCCESS");
-
-      hsuccess.add(success);
-      hping.add(new Date() - startTime);
-
-      if (success && file.exists()) {
-        file.remove(true);
-      }
-    }
 
     function handler(success, callback) {
-      return function(event) { finishRequest(success); callback() };
+      return function(event) {
+        this.finishPingRequest(success, startTime, ping);
+        callback();
+      };
     }
-    request.addEventListener("error", handler(false, onError), false);
-    request.addEventListener("load", handler(true, onSuccess), false);
+    request.addEventListener("error", handler(false, onError).bind(this), false);
+    request.addEventListener("load", handler(true, onSuccess).bind(this), false);
 
     request.setRequestHeader("Content-Encoding", "gzip");
     let payloadStream = Cc["@mozilla.org/io/string-input-stream;1"]
                         .createInstance(Ci.nsIStringInputStream);
-    payloadStream.data = this.gzipCompressString(payload);
+    payloadStream.data = this.gzipCompressString(ping.payload);
     request.send(payloadStream);
   },
 
@@ -624,14 +676,6 @@ TelemetryPing.prototype = {
     }
   },
 
-  savedHistogramsFile: function savedHistogramsFile() {
-    let profileDirectory = Services.dirsvc.get("ProfD", Ci.nsILocalFile);
-    let profileFile = profileDirectory.clone();
-    let name = "savedTelemetry.dat.json";
-    profileFile.append(name);
-    return profileFile;
-  },
-
   /**
    * Initializes telemetry within a timer. If there is no PREF_SERVER set, don't turn on telemetry.
    */
@@ -653,7 +697,9 @@ TelemetryPing.prototype = {
     Services.obs.addObserver(this, "profile-before-change", false);
     Services.obs.addObserver(this, "sessionstore-windows-restored", false);
     Services.obs.addObserver(this, "quit-application-granted", false);
+    Services.obs.addObserver(this, "xul-window-visible", false);
     this._hasWindowRestoredObserver = true;
+    this._hasXulWindowVisibleObserver = true;
 
     // Delay full telemetry initialization to give the browser time to
     // run various late initializers. Otherwise our gathered memory
@@ -667,29 +713,60 @@ TelemetryPing.prototype = {
     }
     this._timer.initWithCallback(timerCallback.bind(this), TELEMETRY_DELAY,
                                  Ci.nsITimer.TYPE_ONE_SHOT);
-    this.loadHistograms(this.savedHistogramsFile(), false);
+    this.loadSavedPings(false);
   },
 
-  addToPendingPings: function addToPendingPings(stream) {
+  verifyPingChecksum: function verifyPingChecksum(ping) {
+    /* A ping from the current session won't have a checksum.  */
+    if (!ping.checksum) {
+      return true;
+    }
+
+    let checksumNow = this.hashString(ping.payload);
+    return ping.checksum == checksumNow;
+  },
+
+  addToPendingPings: function addToPendingPings(file, stream) {
+    let success = false;
+
     try {
       let string = NetUtil.readInputStreamToString(stream, stream.available(), { charset: "UTF-8" });
       stream.close();
-      let data = JSON.parse(string);
-      this._pendingPings = this._pendingPings.concat(data);
-      if (this._doLoadSaveNotifications) {
+      let ping = JSON.parse(string);
+      this._pingLoadsCompleted++;
+
+      if (this.verifyPingChecksum(ping)) {
+        this._pendingPings.push(ping);
+      }
+
+      if (this._doLoadSaveNotifications &&
+          this._pingLoadsCompleted == this._pingsLoaded) {
         Services.obs.notifyObservers(null, "telemetry-test-load-complete", null);
       }
+      success = true;
     } catch (e) {
       // An error reading the file, or an error parsing the contents.
+      stream.close();           // close is idempotent.
+      file.remove(true);
     }
+    let success_histogram = Telemetry.getHistogramById("READ_SAVED_PING_SUCCESS");
+    success_histogram.add(success);
   },
 
   loadHistograms: function loadHistograms(file, sync) {
+    let now = new Date();
+    if (now - file.lastModifiedTime > MAX_PING_FILE_AGE) {
+      // We haven't had much luck in sending this file; delete it.
+      file.remove(true);
+      return;
+    }
+
+    this._pingsLoaded++;
     if (sync) {
       let stream = Cc["@mozilla.org/network/file-input-stream;1"]
                    .createInstance(Ci.nsIFileInputStream);
       stream.init(file, -1, -1, 0);
-      this.addToPendingPings(stream);
+      this.addToPendingPings(file, stream);
     } else {
       let channel = NetUtil.newChannel(file);
       channel.contentType = "application/json"
@@ -698,37 +775,64 @@ TelemetryPing.prototype = {
         if (!Components.isSuccessCode(result)) {
           return;
         }
-        this.addToPendingPings(stream);
+        this.addToPendingPings(file, stream);
       }).bind(this));
     }
   },
 
+  loadSavedPings: function loadSavedPings(sync) {
+    let directory = this.ensurePingDirectory();
+    let entries = directory.directoryEntries
+                           .QueryInterface(Ci.nsIDirectoryEnumerator);
+    this._pingsLoaded = 0;
+    this._pingLoadsCompleted = 0;
+    try {
+      while (entries.hasMoreElements()) {
+        this.loadHistograms(entries.nextFile, sync);
+      }
+    }
+    finally {
+      entries.close();
+    }
+  },
+
   finishTelemetrySave: function finishTelemetrySave(ok, stream) {
-    stream.QueryInterface(Ci.nsISafeOutputStream).finish();
     stream.close();
     if (this._doLoadSaveNotifications && ok) {
       Services.obs.notifyObservers(null, "telemetry-test-save-complete", null);
     }
   },
 
-  saveHistograms: function saveHistograms(file, sync) {
-    let pings = [this.getCurrentSessionPayloadAndSlug("saved-session")];
-    let pingString = JSON.stringify(pings);
+  savePingToFile: function savePingToFile(ping, file, sync, overwrite) {
+    let pingString = JSON.stringify(ping);
 
     let converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
                     .createInstance(Ci.nsIScriptableUnicodeConverter);
     converter.charset = "UTF-8";
 
-    let ostream = Cc["@mozilla.org/network/safe-file-output-stream;1"]
+    let ostream = Cc["@mozilla.org/network/file-output-stream;1"]
                   .createInstance(Ci.nsIFileOutputStream);
-    ostream.init(file, PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE,
-                 RW_OWNER, ostream.DEFER_OPEN);
+    let initFlags = PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE;
+    if (!overwrite) {
+      initFlags |= PR_EXCL;
+    }
+    try {
+      ostream.init(file, initFlags, RW_OWNER, 0);
+    } catch (e) {
+      // Probably due to PR_EXCL.
+      return;
+    }
 
     if (sync) {
       let utf8String = converter.ConvertFromUnicode(pingString);
       utf8String += converter.Finish();
-      let amount = ostream.write(utf8String, utf8String.length);
-      this.finishTelemetrySave(amount == utf8String.length, ostream);
+      let success = false;
+      try {
+        let amount = ostream.write(utf8String, utf8String.length);
+        success = amount == utf8String.length;
+      } catch (e) {
+      }
+      this.finishTelemetrySave(success, ostream);
     } else {
       let istream = converter.convertToInputStream(pingString)
       let self = this;
@@ -752,6 +856,45 @@ TelemetryPing.prototype = {
     return null;
   },
 
+  ensurePingDirectory: function ensurePingDirectory() {
+    let profileDirectory = Services.dirsvc.get("ProfD", Ci.nsILocalFile);
+    let directory = profileDirectory.clone();
+    directory.append("saved-telemetry-pings");
+    try {
+      directory.create(Ci.nsIFile.DIRECTORY_TYPE, RWX_OWNER);
+    } catch (e) {
+      // Already exists, just ignore this.
+    }
+    return directory;
+  },
+
+  saveFileForPing: function saveFileForPing(ping) {
+    if (!('checksum' in ping)) {
+      ping.checksum = this.hashString(ping.payload);
+    }
+    let file = this.ensurePingDirectory();
+    file.append(ping.slug);
+    return file;
+  },
+
+  savePing: function savePing(ping, overwrite) {
+    this.savePingToFile(ping, this.saveFileForPing(ping), true, overwrite);
+  },
+
+  savePendingPings: function savePendingPings() {
+    let sessionPing = this.getCurrentSessionPayloadAndSlug("saved-session");
+    this.savePing(sessionPing, true);
+    this._pendingPings.forEach(function sppcb(e, i, a) {
+                                 this.savePing(e, false);
+                               }, this);
+    this._pendingPings = [];
+  },
+
+  saveHistograms: function saveHistograms(file, sync) {
+    this.savePingToFile(this.getCurrentSessionPayloadAndSlug("saved-session"),
+                        file, sync, true);
+  },
+
   /** 
    * Remove observers to avoid leaks
    */
@@ -761,6 +904,10 @@ TelemetryPing.prototype = {
       Services.obs.removeObserver(this, "sessionstore-windows-restored");
       this._hasWindowRestoredObserver = false;
     }
+    if (this._hasXulWindowVisibleObserver) {
+      Services.obs.removeObserver(this, "xul-window-visible");
+      this._hasXulWindowVisibleObserver = false;
+    }
     Services.obs.removeObserver(this, "profile-before-change");
     Services.obs.removeObserver(this, "private-browsing");
     Services.obs.removeObserver(this, "quit-application-granted");
@@ -799,6 +946,15 @@ TelemetryPing.prototype = {
         this.attachObservers()
       }
       break;
+    case "xul-window-visible":
+      Services.obs.removeObserver(this, "xul-window-visible");
+      this._hasXulWindowVisibleObserver = false;   
+      var counters = processInfo.getCounters();
+      if (counters) {
+        [this._startupIO.startupWindowVisibleReadBytes, 
+          this._startupIO.startupWindowVisibleWriteBytes] = counters;
+      }
+      break;
     case "sessionstore-windows-restored":
       Services.obs.removeObserver(this, "sessionstore-windows-restored");
       this._hasWindowRestoredObserver = false;
@@ -807,6 +963,11 @@ TelemetryPing.prototype = {
       gWasDebuggerAttached = debugService.isDebuggerAttached;
       // fall through
     case "test-gather-startup":
+      var counters = processInfo.getCounters();
+      if (counters) {  
+        [this._startupIO.startupSessionRestoreReadBytes, 
+          this._startupIO.startupSessionRestoreWriteBytes] = counters;
+      }
       this.gatherStartupInformation();
       break;
     case "idle-daily":
@@ -831,6 +992,8 @@ TelemetryPing.prototype = {
       this.saveHistograms(aSubject.QueryInterface(Ci.nsIFile), aData != "async");
       break;
     case "test-load-histograms":
+      this._pingsLoaded = 0;
+      this._pingLoadsCompleted = 0;
       this.loadHistograms(aSubject.QueryInterface(Ci.nsIFile), aData != "async");
       break;
     case "test-enable-load-save-notifications":
@@ -852,7 +1015,9 @@ TelemetryPing.prototype = {
       }
       break;
     case "quit-application-granted":
-      this.saveHistograms(this.savedHistogramsFile(), true);
+      if (Telemetry.canSend) {
+        this.savePendingPings();
+      }
       break;
     }
   },
@@ -861,4 +1026,4 @@ TelemetryPing.prototype = {
   QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),
 };
 
-let NSGetFactory = XPCOMUtils.generateNSGetFactory([TelemetryPing]);
+this.NSGetFactory = XPCOMUtils.generateNSGetFactory([TelemetryPing]);
